
/*
* paperlit.js v. 1.5 - libraries use to communicate between web pages and native client.
* Copyright (C) 2011 Paperlit Inc. All rights reserved.
*/

// if Paperlit name space do not exists,  create it and register to global window
if(typeof Paperlit == "undefined") {
    Paperlit = new Object();
    window.Paperlit = Paperlit;
}
else if(typeof Paperlit._bridgeInitialized !== "undefined") {
	// already initialized
	alert('already initialized');
}

Paperlit._bridgeInitialized = true;

if(typeof ppAppRootPath === 'undefined'){
    Paperlit.appRootPath = '.';
    // check where it is located paperlit.js
    var scripts = document.getElementsByTagName('script');
    var script;
    for(var i=0, l=scripts.length; i<l; i++){
        script = scripts[i];
        if(script.src.indexOf('lib/paperlit.js') > -1){
            Paperlit.appRootPath = script.src.substring(0, script.src.indexOf('lib/paperlit.js'));
            if(Paperlit.appRootPath[Paperlit.appRootPath.length - 1] === '/'){
                // remove last slash
                Paperlit.appRootPath = Paperlit.appRootPath.substring(0, Paperlit.appRootPath.length - 1);
            }
            break;
        }
    }
}
else{
    Paperlit.appRootPath = ppAppRootPath;
}

/**
 * Default log is noop
 */
if (typeof Paperlit.log == "undefined"){
    Paperlit.log = {};
}

if (typeof Paperlit.log.info == "undefined"){
    Paperlit.log.info = function(msg) {
        //setTimeout(function(){alert('info ' + msg)}, 1);
        //console.log(msg);
    };
}
if (typeof Paperlit.log.error == "undefined"){
    Paperlit.log.error = function(msg) {
        //setTimeout(function(){alert('error ' + msg)}, 1);
        //console.error(msg);
    };
}

if (typeof Paperlit.log.debug == "undefined"){
    Paperlit.log.debug = function(msg) {
    };
}

// define some utils that are used here
if (typeof Paperlit.utils == "undefined") {
    // create utils namespace only if it does not exist
    Paperlit.utils = {};
}

(function(){
    var log = document.createElement('div');
    log.setAttribute("id","pp-log");
    //log.setAttribute("shown", "true");
    log.style.backgroundColor = 'white';
    log.style.color = 'black';
    log.style.right = 0;
    log.style.bottom = 0;
    log.style.position = 'fixed';
    log.style.width = '100%';
    log.style.height = '30%';
    log.style['z-index'] = '99999';
    log.style['overflow-y'] = 'scroll';
    log.style['-webkit-overflow-scrolling'] = 'touch';
    log.style['display'] = 'none';

    var logButton = document.createElement('div');
    logButton.style['display'] = 'none';
    logButton.style.right = 0;
    logButton.style.bottom = 0;
    logButton.style.position = 'fixed';
    logButton.style.width = '100px';
    logButton.style.height = '100px';
    logButton.style['z-index'] = '999999';
    logButton.style.backgroundColor = 'rgba(255,0,0,0.5)';
    document.documentElement.appendChild(logButton);
    document.documentElement.appendChild(log);
    function addZero(x,n)
    {
        if (x.toString().length<n) 
        {
            x="0" + x;
        }
        return x;
    }

    function getDate()
    {
        var d=new Date();
        var h=addZero(d.getHours(),2);
        var m=addZero(d.getMinutes(),2);
        var s=addZero(d.getSeconds(),2);
        var ms=addZero(d.getMilliseconds(),3);
        return h + ":" + m + ":" + s + "." + ms;
    }

    Paperlit.log.info = function(msg) {
        var message =  document.createElement('p');
        message.style.paddingBottom = '10px';
        message.textContent  = getDate() + ' - ' + msg;
        log.appendChild(message);
        console.log('INFO: ' + msg);
  
    };
    Paperlit.log.error = function(msg) {
        var message =  document.createElement('p');
        message.style.paddingBottom = '10px';
        message.style.color = 'red';
        message.textContent = getDate() + ' - ERROR: ' + msg;
        log.appendChild(message);
        console.log('ERROR: ' + msg);

    };

    logButton.addEventListener('touchstart', function(e){
		if(log.getAttribute("shown")){
            //hide
            log.style.display = 'none';
            log.removeAttribute("shown");
            // clear
            log.innerHTML = '';
        }
        else{
            log.style.display = 'block';
            log.setAttribute("shown", "true");
        }
        e.stopPropagation();
        e.preventDefault();
    });
    Paperlit._logDiv = log;
    Paperlit._logButton = logButton;
})();


Paperlit.utils.getUrlParameter = function(name)
{
    if (window.location.hash != null)
    {
        var results = new RegExp('[\\?&#]+' + name + '=([^&#]*)').exec(window.location.hash);
        
        if (results != null && results.length > 1)
        {
        	return decodeURIComponent(results[1]);
        }
    }
    
    var results = new RegExp('[\\?&#]' + name + '=([^&#]*)').exec(window.location.href);

    if (results != null && results.length > 1) 
    {
        // console.log("Paperlit.utils.getUrlParam - %s: %s", name, results[1]);

    	return decodeURIComponent(results[1]);
    }

    // console.log("Paperlit.utils.getUrlParam - %s not found in %s", name, window.location.href);

    return null;
};

Paperlit.utils.getParameterFromUrl =  function(url, name)
{
    var results = new RegExp('[\\?&#]' + name + '=([^&#]*)').exec(url);

    if (results != null && results.length > 1) 
    {
        // console.log("Paperlit.utils.getUrlParam - %s: %s", name, results[1]);

        return decodeURIComponent(results[1]);
    }

    // console.log("Paperlit.utils.getUrlParam - %s not found in %s", name, window.location.href);

    return null;
};


Paperlit.utils.getSetting = function(settings, name) {
    var value = null;
    try{
    	// look if there is a setting for the current locale
    	if (typeof Paperlit.utils.getLocale !== 'undefined' && typeof settings[name + '.' + Paperlit.utils.getLocale()] != 'undefined') {
    		value = settings[name + '.' + Paperlit.utils.getLocale()];
    	}
    	else if (typeof settings[name] != 'undefined') {
    		value = settings[name];
    	}
        else{
            return null;
        }
        
        if (value === true || value === false) {
    		return value;
    	} 
    	else if (value === 'true') {
    		value = true;
    	} 
    	else if (value === 'false') {
    		value = false;
    	}
        else if (value === ''){
            value = null;
        }
    	else if (isNaN(value) == false){
    		//convert to integer
    		value = parseInt(value, 10);
    	}
    }
    catch(e)
    {}
    
    return value;
};

Paperlit.utils.getDeviceSetting = function(settings, name, callback){
	Paperlit.log.info('Paperlit.utils.getDeviceSetting : required ' + name);
	Paperlit.getState(function(state){
		Paperlit.log.info('Paperlit.utils.getDeviceSetting : got state evaluating ' + name);
	    var value = Paperlit.utils.getDeviceSettingSync(settings, state, name);
	    callback(value);
	}, function(){
		callback(false)
	});
};

Paperlit.utils.getDeviceSettingSync = function(settings, state, name){
	Paperlit.log.info('Paperlit.utils.getDeviceSettingSync : required ' + name);
    var value = Paperlit.utils.getSetting(settings, name);
    var valueTablet = Paperlit.utils.getSetting(settings, name + '-tablet');
    var valuePhone = Paperlit.utils.getSetting(settings, name + '-phone');
    // check if we have a device specific url
    if(valueTablet != null && state.deviceClass == 'tablet'){
        value = valueTablet;
    }
    else if(valuePhone != null && state.deviceClass == 'phone'){
        value = valuePhone;
    }
    Paperlit.log.info('Paperlit.utils.getDeviceSettingSync : found value ' + value);
    return value;
};

Paperlit.utils.isVersionSupported = function(bundleVersion, minSupportedVersion){
    var bundleVersionNumbers = bundleVersion.split('.');
    var minSupportedVersionNumbers = minSupportedVersion.split('.');

    for (var i = 0; i < bundleVersionNumbers.length; ++i) {
        bundleVersionNumbers[i] = Number(bundleVersionNumbers[i]);
    }
    for (var i = 0; i < minSupportedVersionNumbers.length; ++i) {
        minSupportedVersionNumbers[i] = Number(minSupportedVersionNumbers[i]);
    }

    for (var i = 0; i < Math.min(minSupportedVersionNumbers.length, bundleVersionNumbers.length); i++){
        if (bundleVersionNumbers[i] < minSupportedVersionNumbers[i]){
            // not supported
            return false;
        }
        else if (bundleVersionNumbers[i] > minSupportedVersionNumbers[i]){
            // supported
            return true;
        }
    }

    // all numbers are equals

    if(minSupportedVersionNumbers.length >  bundleVersionNumbers.length){
        return false;
    }
                  
    return true;
};


Paperlit.utils.appendUrlParameter = function(url, parameter, value){
    var tempParameter;
    if(url.indexOf('?') == -1){
        tempParameter = '?';
    }
    else{
        tempParameter = '&';
    }
    tempParameter = tempParameter + parameter + '=' + encodeURIComponent(value);

    //check if there is an hash and preserve it
    if(url.indexOf('#') > -1){
        var indexOfHash = url.indexOf('#');
        url = [url.slice(0, indexOfHash), tempParameter, url.slice(indexOfHash)].join('');
    }
    else{
        url = url + tempParameter;
    }

    return url;
};

Paperlit.utils.getParameter = function(url, name){
    var results = new RegExp('[\\?&#]' + name + '=([^&#]*)').exec(url);

    if (results != null && results.length > 1) 
    {
        // console.log("Paperlit.utils.getUrlParam - %s: %s", name, results[1]);

        return decodeURIComponent(results[1]);
    }

    // console.log("Paperlit.utils.getUrlParam - %s not found in %s", name, window.location.href);

    return null;
}

Paperlit.isIOS =  (navigator.userAgent.match(/iPad|iPhone|iPod/i));  //&& !(navigator.userAgent.indexOf("Safari") > -1);
Paperlit.isAndroid = navigator.userAgent.toLowerCase().indexOf("android") > -1 ;
Paperlit.isKindleFire = false;
Paperlit.isKindleFireHD = false;

Paperlit.isBlackBerry10 = navigator.userAgent.toLowerCase().indexOf("bb10") > -1 && navigator.userAgent.toLowerCase().indexOf("BB Version") > -1;

if(Paperlit.isAndroid){
    // check if it is a kindle fire
    /*
        Kindle Fire – KFOT
        Kindle Fire HD – KFTT
        Kindle Fire HD 8.9″ WiFi – KFJWI
        Kindle Fire HD 8.9″ 4G – KFJWA
    */
    var ua = navigator.userAgent.toLowerCase();
    if(ua.indexOf("kfot") > -1 || ua.indexOf("kftt") > -1 || ua.indexOf("kfjwi") > -1 || ua.indexOf("kfjwa") > -1){
        Paperlit.isKindleFire = true;
        // check if not kindle fire basic
        if(ua.indexOf("kfot") == -1){
            Paperlit.isKindleFireHD = true;
        }
    }
}

Paperlit.isNative = false;
Paperlit.ready = false;
Paperlit.supportsNativeScrolling = false;

Paperlit.propertiesFileName = 'properties.pp5';
Paperlit.nativeBookmarksFileName = 'bookmarks.xml';

// lock properties write
Paperlit.lockWriteProperties = false;

Paperlit.clickEventName = 'click';     
// using Cordova or PhoneGap
Paperlit.isCordova = false;
Paperlit.isPhoneGap = false;
Paperlit.isGap = false;
Paperlit._deviceReadyExecuted = false;

// initializing phonegap globals that will be used also in android
Paperlit._anomFunkMapNextId = 0;
Paperlit._anomFunkMap = {};

Paperlit.alert = function(msg, callback){
    if(Paperlit.isIOS){ 
        // alert async to not freeze webview
        setTimeout(function(message, callback){
            confirm(message);
            if(callback){
                callback();
            }
        }, 1, msg, callback);
    }
    else{
        alert(msg);
        if(callback){
            callback();
        }
    }
};

/* creating bridge */
    
Paperlit.initAdobeBridge = function(){
    var overlay = {};
   
    overlay.goFullscreen = function(successCallback, errorCallback){
        try {
            errorCallback = typeof errorCallback != 'undefined' ? errorCallback : Paperlit.noop;
            Paperlit.log.info('Paperlit.goFullscreen');
            cordova.exec(successCallback, errorCallback, 'paperlit', 'goFullscreen', []);
        } catch(e) {
            Paperlit.log.error('exception calling PaperlitCommands.goFullscreen: '+ e.message);
            errorCallback();
        }
    };
    
    /* 
     * Respond to successCallback giving a dictionary with coordinates and size for snapshot 
     * E.g. {width: 0, height: 0, x: 0, y:0}
    */  
    overlay.prepareFullscreen = function(successCallback, errorCallback){
        try {
            errorCallback = typeof errorCallback != 'undefined' ? errorCallback : Paperlit.noop;
            Paperlit.log.info('Paperlit.prepareFullscreen');
            cordova.exec(successCallback, errorCallback, 'paperlit', 'prepareFullscreen', []);
        } catch(e) {
            Paperlit.log.error('exception calling PaperlitCommands.prepareFullscreen: '+ e.message);
            errorCallback();
        }
    };
    overlay.goInline = function(successCallback, errorCallback){
        try {
            Paperlit.log.info('Paperlit.goInline');
            errorCallback = typeof errorCallback != 'undefined' ? errorCallback : Paperlit.noop;
            cordova.exec(successCallback, errorCallback, 'paperlit', 'goInline', []);
        } catch(e) {
            Paperlit.log.error('exception calling PaperlitCommands.goInline: '+ e.message);
            errorCallback();
        }
    };
    
    /* 
     * Respond to successCallback giving a dictionary with coordinates and size for snapshot 
     * E.g. {width: 0, height: 0, x: 0, y:0}
    */  
    overlay.prepareInline = function(successCallback, errorCallback){
        try {
            Paperlit.log.info('Paperlit.prepareInline');
            errorCallback = typeof errorCallback != 'undefined' ? errorCallback : Paperlit.noop;
            cordova.exec(successCallback, errorCallback, 'paperlit', 'prepareInline', []);
        } catch(e) {
            Paperlit.log.error('exception calling PaperlitCommands.prepareInline: '+ e.message);
            errorCallback();
        }
    };
    
    // create adobe bridge container
    var ADOBEDMPNS = {
        Viewer : {
            overlay : overlay
        } 
    };
    
    //expose to window
    window.ADOBEDMPNS = ADOBEDMPNS;   
};

/**
 ** Fix GetFunctionName in phonegap 0.9.4 considering a function call pattern with two callback functions (success, fail)
 ** Using the original function there is a big memory leak issue because the uncalled function is never deallocated
 ** Should be fixed in phonegap 0.9.5
 ** It also used by gap protocol implemented in Android for share and get product price
 **/
Paperlit.getFunctionsName = function(success, error) {
    try {
        if (success == null)
            success = Paperlit.noop;
        if (error == null)
            error = Paperlit.noop;

        //get two uid
        var funkIdSuccess = "f" + Paperlit._anomFunkMapNextId++;
        var funkIdError = "f" + Paperlit._anomFunkMapNextId++;

        var funkSuccess = function () {
            //execute
            success.apply(this, arguments);
            //remove myself
            Paperlit._anomFunkMap[funkIdSuccess] = null;
            delete Paperlit._anomFunkMap[funkIdSuccess];
            //remove my brother
            Paperlit._anomFunkMap[funkIdError] = null;
            delete Paperlit._anomFunkMap[funkIdError];
        };
        var funkError = function () {
            //execute
            error.apply(this, arguments);
            //remove myself
            Paperlit._anomFunkMap[funkIdError] = null;
            delete Paperlit._anomFunkMap[funkIdError];
            //remove my brother
            Paperlit._anomFunkMap[funkIdSuccess] = null;
            delete Paperlit._anomFunkMap[funkIdSuccess];
        };
        Paperlit._anomFunkMap[funkIdSuccess] = funkSuccess;
        Paperlit._anomFunkMap[funkIdError] = funkError;

        return {successFuncName : "Paperlit._anomFunkMap." + funkIdSuccess, errorFuncName : "Paperlit._anomFunkMap." + funkIdError};
    } catch(e) {
        Paperlit.log.error('Paperlit.getFunctionsName failed: '+e.message);
    }
};

/**
 ** Identify the platform and setup the communication interface
 **/
Paperlit.onDeviceReady = function () {
    try {
        if(Paperlit._deviceReadyExecuted){
            return;
        }
        Paperlit._deviceReadyExecuted = true;
        Paperlit.log.info('Paperlit.onDeviceReady');
        // declare that the device native interface is ready
        Paperlit.isDeviceReady = true;
        // check again which interface we have
        Paperlit.isPhoneGap = typeof PhoneGap !== 'undefined';
        Paperlit.isCordova = typeof cordova !== 'undefined';
        //trigger ppDeviceReady event
        var evObj = document.createEvent('Events');
        evObj.initEvent('ppDeviceReady', false, true);
        document.dispatchEvent(evObj);
        // remove device ready listener
        document.removeEventListener("deviceready", Paperlit.onDeviceReady);
        // remove event listener for focus event as native client manage this
        window.removeEventListener('focus', Paperlit.resumeActivity, false);
        // declare that we are running in a native client
        Paperlit.isNative = true;
        Paperlit.supportsNativeScrolling = false;// typeof(document.body.style["-webkit-overflow-scrolling"]) !== "undefined"; //(navigator.userAgent.match(/OS 5_\d like Mac OS X/i));
        // add system class to body
        try{
            if (typeof $ !== 'undefined'){
                // set platform class into body
                var platform = null;
                if (Paperlit.isIOS){
                    $('body').addClass('system-ios');
                }
                else if(Paperlit.isAndroid){
                    $('body').addClass('system-android');
                    if(Paperlit.isKindleFireHD){
                        $('body').addClass('device-kindlefirehd');
                    }
                    else if(Paperlit.isKindleFire){
                        $('body').addClass('device-kindlefire');
                    }
                }
            }
        }
        catch(e){}
        
        Paperlit.getDocumentPath = function(){
            return navigator.fileMgr.getRootPaths()[0] + '/';
        };
        
        
        
		
        if(Paperlit.isCordova){
            PhoneGap = {};
            if(Paperlit.isIOS){
                //writing adapters for backward compatibility on Cordova IOS

                // getFunctionsName will return exactly the same functions it receives has Cordova is able to deallocate not call callbacks
                Paperlit.getFunctionsName = function(success, error) {
                    return {successFuncName : success, errorFuncName : error};
                }
                // PhoneGap adapter for IOS
                PhoneGap.exec = function(){
                    try {
                        // e.g., PhoneGap.exec("PaperlitCommands.getCachedUrl", url, maxAgeInSeconds, functionsName.successFuncName, functionsName.errorFuncName);
                        // first parameter is old command name, last two parameters are callbacks.
                        
                        if (arguments.length < 3){
                            throw "missing arguments";
                        }
                        var args = Array.prototype.slice.call(arguments);
                        // last two parameters are callbacks
                        var errorCallback = args.pop();
                        var successCallback = args.pop();
                        // first parameter is old command name e.g., PaperlitCommands.getProductPrice 
                        var commandNameParts = args.shift().split('.');
                        var service = commandNameParts[0];
                        var action = commandNameParts[1];
                        // execute in cordova
                        cordova.exec(successCallback, errorCallback, 'paperlit', action, args);
                        
                    } catch(e) {
                        Paperlit.log.error('exception calling phonegap IOS adapter for Cordova ' + e.message);
                    }
                };
            }
            else if(Paperlit.isAndroid){
                // adapters for backward compatibility on Cordova Android
                PhoneGap.addConstructor = function(func) {
                    try {
                        func();
                        Paperlit.log.info("Phonegap fake constructor executed");
                    } catch(e) {
                        Paperlit.log.error("Failed to run constructor: " + e.message);
                    }
                };
                
                PhoneGap.addPlugin = function(name, obj) {
                    window.plugins = {};
                    window.plugins[name] = obj;
                    Paperlit.log.info("Phonegap fake plugin " + name + " registered.");
                }

                PhoneGap.exec = function(successCallback, errorCallback, service, action, parameters){
                    //e.g., PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'getCachedUrl', [url, maxAgeInSeconds]);
                    // execute in cordova
                    Paperlit.log.info('cordova.exec ' + service + ' / ' + action + ' / ' + JSON.stringify(parameters));   
                    cordova.exec(successCallback, errorCallback, service, action, parameters); 
                }
            }

            //new api in cordova

            /*Show newsstand hiding splashscreen*/
            Paperlit.showNewsstandView = function(successCallback){
                cordova.exec(successCallback, Paperlit.noop, 'paperlit', 'showNewsstandView', []);    
            }

            /* Require a persistent filesystem and store the reference in the Paperlit env */
            Paperlit.requireFileSystem = function(successCallback, errorCallback){
                if(typeof Paperlit.fileSystem === 'undefined'){
                    Paperlit.log.info('Paperlit.requireFileSystem: require new fileSystem');
                    window.requestFileSystem(LocalFileSystem.PERSISTENT, 0,  function(fileSystem){
                        Paperlit.log.info('Paperlit.requireFileSystem: got new fileSystem');
                        Paperlit.fileSystem = fileSystem;
                        successCallback(Paperlit.fileSystem);
                    }, errorCallback);
                }
                else{
                    Paperlit.log.info('Paperlit.requireFileSystem: returning saved filesystem');
                    successCallback(Paperlit.fileSystem);
                }
            };
           
            Paperlit.getFileEntry = function(fileName, successCallback, errorCallback, flags){
                Paperlit.log.info('Paperlit.getFileEntry ' + fileName);
                Paperlit.requireFileSystem(function(fileSystem){
                    Paperlit.log.info('Paperlit.getFileEntry: got file system for ' + fileName);
                    fileSystem.root.getFile(fileName, flags, function(fileEntry){
                        Paperlit.log.info('Paperlit.getFileEntry: got file entry for ' + fileName);
                        successCallback(fileEntry);
                    }, errorCallback)
                }, errorCallback);
            }

            Paperlit.getFile = function(fileName, successCallback, errorCallback, flags){
                Paperlit.log.info('Paperlit.getFile ' + fileName);
                Paperlit.getFileEntry(fileName, function(fileEntry){
                    Paperlit.log.info('Paperlit.getFile: got file entry for ' + fileName);
                    fileEntry.file(function(file){
                        Paperlit.log.info('Paperlit.getFile: got file for ' + fileName);
                        successCallback(file);
                    }, errorCallback)
                }, errorCallback, flags);
            }

            /* Read a text file and returns the content */
            Paperlit.readFile = function(fileName, successCallback, errorCallback) {
                if(Paperlit.lockReadWriteFile){
                    Paperlit.log.info('Paperlit.readFile: lockReadWriteFile, waiting ' + fileName);
                    // postpone call
                    setTimeout(function(){
                        Paperlit.readFile(fileName, successCallback, errorCallback);
                    }, 10);
                    return;
                }
                // avoid concurrent read/write
                Paperlit.lockReadWriteFile = true;
                // request file entry
                Paperlit.log.info('Paperlit.readFile: ' + fileName);
                Paperlit._getReadFile(fileName, successCallback, errorCallback)();
            };

            Paperlit._getReadFile = function(fileName, successCallback, errorCallback){
                // create new operation
                var operation = {timeout: null, isValid: true, fileName: fileName, successCallback: successCallback, errorCallback: errorCallback};
                return function(){
                    Paperlit.log.info('Paperlit._getReadFile: ' + operation.fileName);
                    // set a timeout for this operation
                    operation.timeout = setTimeout(function(operation){
                        if(operation.isValid){
                            Paperlit.log.error('Paperlit._getReadFile: timeout for file ' + operation.fileName);
                            // invalidate this operation
                            operation.isValid = false;
                            // unlock semaphore for read/write files
                            Paperlit.lockReadWriteFile = false;
                            // create a new operation and start it async to avoid blocking again on the same file
                            setTimeout(Paperlit._getReadFile(operation.fileName, operation.successCallback, operation.errorCallback), 500);
                        }
                    }, 10000, operation);
                    
                    Paperlit.getFile(operation.fileName, function(file){
                        // operation has been invalidated for timeout
                        if(!operation.isValid){
                            return;
                        }
                        var reader = new FileReader();
                        reader.onload = function(event) {
                            // operation has been invalidated for timeout
                            if(!operation.isValid){
                                return;
                            }
                            Paperlit.log.info('Paperlit._getReadFile: loaded ' + operation.fileName);
                            Paperlit.lockReadWriteFile = false;
                            // remove timeout
                            if(operation.timeout != null){
                                clearTimeout(operation.timeout);
                                operation.timeout = null;
                            }

                            if (typeof event.target.result == 'undefined' || event.target.result == null || event.target.result == ""){
                                operation.errorCallback();
                                return;
                            }
                            try{
                                setTimeout(function(){
                                    operation.successCallback(event.target.result);
                                }, 1);
                            }catch(e){
                                if(operation.errorCallback)
                                    operation.errorCallback();
                            }
                        };
                        reader.onerror = function(){
                            // operation has been invalidated for timeout
                            if(!operation.isValid){
                                return;
                            }
                            Paperlit.log.error('Paperlit._getReadFile: error for ' + operation.fileName);
                            
                            Paperlit.lockReadWriteFile = false;
                            // remove timeout
                            if(operation.timeout != null){
                                clearTimeout(operation.timeout);
                                operation.timeout = null;
                            }
                            operation.errorCallback();
                        };
                        // start reading file
                        reader.readAsText(file);
                    }, function(){
                        // operation has been invalidated for timeout
                        if(!operation.isValid){
                            return;
                        }
                        Paperlit.log.error('Paperlit._getReadFile: error getting file ' + operation.fileName);
                        // error getting file
                        Paperlit.lockReadWriteFile = false;
                        // remove timeout
                        if(operation.timeout != null){
                            clearTimeout(operation.timeout);
                            operation.timeout = null;
                        }
                        operation.errorCallback();
                    });
                };
            };
            
            /* Write a text file on device */
            Paperlit.writeFile = function(text, successCallback, errorCallback, fileName){
                if(Paperlit.lockReadWriteFile){
                    Paperlit.log.info('Paperlit.writeFile: lockReadWriteFile, waiting ' + fileName);
                    // postpone call
                    setTimeout(function(){
                        Paperlit.writeFile(text, successCallback, errorCallback, fileName);
                    }, 10);
                    return;
                }
                // avoid concurrent read/write
                Paperlit.lockReadWriteFile = true;

                errorCallback = typeof errorCallback != 'undefined' ? errorCallback : Paperlit.noop;
                Paperlit._getWriteFile(text, fileName, successCallback, errorCallback)();
            };


            Paperlit._getWriteFile = function(text, fileName, successCallback, errorCallback){ 
                // create new operation
                var operation = {timeout: null, isValid: true, fileName: fileName, text: text, successCallback: successCallback, errorCallback: errorCallback};
                
                return function(){
                    Paperlit.log.info('Paperlit._getWriteFile: ' + operation.fileName);
                    var proxyedErrorCallback = function(){
                        // operation has been invalidated for timeout
                        if(!operation.isValid){
                            return;
                        }
                        Paperlit.log.error('Paperlit._getWriteFile: error for ' + operation.fileName);
                        Paperlit.lockReadWriteFile = false;
                        // remove timeout
                        if(operation.timeout != null){
                            clearTimeout(operation.timeout);
                            operation.timeout = null;
                        }
                        operation.errorCallback();
                    }

                    // set a timeout for this operation
                    operation.timeout = setTimeout(function(operation){
                        if(operation.isValid){
                            Paperlit.log.error('Paperlit._getWriteFile: timeout for file ' + operation.fileName);
                            // invalidate this operation
                            operation.isValid = false;
                            // unlock semaphore for read/write files
                            Paperlit.lockReadWriteFile = false;
                            // create a new operation and start it async to avoid blocking again on the same file
                            setTimeout(Paperlit._getWriteFile(operation.text, operation.fileName, operation.successCallback, operation.errorCallback), 500);
                        }
                    }, 10000, operation);

                    Paperlit.log.info('Paperlit._getWriteFile: get file entry for ' + operation.fileName);
                    Paperlit.getFileEntry(operation.fileName, function(fileEntry){
                        // operation has been invalidated for timeout
                        if(!operation.isValid){
                            return;
                        }
                        Paperlit.log.info('Paperlit._getWriteFile: got file entry for ' + operation.fileName);
                        fileEntry.createWriter(function(writer){
                            // operation has been invalidated for timeout
                            if(!operation.isValid){
                                return;
                            }
                            Paperlit.log.info('Paperlit._getWriteFile: got writer for ' + operation.fileName);
                            writer.onwrite = function(){
                                // operation has been invalidated for timeout
                                if(!operation.isValid){
                                    return;
                                }
                                // write data
                                writer.onwrite = function(){
                                    // operation has been invalidated for timeout
                                    if(!operation.isValid){
                                        return;
                                    }
                                    Paperlit.log.info('Paperlit._getWriteFile: onwrite for ' + operation.fileName);
                                    // write finished
                                    Paperlit.lockReadWriteFile = false; 
                                    // remove timeout
                                    if(operation.timeout != null){
                                        clearTimeout(operation.timeout);
                                        operation.timeout = null;
                                    }
                                    //alert('written'); 
                                    setTimeout(operation.successCallback, 1); 
                                };
                                // error writing new data
                                writer.onerror = proxyedErrorCallback;
                                writer.write(operation.text);
                            }
                            // error truncating
                            writer.onerror = proxyedErrorCallback;
                            // truncate all data 
                            writer.truncate(0);
                             
                        }, proxyedErrorCallback);
                   }, proxyedErrorCallback, {create: true, exclusive: false});
                }
            };

            /* Notify event  to client */
            Paperlit.notifyClient = function(eventName){
                try {
                    Paperlit.log.info('Paperlit.notifyClient ' + eventName);
                    cordova.exec(Paperlit.noop, Paperlit.noop, 'paperlit', eventName, []);
                } catch(e) {
                    Paperlit.log.error('exception calling Paperlit.notifyClient: '+ e.message);
                }
            };

            Paperlit.notifyResourceUpdate = function(resourceUrl, successCallback, errorCallback){
                try {
                    Paperlit.log.info('Paperlit.notifyResourceUpdate ' + resourceUrl);
                    cordova.exec(successCallback, errorCallback, 'paperlit', 'notifyResourceUpdate', [resourceUrl]);
                } catch(e) {
                    Paperlit.log.error('exception calling Paperlit.notifyResourceUpdate: '+ e.message);
                }
            };

            
            Paperlit.getResource = function(resourceUrl, successCallback, errorCallback){
                try {
                    Paperlit.log.info('Paperlit.getResource ' + resourceUrl);
                    cordova.exec(successCallback, errorCallback, 'paperlit', 'getResource', [resourceUrl]);
                } catch(e) {
                    Paperlit.log.error('exception calling Paperlit.getResource: '+ e.message);
                }
            };
            
            
            Paperlit.invalidateResourceOn = function(resourceUrl, events, successCallback, errorCallback){
                try {
                    Paperlit.log.info('Paperlit.invalidateResourceOn ' + resourceUrl);
                    cordova.exec(successCallback, errorCallback, 'paperlit', 'invalidateResourceOn', [resourceUrl, events]);
                } catch(e) {
                    Paperlit.log.error('exception calling Paperlit.invalidateResourceOn: '+ e.message);
                }
            };

            Paperlit.invalidateAllResources = function(successCallback, errorCallback){
                try {
                    Paperlit.log.info('Paperlit.invalidateAllResources');
                    cordova.exec(successCallback, errorCallback, 'paperlit', 'invalidateAllResources', []);
                } catch(e) {
                    Paperlit.log.error('exception calling Paperlit.invalidateAllResources: '+ e.message);
                }
            };
            /* Return a json containing a favourites array with the bookmarks */
            Paperlit.getBookmarks = function(successCallback, errorCallback){
                try {
                    Paperlit.log.info('Paperlit.getBookmarks');
                    var callback = Paperlit._getProcessFavoritesFunction(successCallback, errorCallback);
                    cordova.exec(callback, errorCallback, 'paperlit', 'getBookmarks', []);
                } catch(e) {
                    Paperlit.log.error('exception calling Paperlit.getBookmarks: '+ e.message);
                }
            };

            Paperlit._getProcessFavoritesFunction = function(successCallback, errorCallback){
                return function(favoritesJSON){
                    try{
                        var bookmarks = Paperlit._getBookmarksFromFavorites(favoritesJSON);
                        successCallback(bookmarks);
                    }
                    catch(e){
                        errorCallback();
                    }
                };
            };
            /* convert native favorities into bookmarks*/
            Paperlit._getBookmarksFromFavorites = function(favoritesJSON){
                var favorites = favoritesJSON.favorites;
                var bookmarks = [];
                var bookmark, id, issue;
                for(var i=0, l=favorites.length; i<l; i++){
                    bookmark = favorites[i];
                    id = bookmark.uniqueId;
                    // set id as uniqueId
                    bookmark.bookmarkId = id;
                    
                    if(id.indexOf('paperlit://read/') > -1){
                        issue = id.replace('paperlit://read/', '');
                        issue = issue.split('/');
                        // derive old bookmark fields from url
                        bookmark.publicationId = issue[0];
                        bookmark.issueId = issue[1];
                        bookmark.pageId = issue[2].replace(/^[0]+/g,"");
                        // set old issuename field from title
                        bookmark.issueName = bookmark.title ? bookmark.title : '';
                    }
                    bookmarks.push(bookmark);
                }
                return bookmarks;
            };

            Paperlit._getBookmarkCallbackFunction = function(callback, bookmarkId){
                return function(){
                    callback(bookmarkId);
                }
            };
            /* Remove a bookmark and return the id to the success callback for retro compatibility */
            Paperlit.removeBookmark = function(bookmark, successCallback, errorCallback){
                try {
                    var bookmarkId = bookmark;
                    // if parameter is an object (it is not directly an id)
                    if(typeof bookmark === "object"){
                        // if it has a uniqueId prop
                        if(typeof bookmark.uniqueId !== "undefined"){
                            bookmarkId = bookmark.uniqueId;
                        }
                        else if(typeof bookmark.publicationId !== "undefined"){
                            // add padding to pageId
                            pageZeroPage = '' + bookmark.pageId;
                            while(pageZeroPage.length < 3) {
                                pageZeroPage = "0" + pageZeroPage;
                            }
                            // compatibility mode: old bookmarks format
                            bookmarkId = 'paperlit://read/' + bookmark.publicationId + '/' + bookmark.issueId + '/' + pageZeroPage;
                        }
                    }
                
                    Paperlit.log.info('Paperlit.removeBookmark');
                    var callback = Paperlit._getBookmarkCallbackFunction(successCallback, bookmarkId);
                    cordova.exec(callback, errorCallback, 'paperlit', 'removeBookmark', [bookmarkId]);
                } catch(e) {
                    Paperlit.log.error('exception calling Paperlit.removeBookmark: '+ e.message);
                }
            };

            Paperlit.saveBookmark = function(bookmark, successCallback, errorCallback){
                try {
                    Paperlit.log.info('Paperlit.saveBookmark');
                    if(typeof bookmark.publicationId !== 'undefined'){
                        // add padding to pageId
                        pageZeroPage = '' + bookmark.pageId;
                        while(pageZeroPage.length < 3) {
                            pageZeroPage = "0" + pageZeroPage;
                        }
                        bookmark.uniqueId = 'paperlit://read/' + bookmark.publicationId + '/' + bookmark.issueId + '/' + pageZeroPage;
                        if(typeof bookmark.issueName !== 'undefined' && bookmark.issueName){
                            bookmark.title = bookmark.issueName;
                        }
                        if(typeof bookmark.publicationName !== 'undefined' && bookmark.publicationName){
                            bookmark.subtitle = bookmark.publicationName;
                        }
                        bookmark.imageUrl = 'http://www.paperlit.com/pages/' + bookmark.publicationId + '/' + bookmark.issueId + '/' + bookmark.pageId + '?width=350';
                    }
                    var callback = Paperlit._getBookmarkCallbackFunction(successCallback, bookmark.uniqueId);
                    cordova.exec(callback, errorCallback, 'paperlit', 'saveBookmark', [bookmark]);
                } catch(e) {
                    Paperlit.log.error('exception calling Paperlit.saveBookmark: '+ e.message);
                }
            };

            /** 
                Check if the bookmark exists, returns the response and the bookmark object. If not found returns the response.
                If no response return false 
            */
            Paperlit.getBookmarkIndex = function(myBookmark, bookmarks){
                
                if (typeof bookmarks != 'undefined'){
                    for (var i=0; i<bookmarks.length; i++){
                        var bookmark = bookmarks[i];
                        if (myBookmark.bookmarkId == bookmark.bookmarkId){
                            return i;
                        }
                    }
                }
                return -1;
            };

            /** check if two bookmarks are the same by id or by publication, issue, page*/
            Paperlit.isBookmarkEqual = function(bookmark1, bookmark2){
                return (typeof bookmark1 == 'string' && bookmark1 == bookmark2.bookmarkId) || ( (bookmark1.publicationId == bookmark2.publicationId) && (bookmark1.issueId == bookmark2.issueId) && (bookmark1.pageId == bookmark2.pageId) );
            };

            Paperlit.findBookmark = function(bookmark, successCallback, errorCallback){
                //alert('Paperlit.removeBookmark');
                Paperlit.getBookmarks(function(bookmarks){
                    //alert('Paperlit.removeBookmark: got properties');
                    try{
                        // check if bookmark is present
                        var bookmarkIndex = Paperlit.getBookmarkIndex(bookmark, bookmarks);
                        if (bookmarkIndex != -1){
                            // remove
                            successCallback(bookmark.bookmarkId, bookmarkIndex);
                        }
                        else{
                            successCallback(bookmark.bookmarkId, false);
                        }
                    }
                    catch(e){
                        //alert('Paperlit.findBookmark error: ' + e.message);
                    }
                }, errorCallback);
            };

            Paperlit.checkIfCommandIsSupported('setUser', function(){
                /* Let the client store the user profile. Shoud have at least userId and userEmail fields */
	            Paperlit.setUser = function(user, successCallback, errorCallback){
	            	if(typeof user.userId !== 'undefined' && typeof user.userEmail !== 'undefined'){
	            		try {
		                    Paperlit.log.info('Paperlit.setUser');
		                    cordova.exec(successCallback, errorCallback, 'paperlit', 'setUser', [user]);
		                } catch(e) {
		                    Paperlit.log.error('exception calling Paperlit.setUser: '+ e.message);
		                }
	            	}
	            	else{
	            		errorCallback('missing userId or userEmail');
	            	}
	            };
            }, Paperlit.noop);

            
            Paperlit.checkIfCommandIsSupported('getUser', function(){
	            Paperlit.getUser = function(successCallback, errorCallback){
	            	try {
	                    Paperlit.log.info('Paperlit.getUser');
	                    cordova.exec(successCallback, errorCallback, 'paperlit', 'getUser', []);
	                } catch(e) {
	                    Paperlit.log.error('exception calling Paperlit.getUser: '+ e.message);
	                }
	            };
	        }, Paperlit.noop);

	        Paperlit.checkIfCommandIsSupported('removeUser', function(){
	            Paperlit.removeUser = function(successCallback, errorCallback){
	            	try {
	                    Paperlit.log.info('Paperlit.removeUser');
	                    cordova.exec(successCallback, errorCallback, 'paperlit', 'removeUser', []);
	                } catch(e) {
	                    Paperlit.log.error('exception calling Paperlit.removeUser: '+ e.message);
	                }
	            };
	        }, Paperlit.noop);

	        /* Return result: success, response: server response */
	        Paperlit.checkIfCommandIsSupported('isSubscribed', function(){
		        Paperlit.rcs.isSubscribed = function(successCallback, errorCallback){
		        	try {
	                    Paperlit.log.info('Paperlit.isSubscribed');
	                    var callback = function(json){
	                    	Paperlit.log.info('Paperlit.rcs.isSubscribed: ' + JSON.stringify(json));
	                    	successCallback(json);
	                    }
	                    cordova.exec(callback, errorCallback, 'paperlit', 'isSubscribed', []);
	                } catch(e) {
	                    Paperlit.log.error('exception calling Paperlit.isSubscribed: '+ e.message);
	                }
		        };
	        }, Paperlit.noop);

	        Paperlit.checkIfCommandIsSupported('onPageChange', function(){
		        Paperlit.onPageChange = function(m,n){
				    try {
	                    Paperlit.log.info('Paperlit.onPageChange');
	                    cordova.exec(Paperlit.noop, Paperlit.noop, 'paperlit', 'onPageChange', [m,n]);
	                } catch(e) {
	                    Paperlit.log.error('exception calling Paperlit.onPageChange: '+ e.message);
	                }
				}; 
			}, Paperlit.noop);

	        Paperlit.checkIfCommandIsSupported('getCachedAdvUrl', function(){
		        Paperlit.getCachedAdvUrl = function(url, maxAgeInSeconds, successCallback, errorCallback){
				    try {
				    	maxAgeInSeconds = typeof maxAgeInSeconds === 'undefined' ? 0 : maxAgeInSeconds;
	                    Paperlit.log.info('Paperlit.getCachedAdvUrl, required url ' + url + ' with age ' + maxAgeInSeconds);
	                    cordova.exec(successCallback, errorCallback, 'paperlit', 'getCachedAdvUrl', [url, maxAgeInSeconds]);
	                } catch(e) {
	                    Paperlit.log.error('exception calling Paperlit.getCachedAdvUrl: '+ e.message);
	                }
				}; 
			}, Paperlit.noop);


			Paperlit.checkIfCommandIsSupported('registerForPushNotifications', function(){
		        Paperlit.registerForPushNotifications = function(json, successCallback, errorCallback){
				    try {
	                    Paperlit.log.info('Paperlit.registerForPushNotifications');
	                    cordova.exec(successCallback, errorCallback, 'paperlit', 'registerForPushNotifications', [json]);
	                } catch(e) {
	                    Paperlit.log.error('exception calling Paperlit.registerForPushNotifications: '+ e.message);
	                }
				}; 
			}, Paperlit.noop);

			Paperlit.checkIfCommandIsSupported('getRegisteredPushNotifications', function(){
		        Paperlit.getRegisteredPushNotifications = function(successCallback, errorCallback){
				    try {
	                    Paperlit.log.info('Paperlit.getRegisteredPushNotifications');
	                    cordova.exec(successCallback, errorCallback, 'paperlit', 'getRegisteredPushNotifications', []);
	                } catch(e) {
	                    Paperlit.log.error('exception calling Paperlit.getRegisteredPushNotifications: '+ e.message);
	                }
				}; 
			}, Paperlit.noop);

			Paperlit.checkIfCommandIsSupported('reload', function(){
		        Paperlit.reload = function(){
				    try {
	                    Paperlit.log.info('Paperlit.reload');
	                    cordova.exec(Paperlit.noop, Paperlit.noop, 'paperlit', 'reload', []);
	                } catch(e) {
	                    Paperlit.log.error('exception calling Paperlit.reload: '+ e.message);
	                }
				}; 
			}, Paperlit.noop);

			Paperlit.trackEventOn = function(){
				try {
                    
                    if(arguments.length < 2){
                        throw "missing arguments";
                    }
                    var args = Array.prototype.slice.call(arguments);
                    cordova.exec(Paperlit.noop, Paperlit.noop, 'paperlit', 'trackEventOn', args);
                } catch(e) {
                    Paperlit.log.error('exception calling Paperlit.trackEventOn ' + e.message);
                }
			};
			
        } // end cordova
        else{
            // PHONEGAP retro compatibility methods

            Paperlit.findBookmark = function(bookmark, successCallback, errorCallback){
                //alert('Paperlit.removeBookmark');
                Paperlit.readWriteProperties(function(properties){
                    //alert('Paperlit.removeBookmark: got properties');
                    try{
                        if(typeof properties.bookmarks == 'undefined'){
                            properties.bookmarks = [];
                        }
                        // check if bookmark is present
                        var bookmarkIndex = Paperlit.getBookmarkIndex(bookmark, properties.bookmarks);
                        if (bookmarkIndex != -1){
                             // remove
                            successCallback(bookmark.bookmarkId, bookmarkIndex);
                            
                        }
                        else{
                            successCallback(bookmark.bookmarkId, false);
                        }
                    }
                    catch(e){
                        //alert('Paperlit.findBookmark error: ' + e.message);
                    }
                    // don't need to write
                    return {needToWrite: false};
                }, errorCallback);
            };

            Paperlit.removeBookmark = function(bookmark, successCallback, errorCallback){
                if(typeof $ === 'undefined'){
                    errorCallback();
                    return;
                }
                //alert(JSON.stringify(bookmark));
                if(typeof bookmark.isNative != 'undefined' && bookmark.isNative === true){
                    //alert('native bookmark')
                    Paperlit.readFile(Paperlit.nativeBookmarksFileName, function(response){
                        var xml = $.parseXML(response);
                        //alert(xml)
                        try{
                        // add folio bookmarks
                            var favorites = $(xml).find('Favorite');
                            var favorite;
                            var found = false;
                            //alert(favorites.length)
                            for (var i=0; i<favorites.length; i++){
                                favorite = $(favorites[i]);
                                //alert('evaluating ' + bookmark.bookmarkId + ' against ' + favorite.attr('UniqueId'))
                                if(favorite.attr('UniqueId') == bookmark.bookmarkId){
                                    found = i;
                                    break;
                                }
                            }                
                            
                            if (found !== false){
                                //alert('deleting')
                                // recreate xml
                                $($(xml).find('Favorite')[found]).remove();
                                
                                var nativeUpdate = $(xml).find('Favorites').attr('Update', (parseInt($(xml).find('Favorites').attr('Update'), 10) + 1));
                                // store again overwriting native bookmarks.xml
                                
                                var xmlString = (new XMLSerializer()).serializeToString(xml);
                                //alert(xmlString);
                                //return;
                                
                                Paperlit.overwriteProperties(xmlString, function(){
                                    // return bookmark id of the removed bookmark
                                    successCallback(bookmark.bookmarkId);
                                }, errorCallback, Paperlit.nativeBookmarksFileName);
                                return;
                            }
                        }
                        catch(e){
                            alert(e.message);   
                        }
                        if (typeof errorCallback != 'undefined'){
                            // not found
                            setTimeout(errorCallback, 1);
                        }
                    }, errorCallback);
                }
                else{
                    Paperlit.readWriteProperties(function(properties){
                        try{
                            if(typeof properties.bookmarks == 'undefined'){
                                properties.bookmarks = [];
                            }
                            // check if bookmark is present
                            var bookmarkIndex = Paperlit.getBookmarkIndex(bookmark, properties.bookmarks);
                            if (bookmarkIndex != -1){
                                 // remove
                                properties.bookmarks.splice(bookmarkIndex,1);
                                var bookmarkId = bookmark.bookmarkId;
                                // store again properties
                                
                                return {needToWrite: true, properties: properties, onWriteSuccess: function(){
                                    // return bookmark id of the removed bookmark
                                    successCallback(bookmarkId);
                                }};
                            }
                            else if (typeof errorCallback != 'undefined'){
                                // not found
                                setTimeout(errorCallback, 1);
                            }
                        }
                        catch(e){
                            alert('Paperlit.removeBookmark, error: ' + e.message);
                        }
                        // don't need to write
                        return {needToWrite: false};
                
                    }, errorCallback);
                }
            };

            
            Paperlit.saveBookmark = function(bookmark, successCallback, errorCallback){
                //alert('Paperlit.saveBookmark ');
                Paperlit.readWriteProperties(function(properties){
             
                    //alert('Paperlit.saveBookmark got props');
                    if(typeof properties.bookmarks == 'undefined'){
                        properties.bookmarks = [];
                    }
                    // check if bookmark is present
                    var bookmarkIndex = Paperlit.getBookmarkIndex(bookmark, properties.bookmarks);
                    if (bookmarkIndex == -1){
                        
                        if(bookmark.bookmarkId == null || bookmark.bookmarkId == ''){
                            // generate id for the new bookmark
                            bookmark.bookmarkId = Math.uuid();
                        }
                        // add bookmark
                        properties.bookmarks.push(bookmark);
                        var bookmarkId = bookmark.bookmarkId;
                        // save properties
                        
                        return {needToWrite: true, properties: properties, onWriteSuccess: function(){
                            // return bookmark id of the saved bookmark
                            successCallback(bookmarkId);
                        }};
                    }
                    else{                
                        // call successCallback passing bookmark id
                        var bookmarkId = properties.bookmarks[bookmarkIndex].bookmarkId;
                        setTimeout(function(){
                            successCallback(bookmarkId);
                        }, 1);
                        
                        return {needToWrite: false};
                    }
                
                }, errorCallback);
            };

            /** 
                Check if the bookmark exists, returns the response and the bookmark object. If not found returns the response.
                If no response return false 
            */
            Paperlit.getBookmarkIndex = function(myBookmark, bookmarks){
                
                if (typeof bookmarks != 'undefined'){
                    
                    if (myBookmark.type && myBookmark.type == 'feed'){
                        for (var i=0; i<bookmarks.length; i++){
                             var bookmark = bookmarks[i];
                             if (myBookmark.bookmarkId == bookmark.bookmarkId){
                                return i;
                             }
                        }
                    }
                    else{
                        for (var i=0; i<bookmarks.length; i++){
                             var bookmark = bookmarks[i];
                             if (Paperlit.isBookmarkEqual(myBookmark, bookmark)){
                                return i;
                             }
                        }
                    }
                }
                return -1;
            };

            /** check if two bookmarks are the same by id or by publication, issue, page*/
            Paperlit.isBookmarkEqual = function(bookmark1, bookmark2){
                return (typeof bookmark1 == 'string' && bookmark1 == bookmark2.bookmarkId) || ( (bookmark1.publicationId == bookmark2.publicationId) && (bookmark1.issueId == bookmark2.issueId) && (bookmark1.pageId == bookmark2.pageId) );
            };

            /* Read a text file and returns the content */
            Paperlit.readFile = function(fileName, successCallback, errorCallback) {
                if(Paperlit.lockReadWriteFile){
                    //alert('read locked');
                    // postpone call
                    setTimeout(function(){
                        Paperlit.readFile(fileName, successCallback, errorCallback);
                    }, 10);
                    return;
                }
                //alert('reading');
                // avoid concurrent read/write
                Paperlit.lockReadWriteFile = true;
                try{
                    var reader = new FileReader();
                    //alert('file reader');
                    reader.onload = function(event){
                        //alert('read');
                        Paperlit.lockReadWriteFile = false;
                        //alert('loaded' + event.target.result);
                        if (typeof event.target.result == 'undefined' || event.target.result == null || event.target.result == ""){
                            //alert('no content ' + fileName);
                            errorCallback();
                            return;
                        }
                        
                        try{
                            setTimeout(function(){
                                //alert('success');
                                successCallback(event.target.result);
                            }, 1);
                        }catch(e){
                            //alert('ex  ' + fileName + ' e ' + e.message);
                            if(errorCallback)
                                errorCallback();
                        }
                       
                    };
                    reader.onerror = function(){
                        //alert('error file read ' + fileName);
                        Paperlit.lockReadWriteFile = false;
                        errorCallback();
                    };
                    // read data as text
                    reader.readAsText(fileName);

                }
                catch(e){
                    Paperlit.lockReadWriteFile = false;
                    errorCallback();
                    //alert('exc ' + e.message);
                }
            };
            
            /* Write a text file on device */
            Paperlit.writeFile = function(text, successCallback, errorCallback, fileName){
                //alert('file write');
                if(Paperlit.lockReadWriteFile){
                    // postpone call
                    setTimeout(function(){
                        Paperlit.writeFile(text, successCallback, errorCallback, fileName);
                    }, 10);
                    return;
                }
                // avoid concurrent read/write
                Paperlit.lockReadWriteFile = true;

                errorCallback = typeof errorCallback != 'undefined' ? errorCallback : Paperlit.noop;
                //alert('file writer');
                try{
                    var writer = new FileWriter(fileName);
                    //alert('file writer created');
                    writer.onwriteend = function(){
                        //alert('deleted');
                        var writer = new FileWriter(fileName);
                        writer.onwriteend = function(){
                            // write finished
                            Paperlit.lockReadWriteFile = false; 
                            //alert('written'); 
                            setTimeout(successCallback, 1); 
                        };
                        // error writing new data
                        writer.onerror = function(){
                            Paperlit.lockReadWriteFile = false;
                            errorCallback();
                        };
                        writer.write(text);
                    };
                    // error truncating
                    writer.onerror = function(){
                        Paperlit.lockReadWriteFile = false;
                        errorCallback();
                    }
                    //delete 
                    writer.truncate(0);
                }
                catch(e){
                    Paperlit.lockReadWriteFile = false;
                    errorCallback();
                    //alert('exc ' + e.message);
                }
            };
        }
            
        // starting platform checking
        if(Paperlit.isIOS) {
            if(Paperlit.isCordova === false){
                // restore navigator.geolocation.getCurrentPosition;
                navigator.geolocation.getCurrentPosition = function(){
                    Paperlit.geolocation.getCurrentPosition.apply(this, arguments);
                }
                //FIX for phonegap.js 0.9.4 use iframe to pass commands
                PhoneGap.run_command = function() {
                    if (!PhoneGap.available || !PhoneGap.queue.ready)
                        return;

                    PhoneGap.queue.ready = false;

                    //add iframe as new context for phonegap commands
                    if(!this.gapBridge) {
                        this.gapBridge = document.createElement("IFRAME");
                        this.gapBridge.setAttribute("height","0px");
                        this.gapBridge.setAttribute("width","0px");
                        this.gapBridge.setAttribute("frameborder","0");
                        document.documentElement.appendChild(this.gapBridge);
                    }

                    var args = PhoneGap.queue.commands.shift();
                    if (PhoneGap.queue.commands.length == 0) {
                        clearInterval(PhoneGap.queue.timer);
                        PhoneGap.queue.timer = null;
                    }

                    var uri = [];
                    var dict = null;
                    for (var i = 1; i < args.length; i++) {
                        var arg = args[i];
                        if (arg == undefined || arg == null)
                            continue;
                        if (typeof(arg) == 'object')
                            dict = arg;
                        else
                            uri.push(encodeURIComponent(arg));
                    }
                    var url = "gap://" + args[0] + "/" + uri.join("/");
                    if (dict != null) {
                        url += "?" + encodeURIComponent(JSON.stringify(dict));
                    }
                    //use bridge for url request
                    this.gapBridge.src = url;
                };
                //END FIX

                //Create iframe as bridge for Paperlit commands
                Paperlit.commandsBridge = document.createElement("IFRAME");
                Paperlit.commandsBridge.setAttribute("height","0px");
                Paperlit.commandsBridge.setAttribute("width","0px");
                Paperlit.commandsBridge.setAttribute("frameborder","0");
                document.documentElement.appendChild(Paperlit.commandsBridge);
            }

            Paperlit.getStaticFunctionsName = function(fn) {
                var funkId = "f" + Paperlit._anomFunkMapNextId++;
                var funk = function () {
                    //execute
                    fn.apply(this, arguments);

                };
                Paperlit._anomFunkMap[funkId] = funk;
                return "Paperlit._anomFunkMap." + funkId;
            }

            /** Returns true if a specific method is supported by the native client. */
            Paperlit.isMethodSupported = function(methodName, successCallback, errorCallback) {
                //alert('IsMethodSupported');

                if (methodName == 'getCachedUrlAsBase64'){
                    methodName = 'getCachedUrlAsString';
                }
                errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
                var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                PhoneGap.exec("PaperlitCommands.isMethodSupported", methodName, functionsName.successFuncName, functionsName.errorFuncName);
                
            };
            /** Takes an Apple InApp product id and returns the price (eg: $1.99) localized for the current user store (eg: italian vs british store). */
            Paperlit.getProductPrice = function(productId, successCallback, errorCallback) {
                try {
                    Paperlit.log.info('Paperlit.getProductPrice for iPad: '+productId);
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);

                    PhoneGap.exec("PaperlitCommands.getProductPrice", productId, functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    Paperlit.log.error('exception calling PaperlitCommands.getProductPrice: '+productId);
                    errorCallback();
                }

            };
            /** Returns the list of publication/issues cached by the application (available also offline). */
            Paperlit.getArchivedIssues = function(successCallback, errorCallback) {
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.getArchivedIssues", functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    errorCallback();
                }
            };
            /** Delete an issue from the application cache. */
            Paperlit.deleteArchivedIssue = function(publicationId, issueId, successCallback, errorCallback) {
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.deleteArchivedIssue", publicationId, issueId, functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                }
            };
            /**
             * Returns the cached content of the given url as a file:// url.
             * If the file in cache is older than maxAgeInSeconds it will be
             * reloaded from the network. If maxAgeInSeconds is 0 content will
             * always be refreshed from the network. If maxAgeInSeconds is a
             * really large value (eg: 10 years) you will always get the latest
             * content from the local cache.
             */
            Paperlit._getCachedUrl = function(url, maxAgeInSeconds, successCallback, errorCallback) {
                errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
                if(Paperlit.isRunningLocally === false){
                	Paperlit.log.error('Paperlit._getCachedUrl for Apple: cannot read cache from remote page, '+ url);
                    // cannot read cache from remote page
                    errorCallback();
                    return;
                }
                try {
                    Paperlit.log.info('Paperlit._getCachedUrl for Apple asked for url: '+url);
                    if (Paperlit.isLocalUrl(url)){
                        // url is local, nothing to cache
                        setTimeout(function(){
                            // client < 8.1.0 need to bypass webview cache adding a timestamp
                            if(typeof Paperlit.state === 'undefined' || Paperlit.utils.isVersionSupported(Paperlit.state.bundleVersion, '8.1.0') === false){
                                if(Paperlit.state.bundleId.indexOf('gazzetta') == -1 && Paperlit.state.bundleId.indexOf('rcs') == -1){
                                	// add timestamp to avoid webview caching
                                	url = Paperlit.utils.appendUrlParameter(url, 'ts', (new Date).getTime());
                                }
                            }
                            successCallback(url);
                        }, 1);
                        return;
                    }


                    // avoid browser caching
                    var successF = function(url){
                        // client < 8.1.0 need to bypass webview cache adding a timestamp
                        if(typeof Paperlit.state === 'undefined' || Paperlit.utils.isVersionSupported(Paperlit.state.bundleVersion, '8.1.0') === false){
                            if(Paperlit.state.bundleId.indexOf('gazzetta') == -1 && Paperlit.state.bundleId.indexOf('rcs') == -1){
                            	// add timestamp to avoid webview caching
                            	url = Paperlit.utils.appendUrlParameter(url, 'ts', (new Date).getTime());
                            }
                        }
                        successCallback(url);
                    }
                    var functionsName = Paperlit.getFunctionsName(successF, errorCallback);
                    PhoneGap.exec("PaperlitCommands.getCachedUrl", url, maxAgeInSeconds, functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    //alert('PaperlitCommands.getCachedUrl: exception');
                }

            };
            /**
             * Same as GetCachedUrl but the actual content is returned as a string.
             */
            /*Paperlit.getCachedUrlAsString = function(url, maxAgeInSeconds, successCallback, errorCallback) {

                try {
                    
                    if (url.indexOf('file://') != -1 || url.indexOf('/') == 0 || url.indexOf('.') == 0){
                        // url is local, nothing to cache
                        Paperlit.getAsyncUrl(url, successCallback, errorCallback)
                        return;
                    }
                    
                    //alert('Paperlit.getCachedUrlAsString for Apple asked for url: '+url);
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.getCachedUrlAsString", url, maxAgeInSeconds, functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    Paperlit.log.error('Paperlit.getCachedUrlAsString : failed '+e.message);
                }

            };*/
            /**
             * Uses getCachedUrl and then load the content of the file through an ajax call.
             */
            Paperlit.getCachedUrlAsString = function(url, maxAgeInSeconds, successCallback, errorCallback) {
                //load file
                Paperlit.getCachedUrl(url, maxAgeInSeconds, function(cachedUrl) {
                    try {
                        Paperlit.getAsyncUrl(cachedUrl, successCallback, errorCallback);
                    } catch(e) {
                        errorCallback(e);
                    }
                }, errorCallback);
            };
            /**
             * Same as getCachedUrlAsString but the actual content is returned as a base64 string of the specified mime type.
             * This is useful if an external page wants to cache some data on the native client without inucurring in cross-domain issues.
             * Local files cannot be loaded by external page (i.e., getCachedUrl cannot be use in this case)
             *
             * Warning: this method is defined also at the generic level (not related to platform) and is based on getCachedUrlAsString.
             * 			Here it it redefined because currently the getCachedUrlAsString for the iOS is implemented to a traditional getCachedUrl that is not
             * 			cross-domain safe.
             */
            Paperlit.getCachedUrlAsBase64 = function(url, maxAgeInSeconds, mimeType, successCallback, errorCallback) {
                try {
                    Paperlit.log.info('Paperlit.getCachedUrlAsBase64 for Apple asked for url: '+url);
                    url = 'http://www.paperlit.com/php/services/base64.php?url=' + encodeURIComponent(url)
                    if (mimeType) {
                        url = url +'&amp;mime=' + encodeURIComponent(mimeType);
                    }

                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.getCachedUrlAsString", url, maxAgeInSeconds, functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    Paperlit.log.error('Paperlit.getCachedUrlAsBase64 : failed '+e.message);
                }

            };
            /**	Asks for application state. Response is a JSON string with the keys:
             "uniqueIdentifier",  device unique identifier
             "name",              device name
             "systemName",        device system name
             "systemVersion",     device system version
             "model",             device model
             "localizedModel",    device model as a localized string
             "bundleId",          app bundle id
             "hash",              repubblica hash code
             "bundleVersion",     app bundle version
             "publicationId",     publication id of issue currently being read
             "issueId",           issue id of issue currently being read
             "pageId",            current page
             "productId",         id of product that was last purchased
             "email"              contents of email field in settings
             **/
            Paperlit.getState = function(successCallback, errorCallback) {
                Paperlit.log.info('Paperlit.getState for Apple ');
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.getState", functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    errorCallback();
                }
            };
            //};

            /* Default ipad is to use command bridge as at first openUrl was not supported */
            Paperlit._openUrl = function(url, successCallback, errorCallback) {

                Paperlit.log.info('Paperlit._openUrl for iPad asked for url ' + url);
                try {
                    Paperlit.commandsBridge.src = url;
                    successCallback != null ? successCallback() : Paperlit.noop();
                } catch(e) {
                    Paperlit.log.error('Paperlit._openUrl: failed ' + e.message);
                    errorCallback != null ? errorCallback() : Paperlit.noop();
                }

            };
          
            
            /**** END OF INTERFACE METHODS ***/

            Paperlit.checkIfCommandIsSupported('getIssueMetadata', function(){
                 /** Returns the XML metadata of the required issue. The metadata is downloaded by the client with the issue and stored in the local cache*/
                    Paperlit.getIssueMetadata = function(publicationId, issueId, successCallback, errorCallback){
                        try {
                            Paperlit.log.info('Paperlit.getIssueMetadata for iOS');
                            errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
                            var callback = function(cachedUrl){
                                Paperlit.getCachedUrlAsXML(cachedUrl, 0, successCallback, errorCallback);
                            }
                            var functionsName = Paperlit.getFunctionsName(callback, errorCallback);
                            PhoneGap.exec("PaperlitCommands.getIssueMetadata", publicationId, issueId, functionsName.successFuncName, functionsName.errorFuncName);
                        } catch(e) {
                            Paperlit.log.error('Paperlit.getMetadata for iOS: error calling PaperlitCommands.getIssueMetadata');
                        }
                    };
            }, Paperlit.noop);
            
            Paperlit.checkIfCommandIsSupported('getCachedIssueMetadata', function(){
                 /** Returns the XML metadata of the required issue. The metadata is downloaded by the client with the issue and stored in the local cache*/
                    Paperlit.getIssueMetadata = function(publicationId, issueId, successCallback, errorCallback){
                        try {
                            Paperlit.log.info('Paperlit.getIssueMetadata for iOS');
                            errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
                            var callback = function(cachedUrl){
                                Paperlit.getCachedUrlAsXML(cachedUrl, 0, successCallback, errorCallback);
                            }
                            var functionsName = Paperlit.getFunctionsName(callback, errorCallback);
                            PhoneGap.exec("PaperlitCommands.getCachedIssueMetadata", publicationId, issueId, 0, functionsName.successFuncName, functionsName.errorFuncName);
                        } catch(e) {
                            Paperlit.log.error('Paperlit.getMetadata for iOS: error calling PaperlitCommands.getCachedIssueMetadata');
                        }
                    };
                    
                    Paperlit.getCachedIssueMetadata = function(publicationId, issueId, maxAgeInSeconds, successCallback, errorCallback){
                        try {
                            Paperlit.log.info('Paperlit.getCachedIssueMetadata for iOS');
                            errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
                            var callback = function(cachedUrl){
                                Paperlit.getCachedUrlAsXML(cachedUrl, 0, successCallback, errorCallback);
                            }
                            var functionsName = Paperlit.getFunctionsName(callback, errorCallback);
                            PhoneGap.exec("PaperlitCommands.getCachedIssueMetadata", publicationId, issueId, maxAgeInSeconds, functionsName.successFuncName, functionsName.errorFuncName);
                        } catch(e) {
                            Paperlit.log.error('Paperlit.getCachedIssueMetadata for iOS: error calling PaperlitCommands.getCachedIssueMetadata');
                        }
                    };
            }, Paperlit.noop);
            
            Paperlit.checkIfCommandIsSupported('getCachedPageUrl', function(){
                    /** Returns url of a cached issue page image */
                    Paperlit.getCachedPageUrl = function(issue, pageId, successCallback, errorCallback, width, height){
                        var url = Paperlit.getServerPageUrl(issue, pageId, width, height);
                        Paperlit.getCachedUrl(url, 60*60*24*30*12, function(cachedUrl) {successCallback({url: cachedUrl})}, 
                            function(){
                                // in case of failure, try with native rendering
                                Paperlit._getCachedPageUrl(issue, pageId, successCallback, errorCallback, width, height);
                            }
                        );
                    };
                 
                    Paperlit._getCachedPageUrl = function(issue, pageId, successCallback, errorCallback, width, height){
                        try {
                            
                            var devicePixelRatio = (typeof window.devicePixelRatio != "undefined") ?  window.devicePixelRatio : 1;
                            
                            var page = { url : "", width: 0, height: 0};
                            if (typeof width != 'undefined' && width != null) {
                                page.width = Math.round(width * devicePixelRatio);
                            }
                            if (typeof height  != 'undefined' && height != null) {
                                page.height = Math.round(height * devicePixelRatio);
                            }
                            
                            // callback for success
                            var callback = function(cachedUrl){
                                //alert('cached image');
                                page.url = cachedUrl; 
                                successCallback(page);
                            }
                            var defaultErrorCallback = function(error) {
                                setTimeout(function(){
                                    if(typeof navigator.onLine == 'undefined' || navigator.onLine){
                                        //alert('I am online. Retry request getCachedUrl: ' + url);
                                        Paperlit._getCachedPageUrl(issue, pageId, callback, defaultErrorCallback, width, height);
                                    }
                                    else{
                                        // reschedule
                                        defaultErrorCallback();
                                    }
                                }, 5000);
                            };
                             //create an error callback that will try again to load
                            errorCallback = errorCallback != null ? errorCallback : defaultErrorCallback;
                           //alert('requesting image ' + issue.publicationId + ' ' + issue.issueId + ' ' + pageId + ' ' + page.width + ' ' + page.height);
                           var functionsName = Paperlit.getFunctionsName(callback, errorCallback);
                           PhoneGap.exec("PaperlitCommands.getCachedPageUrl", issue.publicationId, issue.issueId, pageId, page.width, page.height, functionsName.successFuncName, functionsName.errorFuncName);
                        } catch(e) {
                            Paperlit.log.error('Paperlit.getCachedPageUrl for iOS: error calling PaperlitCommands.getCachedPageUrl');
                        }
                    };
            }, Paperlit.noop);
           
            
            Paperlit.checkIfCommandIsSupported('openUrl', function(){
                    //replace Paperlit._openUrl that uses iframe command bridge
                    Paperlit._openUrl = function(url, successCallback, errorCallback) {

                        Paperlit.log.info('Paperlit._openUrl for iPad asked for url ' + url);
                        try {
                            var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                            PhoneGap.exec("PaperlitCommands.openUrl",  url,  functionsName.successFuncName, functionsName.errorFuncName);
                        } catch(e) {
                            Paperlit.log.error('Paperlit._openUrl: failed ' + e.message);
                            errorCallback != null ? errorCallback() : Paperlit.noop();
                        }

                    };
            }, Paperlit.noop);
            
            Paperlit.trackEvent = function() {
                try {
                    
                    if (arguments.length < 3){
                        throw "missing arguments";
                    }
                    var args = Array.prototype.slice.call(arguments);
                    
                    var errorCallback = args.pop();
                    var successCallback = args.pop();
                
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    
                    args.push(functionsName.successFuncName);
                    args.push(functionsName.errorFuncName);
                    // add command at top
                    args.unshift('PaperlitCommands.trackEvent');
                    
                    //setTimeout(function(msg){alert(msg)}, 1, ('Paperlit.webtrekk.trackEvent: calling ' + args.toString()));
                    PhoneGap.exec.apply(this, args);
                    
                } catch(e) {
                    Paperlit.log.error('exception calling Paperlit.trackEvent ' + e.message);
                }
            };        
            
            /* Social feature, login on platform*/
            Paperlit.social.login = function(platform, successCallback, errorCallback){
                Paperlit.log.info('Paperlit.login for iPad asked for platform ' + platform);
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.login",  platform,  functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    Paperlit.log.error('Paperlit.login: failed ' + e.message);
                    errorCallback != null ? errorCallback() : Paperlit.noop();
                }
            };               
          
            /* Social feature, logout on platform*/
            Paperlit.social.logout = function(platform, successCallback, errorCallback){
                Paperlit.log.info('Paperlit.logout for iPad asked for platform ' + platform);
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.logout",  platform,  functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    Paperlit.log.error('Paperlit.logout: failed ' + e.message);
                    errorCallback != null ? errorCallback() : Paperlit.noop();
                }
            };    
            
            /* Social feature, get login status on platform*/
            Paperlit.social.getLoginStatus = function(platform, successCallback, errorCallback){
                Paperlit.log.info('Paperlit.getLoginStatus for iPad asked for platform ' + platform);
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.getLoginStatus",  platform,  functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    Paperlit.log.error('Paperlit.getLoginStatus: failed ' + e.message);
                    errorCallback != null ? errorCallback() : Paperlit.noop();
                }
            };  
            
            /* share url on platform */
            Paperlit.social.share = function(platform, url, successCallback, errorCallback){
                Paperlit.log.info('Paperlit.share for iPad asked for platform ' + platform);
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.share",  platform, url, functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    Paperlit.log.error('Paperlit.share: failed ' + e.message);
                    errorCallback != null ? errorCallback() : Paperlit.noop();
                }
            };  
            
            /* inviteFriends on platform */
            Paperlit.social.inviteFriends = function(platform, successCallback, errorCallback){
                Paperlit.log.info('Paperlit.inviteFriends for iPad asked for platform ' + platform);
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.inviteFriends",  platform, functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    Paperlit.log.error('Paperlit.inviteFriends: failed ' + e.message);
                    errorCallback != null ? errorCallback() : Paperlit.noop();
                }
            };

            /* 
                Mini App specific commands (only iOS)
            */
            
            // closes the current miniapp
            Paperlit.closeApp = function(errorCallback) {
                try {
                    var functionsName = Paperlit.getFunctionsName(Paperlit.noop(), errorCallback);
                    PhoneGap.exec("PaperlitCommands.closeApp", functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    //alert('PaperlitCommands.closeApp: exception');
                }
                
            };
            
            // tells the system the app is ready to be displayed
            Paperlit.startApp = function(successCallback, errorCallback) {
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.startApp", functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    //alert('PaperlitCommands.startApp: exception');
                }  
            };
            
            // saves the provided base64 encoded image data as the application content
            Paperlit.saveBase64AppImageContent = function(dataUrl, successCallback, errorCallback) {
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.saveBase64AppImageContent", dataUrl, false, functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    //alert('PaperlitCommands.saveBase64AppContent: exception');
                }  
            };
            
            // saves the provided base64 encoded image data as the application content, closes the app when finished
            Paperlit.saveBase64AppImageContentThenExit = function(dataUrl, errorCallback) {
                try {
                    var functionsName = Paperlit.getFunctionsName(Paperlit.noop(), errorCallback);
                    PhoneGap.exec("PaperlitCommands.saveBase64AppImageContent", dataUrl, true, functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    //alert('PaperlitCommands.saveBase64AppImageContentAndExit: exception');
                }  
            };
            
            // saves the provided base64 encoded state data and image data as the application content
            Paperlit.saveBase64AppContentAndImage = function(dataUrl, imageDataUrl, successCallback, errorCallback) {
                try {
                    var functionsName = Paperlit.getFunctionsName(Paperlit.noop(), errorCallback);
                    PhoneGap.exec("PaperlitCommands.saveBase64AppContentAndImage", dataUrl, imageDataUrl, false, functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    //alert('PaperlitCommands.saveBase64AppImageContentAndExit: exception');
                }  
            };
            
            // saves the provided base64 encoded state data and image as the application content, closes the app when finished
            Paperlit.saveBase64AppContentAndImageThenExit = function(dataUrl, imageDataUrl, errorCallback) {
                try {
                    var functionsName = Paperlit.getFunctionsName(Paperlit.noop(), errorCallback);
                    PhoneGap.exec("PaperlitCommands.saveBase64AppContentAndImage", dataUrl, imageDataUrl, true, functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    //alert('PaperlitCommands.saveBase64AppImageContentAndExit: exception');
                }  
            };
            
            // deletes any saved app content
            Paperlit.deleteSavedAppContent = function(successCallback, errorCallback) {
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.deleteSavedAppContent", functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    //alert('PaperlitCommands.deleteSavedAppContent: exception');
                }  
            };
            
            // exports the provided base64 encoded data as an image to the photo album
            Paperlit.exportBase64AppImageContentToPhotos = function(dataUrl, width, height, overPdf, successCallback, errorCallback) {
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.exportBase64AppImageContentToPhotos", dataUrl, width, height, "", overPdf, functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    //alert('PaperlitCommands.exportBase64AppImageContentToPhotos: exception');
                }  
            };
            
            // exports the provided base64 encoded data as an image over a rect that lays inside the miniapp box to the photo album
            Paperlit.exportBase64AppImageContentRectToPhotos = function(dataUrl, width, height, rectString, successCallback, errorCallback) {
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.exportBase64AppImageContentToPhotos", dataUrl, width, height, rectString, true, functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    //alert('PaperlitCommands.exportBase64AppImageContentToPhotos: exception');
                }
            };
            
            // retrieves the path to the saved miniapp content
            Paperlit.getPathToAppContentFile = function(successCallback, errorCallback) {
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.getPathToAppContentFile", functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    //alert('PaperlitCommands.getPathToAppContentFile: exception');
                }  
            };
            
            // shows the activity indicator
            Paperlit.showActivityIndicator = function(successCallback, errorCallback) {
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.showActivityIndicator", functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    //alert('PaperlitCommands.showActivityIndicator: exception');
                }  
            };
            
            // hide the activity indicator
            Paperlit.hideActivityIndicator = function(successCallback, errorCallback) {
                try {
                    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
                    PhoneGap.exec("PaperlitCommands.hideActivityIndicator", functionsName.successFuncName, functionsName.errorFuncName);
                } catch(e) {
                    //alert('PaperlitCommands.hideActivityIndicator: exception');
                }  
            };  
          
            // now check if touchend is supported (>=6.0)
            
            Paperlit.getState(function(state){
                Paperlit.state = state;
                try{
                    if(parseInt(state.systemVersion.substring(0,1)) >= 6){
                        Paperlit.clickEventName = 'touchend';  
                    }
                }
                catch(e){}
                
                Paperlit.processSupportedCommandsQueue();
            }, Paperlit.processSupportedCommandsQueue);
          
       
        } 
        else if(Paperlit.isAndroid) {
            
            Paperlit.clickEventName = 'touchend';
            
            if(Paperlit.isCordova === false){
                // force phonegap plugin for geolocation
                Geolocation.usePhoneGap();
            }
            
            Paperlit.nativeBookmarksFileName =  window.location.href.substring(0, window.location.href.lastIndexOf('/') + 1 ).replace('file://','') + '../app_PPBookmarks/PPBookmarks.xml';
            //Creating PaperlitCommands prototype for Phonegap plugint
            var PaperlitCommands = function() {
            };
            PaperlitCommands.prototype.echo = function(message, successCallback, errorCallback) {
                try {
                    Paperlit.log.info('Paperlit.echo for Android message: '+message);
                    PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'echo', [message]);
                } catch(e) {
                    Paperlit.log.error('Paperlit.echo: failed calling phonegap ' + e.message);
                }
            };
            PaperlitCommands.prototype.isMethodSupported = function(methodName, successCallback, errorCallback) {
                Paperlit.log.info('PaperlitCommands.prototype.isMethodSupported for Android ');
                PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'isMethodSupported', [methodName]);
            };
    
            PaperlitCommands.prototype.getCachedUrl = function(url, maxAgeInSeconds, successCallback, errorCallback) {
                try {

                    Paperlit.log.info('PaperlitCommands.prototype.getCachedUrl for Android asked for url: '+url);
                    PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'getCachedUrl', [url, maxAgeInSeconds]);

                } catch(e) {
                    Paperlit.log.error('PaperlitCommands.prototype.getCachedUrl: failed calling phonegap ' + e.message);
                }
            };
            PaperlitCommands.prototype.getCachedUrlAsString = function(url, maxAgeInSeconds, successCallback, errorCallback) {

                try {

                    Paperlit.log.info('PaperlitCommands.prototype.getCachedUrlAsString for Android asked for url: '+url);
                    PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'getCachedUrlAsString', [url, maxAgeInSeconds]);
                } catch(e) {
                    Paperlit.log.error('PaperlitCommands.prototype.getCachedUrlAsString: failed calling phonegap ' + e.message);
                }

            };
            PaperlitCommands.prototype.getState = function(successCallback, errorCallback) {

                try {
                    Paperlit.log.info('PaperlitCommands.prototype.getState for Android');
                    PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'getState', []);
                } catch(e) {
                    Paperlit.log.error('PaperlitCommands.prototype.getState: failed calling phonegap ' + e.message);
                    errorCallback();
                }

            };
            PaperlitCommands.prototype.openUrl = function(url, successCallback, errorCallback) {
                Paperlit.log.info('PaperlitCommands.prototype.openUrl for Android ');
                try {

                    PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'openUrl', [url]);
                } catch(e) {
                    Paperlit.log.error('PaperlitCommands.prototype.openUrl: failed calling phonegap ' + e.message);
                    errorCallback();
                }

            };
			PaperlitCommands.prototype.getArchivedIssues = function(successCallback, errorCallback) {
                Paperlit.log.info('PaperlitCommands.prototype.getArchivedIssues for Android ');
                try {
                    PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'getArchivedIssues', []);
                } catch(e) {
                    Paperlit.log.error('PaperlitCommands.prototype.getArchivedIssues: failed calling phonegap ' + e.message);
                    errorCallback();
                }

            };
			PaperlitCommands.prototype.deleteArchivedIssue = function(publicationId, issueId, successCallback, errorCallback) {
                Paperlit.log.info('PaperlitCommands.prototype.deleteArchivedIssue for Android ');
                try {
                    PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'deleteArchivedIssue', [publicationId, issueId]);
                } catch(e) {
                    Paperlit.log.error('PaperlitCommands.prototype.deleteArchivedIssue: failed calling phonegap ' + e.message);
                    errorCallback();
                }

            };
            
           	PaperlitCommands.prototype.getIssueMetadata = function(publicationId, issueId, successCallback, errorCallback) {
                Paperlit.log.info('PaperlitCommands.prototype.getIssueMetadata for Android ');
                try {
                    PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'getIssueMetadata', [publicationId, issueId]);
                } catch(e) {
                    Paperlit.log.error('PaperlitCommands.prototype.getIssueMetadata: failed calling phonegap ' + e.message);
                    errorCallback();
                }

            };
            
            PaperlitCommands.prototype.getCachedPageUrl = function(publicationId, issueId, pageId, width, height, successCallback, errorCallback) {
                Paperlit.log.info('PaperlitCommands.prototype.getCachedPageUrl for Android');
                try {
                    PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'getCachedPageUrl', [publicationId, issueId, pageId, width, height]);
                } catch(e) {
                    Paperlit.log.error('PaperlitCommands.prototype.getCachedPageUrl: failed calling phonegap ' + e.message);
                    errorCallback();
                }

            };
            
            PaperlitCommands.prototype.trackEvent = function(parameters, successCallback, errorCallback) {
                Paperlit.log.info('PaperlitCommands.prototype.trackEvent for Android');
                try {
                    PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'trackEvent', parameters);
                } catch(e) {
                    Paperlit.log.error('PaperlitCommands.prototype.trackEvent: failed calling phonegap ' + e.message);
                    errorCallback();
                }
            };
            
            PaperlitCommands.prototype.login = function(platform, successCallback, errorCallback) {
                Paperlit.log.info('PaperlitCommands.prototype.login for Android');
                try {
                    PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'login', [platform]);
                } catch(e) {
                    Paperlit.log.error('PaperlitCommands.prototype.login: failed calling phonegap ' + e.message);
                    errorCallback();
                }
            };
            
            PaperlitCommands.prototype.logout = function(platform, successCallback, errorCallback) {
                Paperlit.log.info('PaperlitCommands.prototype.logout for Android');
                try {
                    PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'logout', [platform]);
                } catch(e) {
                    Paperlit.log.error('PaperlitCommands.prototype.logout: failed calling phonegap ' + e.message);
                    errorCallback();
                }
            };
            
            PaperlitCommands.prototype.getLoginStatus = function(platform, successCallback, errorCallback) {
                Paperlit.log.info('PaperlitCommands.prototype.getLoginStatus for Android');
                try {
                    PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'getLoginStatus', [platform]);
                } catch(e) {
                    Paperlit.log.error('PaperlitCommands.prototype.getLoginStatus: failed calling phonegap ' + e.message);
                    errorCallback();
                }
            };
            
            PaperlitCommands.prototype.inviteFriends = function(platform, successCallback, errorCallback) {
                Paperlit.log.info('PaperlitCommands.prototype.inviteFriends for Android');
                try {
                    PhoneGap.exec(successCallback, errorCallback, 'paperlit', 'inviteFriends', [platform]);
                } catch(e) {
                    Paperlit.log.error('PaperlitCommands.prototype.inviteFriends: failed calling phonegap ' + e.message);
                    errorCallback();
                }
            };
            
           
            //Creating plugin for Phonegap
            PhoneGap.addConstructor( function() {
                // Register the javascript plugin with PhoneGap
                PhoneGap.addPlugin('paperlit', new PaperlitCommands());

                //Register the native class of plugin with PhoneGap (n.b. done in native client)
                //PluginManager.addService("paperlit", "com.paperlit.reader.activity.PPWebActivityPlugin");
            });
            /** Returns true if a specific method is supported by the native client. */
            Paperlit.isMethodSupported = function(methodName, successCallback, errorCallback) {
                Paperlit.log.info('Paperlit.isMethodSupported for Android ');
                errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
                window.plugins.paperlit.isMethodSupported(methodName, successCallback, errorCallback);
            };
           
              /** Returns the list of publication/issues cached by the application (available also offline). */
            Paperlit.getArchivedIssues = function(successCallback, errorCallback) {
                errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
                try {
					Paperlit.log.info('Paperlit.getArchivedIssues for Android ');
                    window.plugins.paperlit.getArchivedIssues(successCallback, errorCallback);
                } catch(e) {
					Paperlit.log.error('Paperlit.getArchivedIssues: failed calling phonegap ' + e.message);
                    errorCallback();
                }
            };
            /** Delete an issue from the application cache. */
            Paperlit.deleteArchivedIssue = function(publicationId, issueId, successCallback, errorCallback) {
                errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
                try {
					Paperlit.log.info('Paperlit.deleteArchivedIssue for Android');
                    window.plugins.paperlit.deleteArchivedIssue(publicationId, issueId, successCallback, errorCallback);
                } catch(e) {
					Paperlit.log.error('Paperlit.deleteArchivedIssue: failed calling phonegap ' + e.message);
                    errorCallback();
                }
            };
            /**
             * Returns the cached content of the given url as a file:// url.
             * If the file in cache is older than maxAgeInSeconds it will be
             * reloaded from the network. If maxAgeInSeconds is 0 content will
             * always be refreshed from the network. If maxAgeInSeconds is a
             * really large value (eg: 10 years) you will always get the latest
             * content from the local cache.
             */
            Paperlit._getCachedUrl = function(url, maxAgeInSeconds, successCallback, errorCallback) {
                errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
                if(Paperlit.isRunningLocally === false){
                    // cannot read cache from remote page
                    errorCallback();
                    return;
                }
                try {
                    Paperlit.log.info('Paperlit._getCachedUrl for Android ');
                    if (Paperlit.isLocalUrl(url)){
                        // client < 8.1.0 need to bypass webview cache adding a timestamp
                        if(typeof Paperlit.state === 'undefined' || Paperlit.utils.isVersionSupported(Paperlit.state.bundleVersion, '8.1.0') === false){
                            if(Paperlit.state.bundleId.indexOf('gazzetta') == -1 && Paperlit.state.bundleId.indexOf('rcs') == -1){
                            	// add timestamp to avoid webview caching
                            	url = url + '?ts=' + (new Date).getTime();
                            }
                        }
                        // url is local, nothing to cache
                        successCallback(url);
                        return;
                    }
                    // avoid browser caching
                    var successF = function(url){
                        // client < 8.1.0 need to bypass webview cache adding a timestamp
                        if(typeof Paperlit.state === 'undefined' || Paperlit.utils.isVersionSupported(Paperlit.state.bundleVersion, '8.1.0') === false){
                        	if(Paperlit.state.bundleId.indexOf('gazzetta') == -1 && Paperlit.state.bundleId.indexOf('rcs') == -1){
                            	// add timestamp to avoid webview caching
                            	url = url + '?ts=' + (new Date).getTime();
                            }
                        }
                        
                        successCallback(url);
                    }
                    window.plugins.paperlit.getCachedUrl(url, maxAgeInSeconds, successF, errorCallback);
                } catch(e) {
                    Paperlit.log.error('Paperlit.getCachedUrl: failed calling phonegap ' + e.message);
                }
            };
            /**
             * Same as GetCachedUrl but the actual content is returned as a string.
             */
            /*Paperlit.getCachedUrlAsString = function(url, maxAgeInSeconds, successCallback, errorCallback) {

                errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
                try {
                    Paperlit.log.info('Paperlit.getCachedUrlAsString for Android ');
                    if (url.indexOf('file:') != -1 || url.indexOf('/') == 0 || url.indexOf('.') == 0){
                        // url is local, nothing to cache
                        Paperlit.getAsyncUrl(url, successCallback, errorCallback)
                        return;
                    }
                    window.plugins.paperlit.getCachedUrlAsString(url, maxAgeInSeconds, successCallback, errorCallback);
                } catch(e) {
                    Paperlit.log.error('Paperlit.getCachedUrl: failed calling phonegap ' + e.message);
                }

            };*/
            
            /**
             * Uses getCachedUrl and then load the content of the file through an ajax call.
             */
            Paperlit.getCachedUrlAsString = function(url, maxAgeInSeconds, successCallback, errorCallback) {

                //load file
                Paperlit.getCachedUrl(url, maxAgeInSeconds, function(cachedUrl) {
                    try {
                        Paperlit.getAsyncUrl(cachedUrl, successCallback, errorCallback);
                    } catch(e) {
                        errorCallback()
                    }
                }, errorCallback);
            };
            /**	Asks for application state. Response is a JSON string with the keys:
             "uniqueIdentifier",  device unique identifier
             "name",              device name
             "systemName",        device system name
             "systemVersion",     device system version
             "model",             device model
             "localizedModel",    device model as a localized string
             "bundleId",          app bundle id
             "hash",              repubblica hash code
             "bundleVersion",     app bundle version
             "publicationId",     publication id of issue currently being read
             "issueId",           issue id of issue currently being read
             "pageId",            current page
             "productId",         id of product that was last purchased
             "email"              contents of email field in settings
             **/
            Paperlit.getState = function(successCallback, errorCallback) {
                try {
                    Paperlit.log.info('Paperlit.getState for Android ');
                    window.plugins.paperlit.getState(successCallback, errorCallback);
                } catch(e) {
                    Paperlit.log.error('Paperlit.getState: failed ' + e.message);
                    errorCallback();
                }
            };
            //ask native client to open the given url
            Paperlit._openUrl = function(url, successCallback, errorCallback) {
                if (successCallback == null)
                    successCallback = function() {
                    };
                if (errorCallback == null)
                    errorCallback = function() {
                    };
                Paperlit.log.info('Paperlit._openUrl for Android ');
                try {
                    window.plugins.paperlit.openUrl(url, successCallback, errorCallback);
                } catch(e) {
                    Paperlit.log.error('Paperlit._openUrl for Android: failed ' + e.message);
                    errorCallback();
                }
            };
            
            Paperlit.checkIfCommandIsSupported('getIssueMetadata', function(){
                 /** Returns the XML metadata of the required issue. The metadata is downloaded by the client with the issue and stored in the local cache*/
                    Paperlit.getIssueMetadata = function(publicationId, issueId, successCallback, errorCallback){
                        try {
                            Paperlit.log.info('Paperlit.getIssueMetadata for Android');
                            errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
                            var callback = function(cachedUrl){
                                Paperlit.getCachedUrlAsXML(cachedUrl, 0, successCallback, errorCallback);
                            }
                            window.plugins.paperlit.getIssueMetadata(publicationId, issueId, callback, errorCallback);
                        } catch(e) {
                            Paperlit.log.error('Paperlit.getMetadata for Android: error calling PaperlitCommands.getIssueMetadata');
                        }
                    };

                    /* The logic of this method is supported  directly into Paperlit.getIssueMetadata on Android. no age is available*/
                    Paperlit.getCachedIssueMetadata = function(publicationId, issueId, maxAgeInSeconds, successCallback, errorCallback){
                    	Paperlit.getIssueMetadata(publicationId, issueId, successCallback, errorCallback);
                    };

            }, Paperlit.noop);
            
            Paperlit.checkIfCommandIsSupported('getCachedPageUrl', function(){
                
                    Paperlit.getCachedPageUrl = function(issue, pageId, successCallback, errorCallback, width, height){
                        var url = Paperlit.getServerPageUrl(issue, pageId, width, height);
                        Paperlit.getCachedUrl(url, 60*60*24*30*12, function(cachedUrl) {successCallback({url: cachedUrl})}, 
                            function(){
                                // in case of failure, try with native rendering
                                Paperlit._getCachedPageUrl(issue, pageId, successCallback, errorCallback, width, height);
                            }
                        );
                    };
                    
                    /** Returns url of a cached issue page image */
                    Paperlit.queue.getCachedPageUrl = [];
                    
                    /** as image request are processed using a LIFO in Android, we want to make batch of requests in reverse order in order to have theme processed as a FIFO */
                    Paperlit._getCachedPageUrl = function(issue, pageId, successCallback, errorCallback, width, height){
                        try {
                            var funk = function() {
                                 Paperlit.requireGetCachedPageUrl(issue, pageId, successCallback, errorCallback, width, height);
                            };
                            if(Paperlit.queue.getCachedPageUrl.length == 0){
                                // when inserting the first element in the queue, schedule the queue processing first time we are idle
                                setTimeout(Paperlit.processGetCachedPageUrlQueue, 1);
                            }
                            // enqueue the request
                            Paperlit.queue.getCachedPageUrl.push(funk);
                        } catch(e) {
                            Paperlit.log.error('Paperlit.getCachedPageUrl for Android: error calling PaperlitCommands.getCachedPageUrl');
                        }
                    };
                 
                    /** actual implementation of the getCachedPageUrl request **/
                    Paperlit.requireGetCachedPageUrl = function(issue, pageId, successCallback, errorCallback, width, height){
                        try {
                            var devicePixelRatio = (typeof window.devicePixelRatio != "undefined") ?  window.devicePixelRatio : 1;
                            
                            var page = { url : "", width: 0, height: 0};
                            if (typeof width != 'undefined' && width != null) {
                                page.width = Math.round(width * devicePixelRatio);
                            }
                            if (typeof height  != 'undefined' && height != null) {
                                page.height = Math.round(height * devicePixelRatio);
                            }
                            
                            // callback for success
                            var callback = function(cachedUrl){
                                //alert('cached image');
                                page.url = cachedUrl; 
                                successCallback(page);
                            }
                            
                            var defaultErrorCallback = function(error) {
                                setTimeout(function(){
                                    //if((typeof navigator.connection != 'undefined' && typeof navigator.connection.type != 'undefined' && navigator.connection.type != 0) || (typeof navigator.connection.type == 'undefined'  && (typeof navigator.onLine == 'undefined' || navigator.onLine))){
                                        //alert('I am online. Retry request getCachedUrl: ' + url);
                                        Paperlit._getCachedPageUrl(issue, pageId, callback, defaultErrorCallback, width, height);
                                    //}
                                    //else{
                                        // reschedule
                                    //    defaultErrorCallback();
                                    //}
                                }, 5000);
                            };
                            //create an error callback that will try again to load
                            errorCallback = errorCallback != null ? errorCallback : defaultErrorCallback;
                            window.plugins.paperlit.getCachedPageUrl(issue.publicationId, issue.issueId, pageId, page.width, page.height, callback, errorCallback);
                        } catch(e) {
                            Paperlit.log.error('Paperlit.requireGetCachedPageUrl for Android: error calling PaperlitCommands.requireGetCachedPageUrl');
                        }
                    };
                    
                    
                    /**
                     *** Process the queue of getCachedPageUrl commands
                    **/
                    Paperlit.processGetCachedPageUrlQueue = function() {
                        Paperlit.log.info('processGetCachedPageUrlQueue: processing ' + Paperlit.queue.getCachedPageUrl.length + ' commands');
                        var funk;
                        // process the requests in reverse order
                        while(Paperlit.queue.getCachedPageUrl.length > 0) {
                            funk = Paperlit.queue.getCachedPageUrl.pop();
                            setTimeout(funk,0);
                        }
                    };
                    
            }, Paperlit.noop);
			
			if(Paperlit.isCordova){
                Paperlit.log.info('Using Paperlit.getProductPrice cordova for Android');
                Paperlit.getProductPrice = function(productId, successCallback, errorCallback) {
                    errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
                    try{
                        Paperlit.log.info('Paperlit.getProductPrice cordova for Android')
                        cordova.exec(successCallback, errorCallback, 'paperlit', 'getProductPrice', [productId]);
                    }
                    catch(e){
                        Paperlit.log.error('Error calling Paperlit.getProductPrice cordova for android: ' + e.message);
                    }
                }
            }
            else{
                Paperlit.log.info('Check if gap/getProductPrice is supported')
    			Paperlit.checkIfCommandIsSupported('gap/getProductPrice', function(){
    				//alert('gap/getProductPrice is supported by native client')
                    // get Product price uses Paperlit gap interface
    				Paperlit.getProductPrice = function(productId, successCallback, errorCallback) {
    					errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
    					try {
    						//alert('Paperlit.getProductPrice for Android: '+productId);
    						var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
    						// send command 
    						Paperlit.gap.exec('getProductPrice/' + productId + '/' + functionsName.successFuncName + '/' + functionsName.errorFuncName);
    					} catch(e) {
    						//alert('exception calling gap/getProductPrice: '+productId);
    						errorCallback();
    					}

    				};
                }, function(){
    				//alert('gap/getProductPrice not supported by native client')
    				Paperlit.getProductPrice = function(productId, successCallback, errorCallback){
    					// not supported
    					setTimeout(errorCallback, 1);
    				}
    			});
            }
            
            Paperlit.trackEvent = function() {
                Paperlit.log.info('Paperlit.trackEvent for Android');
                try {
                    if (arguments.length < 3){
                        throw "missing arguments";
                    }
                    var args = Array.prototype.slice.call(arguments);
                    // remove callbacks
                    var errorCallback = args.pop();
                    var successCallback = args.pop();
        
                    window.plugins.paperlit.trackEvent(args, successCallback, errorCallback);
                    
                } catch(e) {
                    Paperlit.log.error('Paperlit.trackEvent for Android: failed ' + e.message);
                    errorCallback();
                }
            };
            
            
            Paperlit.social.login = function(platform, successCallback, errorCallback){
                try {
                    Paperlit.log.info('Paperlit.login for Android ');
                    window.plugins.paperlit.login(platform, successCallback, errorCallback);
                } catch(e) {
                    Paperlit.log.error('Paperlit.login: failed ' + e.message);
                    errorCallback();
                }
            };
            
            Paperlit.social.logout = function(platform, successCallback, errorCallback){
                try {
                    Paperlit.log.info('Paperlit.logout for Android ');
                    window.plugins.paperlit.logout(platform, successCallback, errorCallback);
                } catch(e) {
                    Paperlit.log.error('Paperlit.logout: failed ' + e.message);
                    errorCallback();
                }
            };
            
            Paperlit.social.getLoginStatus = function(platform, successCallback, errorCallback){
                try {
                    Paperlit.log.info('Paperlit.getLoginStatus for Android ');
                    window.plugins.paperlit.getLoginStatus(platform, successCallback, errorCallback);
                } catch(e) {
                    Paperlit.log.error('Paperlit.getLoginStatus: failed ' + e.message);
                    errorCallback();
                }
            };
            
            Paperlit.social.inviteFriends = function(platform, successCallback, errorCallback){
                try {
                    Paperlit.log.info('Paperlit.inviteFriends for Android ');
                    window.plugins.paperlit.inviteFriends(platform, successCallback, errorCallback);
                } catch(e) {
                    Paperlit.log.error('Paperlit.inviteFriends: failed ' + e.message);
                    errorCallback();
                }
            };
            
            if(Paperlit.isCordova){
                Paperlit.log.info('Using Paperlit.social.share cordova for Android');
                Paperlit.social.share = function(platform, url, successCallback, errorCallback) {
                    errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
                    try {
                        Paperlit.log.info('Paperlit.social.share cordova for android')
                        cordova.exec(successCallback, errorCallback, 'paperlit', 'share', [platform, url])
                    } catch(e) {
                        Paperlit.log.error('Error calling Paperlit.social.share cordova for android: ' + e.message);
                        errorCallback();
                    }
                };
            }
            else{
                Paperlit.log.info('Check if gap/share is supported');
                /* share command is invoked using gap interface because it must be retrieven also from cross domain page*/
                Paperlit.checkIfCommandIsSupported('gap/share', function(){
                    // get Product price uses Paperlit gap interface
    				Paperlit.social.share = function(platform, url, successCallback, errorCallback) {
    					errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
    					try {
    						var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
    						// send command 
    						Paperlit.gap.exec('share/' + platform + '/' + encodeURIComponent(url) + '/' + functionsName.successFuncName + '/' + functionsName.errorFuncName);
    					} catch(e) {
    						errorCallback();
    					}

    				};
                }, function(){
    				Paperlit.social.share = function(platform, url, successCallback, errorCallback){
    					// not supported
    					setTimeout(errorCallback, 1);
    				}
    			});
			}

			if(Paperlit.isGap){
				// new methods used with gap protocol
				Paperlit.readFile = function(fileName, successCallback, errorCallback) {
					try {
			            errorCallback = typeof errorCallback != 'undefined' ? errorCallback : Paperlit.noop;
			            Paperlit.log.info('Paperlit.readFile');
			            cordova.exec(successCallback, errorCallback, 'paperlit', 'readFile', [fileName]);
			        } catch(e) {
			            Paperlit.log.error('exception calling PaperlitCommands.readFile: '+ e.message);
			            errorCallback();
			        }
				};

				Paperlit.writeFile = function(text, successCallback, errorCallback, fileName) {
					try {
			            errorCallback = typeof errorCallback != 'undefined' ? errorCallback : Paperlit.noop;
			            Paperlit.log.info('Paperlit.writeFile');
			            cordova.exec(successCallback, errorCallback, 'paperlit', 'writeFile', [fileName, text]);
			        } catch(e) {
			            Paperlit.log.error('exception calling PaperlitCommands.writeFile: '+ e.message);
			            errorCallback();
			        }
				};
			}

            Paperlit.getState(function(state){
                Paperlit.state = state;
                // set properties file path to bundle root
                Paperlit.propertiesFileName = '/data/data/' + state.bundleId + '/properties.pp5';
                Paperlit.processSupportedCommandsQueue();
            }, Paperlit.processSupportedCommandsQueue);
        }   // end Android
        
    } catch(e) {
        Paperlit.log.error('Paperlit.onDeviceReady failed ' + e.message);
    }
};


// support paperlit://gap commands
			
// allow for cross-domain communication without phonegap on Android
Paperlit.gap = {
	commands: [],
	ready: true,
	timer: null
};

Paperlit.gap.run_command = function () {
	// native client is still retrieving previous call
	if (Paperlit.gap.ready == false) {
		return;
	}
	Paperlit.gap.ready = false;

	var command = Paperlit.gap.commands.shift();
	if (Paperlit.gap.commands.length == 0) {
		clearInterval(Paperlit.gap.timer);
		Paperlit.gap.timer = null
		//console.log('Paperlit.gap.run_command: no more command in queue, reset timer');
	}
	//console.log('Paperlit.gap.run_command: calling ' + 'paperlit://gap/' + command);
	document.location = 'paperlit://gap/' + command;
};

Paperlit.gap.exec = function (command) {
	console.log('Paperlit.gap.exec: received command ' + command);
	//alert('phone gap exec: '+arguments);
	if(command.indexOf('openUrl') == 0){
		// give precedence to open url commands
		Paperlit.gap.commands.unshift(command);
	}
	else{
		Paperlit.gap.commands.push(command);
	}
	if (Paperlit.gap.timer == null) {
		//console.log('Paperlit.gap.run_command: set timer');
		Paperlit.gap.timer = setInterval(Paperlit.gap.run_command, 10);
	}
};

Paperlit.start = function() {
    try {
    	//process any commands in queue
        Paperlit.processCommandsQueue();
        // trigger ppReadyEvent
        Paperlit.triggerReady();
    } catch(e) {
        Paperlit.log.error('Paperlit.start failed ' + e.message);
    }
};
/********************************************************************************************************************
 *** COMMONS
 *** Some may be overloaded by platform specific implementation when the device ready event is triggered by phonegap
 *********************************************************************************************************************/

/**
 * Async call to get url content
 */

Paperlit._getAsyncUrl = function(xmlhttp, url, successCallback, errorCallback) {
    return function() {
        //var isLocal = url.indexOf('file://') != -1 || url.indexOf('/') == 0 || url.indexOf('.') == 0;
        if (xmlhttp.readyState == 4) {
            xmlhttp.onreadystatechange = null;
            //for local file status is 0
            if (xmlhttp.status >= 200 && xmlhttp.status < 300 || xmlhttp.status === 304 || (xmlhttp.status == 0 && xmlhttp.responseText != '')) {
                /*setTimeout(function(response){
                    alert('success ' + url + ', response: ' + response);
                }, 0, xmlhttp.responseText);*/
                successCallback(xmlhttp.responseText);
            } else {
                /*setTimeout(function(status, response){
                    alert('failed load async for ' + url + ', status ' + status + ', response: ' + response);
                }, 0, xmlhttp.status, xmlhttp.responseText)*/
                errorCallback != null ? errorCallback() : Paperlit.noop();
            }
            // destroy
            xmlhttp = null;
        }
    }
}

Paperlit.getAsyncUrl = function(url, successCallback, errorCallback) {
    try {
        var xmlhttp;
        if (window.XMLHttpRequest) {
            // code for IE7+, Firefox, Chrome, Opera, Safari
            xmlhttp = new XMLHttpRequest();
        } else {// code for IE6, IE5
            xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
        }
        //xmlhttp.isLocal = url.indexOf('file://') != -1 || url.indexOf('/') == 0 || url.indexOf('.') == 0;
        xmlhttp.onreadystatechange = Paperlit._getAsyncUrl(xmlhttp, url, successCallback, errorCallback);
        //xmlhttpCollection[xmlhttpCollectionIndex++] = xmlhttp;
        /*setTimeout(function(){
            alert('loading async for ' + url);
        }, 0);*/
        xmlhttp.open("GET", url, true);
        xmlhttp.send();
    } catch(e) {
        Paperlit.log.error('Paperlit.getAsyncUrl ' + e.message);
    }
};

Paperlit.getSyncUrl = function(url, successCallback, errorCallback) {
    var xmlhttp = new XMLHttpRequest();
    xmlhttp.open('GET', url, false);  // false makes the request synchronous
    xmlhttp.send(null);
    try{
        if (xmlhttp.status >= 200 && xmlhttp.status < 300 || xmlhttp.status === 304 || (xmlhttp.status == 0)) {
            successCallback(xmlhttp.responseText);
        }
        else{
            errorCallback ? errorCallback() : Paperlit.noop();
        }
    }
    catch(e){

    }
    finally{
        xmlhttp = null;
    }
}

Paperlit.getArchivedIssues = function(successCallback, errorCallback) {
    errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
    errorCallback();
};
/**
 ** Proxy urls if running under paperlit.com or localhost
 */
Paperlit.proxyUrl = function(url, maxAgeInSeconds){	return url;
    maxAgeInSeconds = typeof maxAgeInSeconds === 'undefined' ? 0 : maxAgeInSeconds;
    //if (url.indexOf('%') == -1){
        url = encodeURIComponent(url);
    //}
	var baseDomain = window.location.href.split(/\/+/g)[1];
	if (baseDomain.indexOf('paperlit.com') != -1 || baseDomain.indexOf('localhost')  != -1 || baseDomain.indexOf('192.168.') != -1){
		return 'http://'+ baseDomain + '/php/services/proxy.php?ttl=' + maxAgeInSeconds + '&version=' + (new Date()).getTime() + '&url=' + url;
	}
	return url;
}

Paperlit._getCachedUrl = function(url, maxAgeInSeconds, successCallback, errorCallback) {
    // replace url parameters
    url = Paperlit.replaceUrlParametersSync(url);
    if(Paperlit.isLocalUrl(url)){
        setTimeout(function(){
            // add timestamp to avoid webview caching
            successCallback(url + '?ts=' + (new Date).getTime());
        }, 1);
    }
    else{
        // load via proxy
        setTimeout(successCallback, 1, Paperlit.proxyUrl(url, maxAgeInSeconds));
    }
}
/**
 ** Returns a url to a local proxy
 */
Paperlit.getCachedUrl = function(url, maxAgeInSeconds, successCallback, errorCallback) {
    try {
    	Paperlit.log.info('Paperlit.getCachedUrl generic: url ' + url + ' age ' + maxAgeInSeconds);
        Paperlit.handleSpecialUrls(url, function(url, parameters){
        	Paperlit.log.info('Paperlit.getCachedUrl generic: url after special handling ' + url);
            if(typeof parameters !== 'undefined' && typeof parameters.contentUrl !== 'undefined'){
            	Paperlit.log.info('Paperlit.getCachedUrl generic: caching content ' + parameters.contentUrl);
                // cache content
                Paperlit._getCachedUrl(parameters.contentUrl, maxAgeInSeconds, successCallback, errorCallback);
            }
            else{
            	Paperlit.log.info('Paperlit.getCachedUrl generic: caching url ' + url );
                Paperlit._getCachedUrl(url, maxAgeInSeconds, successCallback, errorCallback);    
            }
        }, errorCallback);
    } catch(e) {
        Paperlit.log.error('Paperlit.getCachedUrl generic: exception trying to run through proxy ' + e.message);
        Paperlit.noop();
    }
};
/**
 * Same as GetCachedUrl but the actual content is returned as a string.
 */
Paperlit.getCachedUrlAsString = function(url, maxAgeInSeconds, successCallback, errorCallback, proxy) {
    try {
    	proxy = typeof proxy === 'undefined' ? true : proxy;
        // replace url parameters
        url = Paperlit.replaceUrlParametersSync(url);
        if(proxy){
	        if(Paperlit.isLocalUrl(url) === false){
	            url = Paperlit.proxyUrl(url);
	        }
	        else{
	            url = url + '?ts=' + (new Date).getTime();
	        }
	    }
        Paperlit.getAsyncUrl(url, successCallback, errorCallback)
    } catch(e) {
        Paperlit.log.error('PaperlitCommands.getCachedUrlAsString: exception trying to run through proxy ' + e.message);
        Paperlit.noop();
    }

};
/**
 * Same as getCachedUrlAsString but the actual content is returned as a base64 string of the specified mime type.
 * This is useful if an external page wants to cache some data on the native client without inucurring in cross-domain issues.
 * Local files cannot be loaded by external page (i.e., getCachedUrl cannot be use in this case)
 */
Paperlit.getCachedUrlAsBase64 = function(url, maxAgeInSeconds, mimeType, successCallback, errorCallback) {
    try {
        Paperlit.log.info('Paperlit.getCachedUrlAsBase64 for browser asked for url: '+url);
        url = 'http://www.paperlit.com/php/services/base64.php?url=' + encodeURIComponent(url)
        if (mimeType) {
            url = url +'&amp;mime=' + encodeURIComponent(mimeType);
        }

        Paperlit.getCachedUrlAsString(url, maxAgeInSeconds, successCallback, errorCallback);

    } catch(e) {
        Paperlit.log.error('Paperlit.getCachedUrlAsBase64 : failed '+e.message);
    }

};
/**
 * Register a callback on the native client that will be called when the web view goes on focus
 */


    // successCallback();
/**
 * Called by the native client when the webview is going to sleep
 */
Paperlit.pauseActivity = function() {
	//clear any scheduled resume activity
	// we don't need to wake up during sleeping
	if (typeof Paperlit.resumeTimeout != 'undefined' && Paperlit.resumeTimeout){
		clearTimeout(Paperlit.resumeTimeout);
		Paperlit.resumeTimeout = null;
	}
	
    var evObj = document.createEvent('Events');
    evObj.initEvent('ppPause',true,true);
    document.dispatchEvent(evObj);

    evObj = document.createEvent('Events');
    evObj.initEvent('ppOnPause', true, true);
    document.dispatchEvent(evObj);
};

/**
** Fire a ppResume event that will be used to refresh contents
**/
Paperlit.fireResume = function(){
    var evObj = document.createEvent('Events');
    evObj.initEvent('ppResume',true,true);
    document.dispatchEvent(evObj);

    evObj = document.createEvent('Events');
    evObj.initEvent('ppOnResume', true, true);
    document.dispatchEvent(evObj);
}
/**
** This is called by native client each time we are put on focus
** As in some occasion this can be called many times in a short time we need some mechanism to 
** avoid refreshing too much
**/
Paperlit.resumeActivity = function() {
	var evObj = document.createEvent('Events');
    evObj.initEvent('ppResumeRequest',true,true);
    document.dispatchEvent(evObj);
	//if resume has already been scheduled, clear the schedule
	if (typeof Paperlit.resumeTimeout != 'undefined' && Paperlit.resumeTimeout){
		clearTimeout(Paperlit.resumeTimeout);
	}
	//schedule resume with some delay to avoid quick fires
	Paperlit.resumeTimeout = setTimeout(Paperlit.fireResume, 50);
}

/**
 * Called by the native client when the webview is starting to rotate
 */
Paperlit.startingDeviceRotation = function() {
    //alert('startingDeviceRotation: dispatching event');
    var evObj = document.createEvent('Events');
    evObj.initEvent('ppStartRotation',true,true);
    document.dispatchEvent(evObj);
};

window.addEventListener('focus',Paperlit.resumeActivity, false);


Paperlit.getBrowserState = function(){
    return {
        uniqueIdentifier: Paperlit.utils.getUrlParameter('uniqueIdentifier') || Paperlit.utils.getUrlParameter('deviceId'),
        deviceId: Paperlit.utils.getUrlParameter('uniqueIdentifier') || Paperlit.utils.getUrlParameter('deviceId'),
        name: Paperlit.utils.getUrlParameter('name'),
        systemName: Paperlit.utils.getUrlParameter('systemName'),
        systemVersion: Paperlit.utils.getUrlParameter('systemVersion'),
        model: Paperlit.utils.getUrlParameter('model'),
        localizedModel: Paperlit.utils.getUrlParameter('localizedModel'),
        bundleId: Paperlit.utils.getUrlParameter('bundleId'),
        bundleVersion: Paperlit.utils.getUrlParameter('bundleVersion'),
        hash: Paperlit.utils.getUrlParameter('hash'),
        publicationId: Paperlit.utils.getUrlParameter('publicationId'),
        issueId: Paperlit.utils.getUrlParameter('issueId'),
        pageId: Paperlit.utils.getUrlParameter('pageId'),
        productId: Paperlit.utils.getUrlParameter('productId'),
        email: Paperlit.utils.getUrlParameter('email'),
        publicationName: Paperlit.utils.getUrlParameter('publicationName'),
        issueName: Paperlit.utils.getUrlParameter('issueName'),
        folioVersion: Paperlit.utils.getUrlParameter('folioVersion'),
        screenWidth: window.innerWidth,
        screenHeight: window.innerHeight,
        randomNumber: '1' + (Math.floor(Math.random() * (999999999 - 100000000 + 1)) + 100000000),
        deviceClass : window.innerWidth <= 480 ? 'phone' : 'tablet',
        userId : 'pippo',
        userEmail : 'pippo@pluto.com'
    };
}

/**	Asks for application state. Response is a JSON string with the keys:
 "uniqueIdentifier",  device unique identifier
 "name",              device name
 "systemName",        device system name
 "systemVersion",     device system version
 "model",             device model
 "localizedModel",    device model as a localized string
 "bundleId",          app bundle id
 "hash",              repubblica hash code
 "bundleVersion",     app bundle version
 "publicationId",     publication id of issue currently being read
 "issueId",           issue id of issue currently being read
 "pageId",            current page
 "productId",         id of product that was last purchased
 "email"              contents of email field in settings
 **/
Paperlit.getState = function(successCallback, errorCallback) {

    Paperlit.log.info('Paperlit.getState for browser ');
    try {
        Paperlit.log.info('Paperlit.getState for browser ');
        // state is configurable using url parameters
        setTimeout(function() {successCallback(Paperlit.getBrowserState());
        }, 1);

    } catch(e) {
        Paperlit.log.error('Paperlit.getState: failed ' + e.message);
        errorCallback();
    }
};

/* Trasform special urls, return urls and a dictionary of parameters */
Paperlit.handleSpecialUrls = function(url, successCallback, errorCallback){
    if(url.indexOf('paperlit://template/') != -1){
    	Paperlit.log.info('Paperlit.handleSpecialUrls: detected template');
        // this url maps to a template e.g. paperlit://template/article?id=[id articolo]
        var parametersSectionBeginIndex = url.indexOf('?');
        var query = '';
        var template = url;

        if(parametersSectionBeginIndex != -1){
            query = url.substring(parametersSectionBeginIndex);
            template = template.substring(0, parametersSectionBeginIndex);
        }
        // removing prefix
        template = template.replace('paperlit://template/', '');
        
            
		var id = Paperlit.utils.getParameter(query, 'id');
		// remove square brackets if any
		id = id.replace(/\[/g, '');
		id = id.replace(/\]/g, '');
		if(!id){
			errorCallback();
			return;
		}
		Paperlit.log.info('Paperlit.handleSpecialUrls: template ' + template + ' id ' + id);
		// open article template loading content
		Paperlit.getAppConfiguration(function(configuration){
			var url;
			var contentUrl = '';
			// FIXED URL
			// /21944/fcs.quotidiani/mediacenter/mysynd/data/gazzettaDE/contents/123456.zip
			contentUrl = Paperlit.utils.getSetting(configuration, 'newsstand-contents-url');
			// replaceId
			contentUrl = contentUrl.replace(/{id}/g, id);

			Paperlit.log.info('Paperlit.handleSpecialUrls: content url ' + contentUrl);
			// templates are unzipped in templates folder. Reply uses html folder as root of templates
			if(template === 'storia'){
				// open .zip content with browser type panels
				url = contentUrl + '#ppbt=panels';
				successCallback(url, {});
				return;
			}
			
			url = 'file://templates/html/' + template + '/' + template + '.html?hwacceleration=true#ppbt=header&ppnav=secondary&bounce=false';
			
			if(contentUrl){
				var parameters = {'contentUrl': contentUrl, customParameters : 'filename:' + id + '.xml,xpath://' + template.toUpperCase()};
				Paperlit.log.info('opening ' + url + ' with content url = ' + contentUrl + ' id= ' + id);
				successCallback(url, parameters);
			}
			else{
				errorCallback();
			}
		}, errorCallback);

        return;
    }
    else if(url.indexOf('paperlit://openlastissue/') != -1){
    	var publicationId = url.replace('paperlit://openlastissue/', '');;
    	Paperlit.getOpenLastIssueUrl(publicationId, function(url){
    		successCallback(url);
    	});
    	return;
    }
    else if(url.indexOf('paperlit://billing-login') != -1){
    	Paperlit.getAppConfiguration(function(configuration){
			var billingLoginUrl = null;
	    	billingLoginUrl = Paperlit.utils.getSetting(configuration, 'billing-login-url');
	    	if(billingLoginUrl != null){
	    		// append query string to default url
	    		successCallback(Paperlit.copyQueryString(url, billingLoginUrl));
	    	}
	    });
	    return;
    }
    else if(url.indexOf('paperlit://billing-mpay') != -1){
    	Paperlit.getAppConfiguration(function(configuration){
			var billingMpay = null;
	    	billingMpay = Paperlit.utils.getSetting(configuration, 'billing-mpay-url');
	    	if(billingMpay != null){
	    		// append query string to default url
	    		successCallback(Paperlit.copyQueryString(url, billingMpay));
	    	}
	    });
	    return;
    }
    else if(url.indexOf('paperlit://billing-creditcard') != -1){
    	Paperlit.getAppConfiguration(function(configuration){
			var billingCreditCard = null;
	    	billingCreditCard = Paperlit.utils.getSetting(configuration, 'billing-creditcard-url');
	    	if(billingCreditCard != null){
	    		// append query string to default url
	    		successCallback(Paperlit.copyQueryString(url, billingCreditCard));
	    	}
	    });
	    return;
    }
    successCallback(url);
};

// copy query string of url1 into url 2
Paperlit.copyQueryString = function(url1, url2){
	var parametersSectionBeginIndex = url1.indexOf('?');
	var hashSectionBeginIndex = url1.indexOf('#');
	var query = '';
    if(parametersSectionBeginIndex != -1){
    	parametersSectionBeginIndex++;
    	if(hashSectionBeginIndex != -1){
    		query = url1.substring(parametersSectionBeginIndex, hashSectionBeginIndex);
    	}
    	else{
    		query = url1.substring(parametersSectionBeginIndex);
    	}
    }

    if(query != ''){
    	var queryParameters = {};
    	var url1QueryParameters = query.split('&');
    	// copy url 1 parameters
   		for (var i=0, l=url1QueryParameters.length; i<l; i++){
			var parameter = url1QueryParameters[i].split('=');
			queryParameters[parameter[0]] = decodeURIComponent(parameter[1]);
		}
    	
    	parametersSectionBeginIndex = url2.indexOf('?');
		hashSectionBeginIndex = url2.indexOf('#');
		query = '';
    	if(parametersSectionBeginIndex != -1){
    		parametersSectionBeginIndex++;
	    	if(hashSectionBeginIndex != -1){
	    		query = url2.substring(parametersSectionBeginIndex, hashSectionBeginIndex);
	    	}
	    	else{
	    		query = url2.substring(parametersSectionBeginIndex);
	    	}
	    }
	    if(query != ''){
	    	var url2QueryParameters = query.split('&');
	    	// now append new query parameters to url 2
			for (var i=0, l=url2QueryParameters.length; i<l; i++){
				var parameter = url2QueryParameters[i].split('=');
				var parameterName = parameter[0];
				// add only if parameter name is not already present
				if(parameterName in queryParameters === false){
					queryParameters[parameterName] = decodeURIComponent(parameter[1]);
				}
			}
    	}
    	for (var key in queryParameters) {
		  	if (queryParameters.hasOwnProperty(key)) {
		    	url2 = Paperlit.utils.appendUrlParameter(url2, key, queryParameters[key]);
		  	}	
		}
    }
    return url2;
}

/* Get the real url to open. Some url need to be interpreted before they can be passed to openUrl*/
Paperlit.openUrl = function(url, onlyIfAuthorized, successCallback, errorCallback){
	//Paperlit.openUrl(url, premium)
	Paperlit.log.info('Paperlit.openUrl: required ' + url);
    try {
    	if(typeof onlyIfAuthorized == 'undefined'){
    		successCallback = Paperlit.noop;
    		errorCallback = Paperlit.noop;
    		onlyIfAuthorized = false;
    	}
    	else if (typeof onlyIfAuthorized == 'function'){
    		successCallback = typeof onlyIfAuthorized !== 'undefined' ? onlyIfAuthorized : Paperlit.noop;
        	errorCallback = typeof successCallback !== 'undefined' ? successCallback : Paperlit.noop;
        	onlyIfAuthorized = false;
    	}
    	else{
    		successCallback = typeof successCallback !== 'undefined' ? successCallback : Paperlit.noop;
        	errorCallback = typeof errorCallback !== 'undefined' ? errorCallback : Paperlit.noop;
    	}
    	Paperlit.log.info('Paperlit.openUrl: ' + url + ' only if authorized: ' + onlyIfAuthorized);

        Paperlit.handleSpecialUrls(url, function(url, parameters){
        	Paperlit.log.info('Paperlit.openUrl: after handleSpecialUrls: ' + url);
            if(!Paperlit.isNative){
                /*/ only in browser
                if (url.indexOf('paperlit://read/') != -1){
                    url = url.replace('paperlit://read/','');
                    var urlParts = url.split('/');
                    // open purchase option panel
                    url = 'http://api.paperlit.com/php/billing/subscriptions2.php?deviceId={uniqueIdentifier}&bundleId={bundleId}&bundleVersion={bundleVersion}&issueId=' + urlParts[1] + '&publicationId=' + urlParts[0] + '&read=Read&buy=Buy'
                }*/
                
                url = Paperlit.replaceUrlParametersSync(url);
                // add test paramter (default is false)
                var test = Paperlit.utils.getUrlParameter('test');
                if (!test){
                    test = 'false';
                }
                var parametersSectionBeginIndex = url.indexOf('?');
                if(parametersSectionBeginIndex == -1){
                    // no parameters
                    url = url + '?test=' + test;
                }
                else{
                    //insert test parameter as first parameter
                    url = url.substring(0, parametersSectionBeginIndex) + '?test=' + test + '&' + url.substring(parametersSectionBeginIndex + 1);
                }
            }
            // append parameters
            if(typeof parameters !== 'undefined' && parameters != null){
                for(var key in parameters){
                    url = Paperlit.utils.appendUrlParameter(url, key, parameters[key]);
                }
            }
            //alert('openUrl ' + url);
            if(onlyIfAuthorized){
            	Paperlit.log.info('Paperlit.openUrl: check if autorized for ' + url);
            	Paperlit.openUrlIfAuthorized(url, successCallback, errorCallback);
            }
            else{
            	Paperlit.log.info('Paperlit.openUrl: opening ' + url);
            	Paperlit._openUrl(url);
            }
        }, errorCallback);        
    } catch(e) {
        Paperlit.log.error('Paperlit.openUrl: failed ' + e.message);
        errorCallback();
    }
}
//browser
Paperlit._openUrl = function(url, successCallback, errorCallback) {
    Paperlit.log.info('Paperlit._openUrl for browser: ' + url);
    window.location.href = url;
};


Paperlit.openUrlIfAuthorized = function(url, successCallback, errorCallback){
	Paperlit.log.info('Paperlit.openUrlIfAuthorized: check for url ' + url);
	Paperlit.rcs.isSubscribed(function(data){
		if(data.result == 'success') {
			Paperlit.log.info('Paperlit.openUrlIfAuthorized: authorized for url ' + url);
			Paperlit.openUrl(url);
		}
		else{
			Paperlit.log.info('Paperlit.openUrlIfAuthorized: NOT authorized for url ' + url + ' opening paywall');
			// not authorized, open paywall
			Paperlit.getAppConfiguration(function(configuration){
               	// use feed-url
            	var paywallUrl = Paperlit.utils.getSetting(configuration, "billing-paywall-url");
            	Paperlit.openUrl(paywallUrl);
            });
		}
	});
}

/** Returns the XML metadata of the required issue.*/
Paperlit.getIssueMetadata = function(publicationId, issueId, successCallback, errorCallback){
    try {
        Paperlit.log.info('Paperlit.getIssueMetadata');
        Paperlit.getCachedIssueMetadata(publicationId, issueId, 0, successCallback, errorCallback);
    } catch(e) {
        Paperlit.log.error('Paperlit.getIssueMetadata: error');
    }
}

/** Returns the XML metadata of the required issue with a certain expiration.*/
Paperlit.getCachedIssueMetadata = function(publicationId, issueId, maxAgeInSeconds, successCallback, errorCallback){
    try {
        Paperlit.log.info('Paperlit.getCachedIssueMetadata');
        
        Paperlit.getCachedUrlAsXML('http://www.paperlit.com/api/'+publicationId+'/'+issueId+'/metadata.xml', maxAgeInSeconds, successCallback, errorCallback);
    } catch(e) {
        Paperlit.log.error('Paperlit.getCachedIssueMetadata: error');
    }
}

/**
 * Same as GetCachedUrlAsString but the actual content is returned as an object containing the JSON parsed from the string.
 */
Paperlit.jsonp2json  = function(jsonp) {
    return jsonp.substring(jsonp.indexOf('(') + 1, jsonp.lastIndexOf(')'));
}
Paperlit.getCachedUrlAsJSON = function(url, maxAgeInSeconds, successCallback, errorCallback, format) {

    Paperlit.getCachedUrlAsString(url, maxAgeInSeconds, function(response) {
        try {
            //try to parse
            //replace any new line character with a space (new line char is not allowed inside the json elements)
            var json = response.replace(new RegExp("\\n", "g"),' ');
            if (format == 'jsonp') {
                //blackbird.profile('JSONP response '+response);
                json =  JSON.parse(Paperlit.jsonp2json(json));
            } else {

                json = JSON.parse(json)
            }

            //callback
            successCallback(json);

        } catch(e) {
            Paperlit.log.error('Paperlit.getCachedUrlAsJSON : failed parsing response into JSON');
            //call errorCallback callback

            try {
                //try removing callback if jsonp
                // blackbird.profile('try removing callback if jsonp');

                //try to parse
                var json = JSON.parse(Paperlit.jsonp2json(response))
                //callback
                successCallback(json);
            } catch(e) {
                Paperlit.log.error('Paperlit.getCachedUrlAsJSON : failed parsing JSONP');
                errorCallback != null ? errorCallback() : Paperlit.noop();
            }
        }

    },errorCallback);
};
/**
 * Same as GetCachedUrlAsJson but uses getCachedUrl and then load the content of the file through an ajax call.
 */
Paperlit.getCachedUrlAsJSONFromFile = function(url, maxAgeInSeconds, successCallback, errorCallback) {

    var loadJson = function(response) {
        try {

            //try to parse
            var json = JSON.parse(response)
            //callback
            successCallback(json);
        } catch(e) {
            Paperlit.log.error('Paperlit.getCachedUrlAsJSONFromFile : failed parsing response into JSON');
            //call errorCallback callback

            try {
                //try removing callback if jsonp
                Paperlit.log.info('try removing callback if jsonp');
                response = response.substring(response.indexOf('(') + 1, response.length - 1)
                //try to parse
                var json = JSON.parse(response)
                //callback
                successCallback(json);
            } catch(e) {
                Paperlit.log.error('Paperlit.getCachedUrlAsJSONFromFile : failed parsing JSONP');
                errorCallback != null ? errorCallback() : Paperlit.noop()
            }
        }
    }
    //load file
    Paperlit.getCachedUrl(url, maxAgeInSeconds, function(url) {
        Paperlit.getAsyncUrl(url, loadJson, errorCallback);
    }, errorCallback);
};
Paperlit.parseXML = function(data) {
    //to be sure that we have no fake chars at the beginning of the string (e.g., remove any BOM in UTF-8 files)
    data = data.substring(data.indexOf('<'));

    var xml;
    if ( window.DOMParser ) { // Standard
        var parser = new DOMParser();
        xml = parser.parseFromString(data, "text/xml" );
    } else { // IE
        xml = new ActiveXObject( "Microsoft.XMLDOM" );
        xml.async = "false";
        xml.loadXML( data );
    }

    var doc = xml.documentElement;

    if ( ! doc || ! doc.nodeName || doc.nodeName === "parsererror") {
        throw "parsing error";
    }

    return xml;
}
/**
 * Same as GetCachedUrlAsString but the actual content is returned as an object containing the XML parsed from the string.
 */
Paperlit.getCachedUrlAsXML = function(url, maxAgeInSeconds, successCallback, errorCallback) {
    Paperlit.getCachedUrlAsString(url, maxAgeInSeconds, function(response) {
        try {
            //try to parse
            var xml = Paperlit.parseXML(response);
            //callback
            successCallback(xml);
        } catch(e) {
            Paperlit.log.error('Paperlit.getCachedUrlAsXML : failed parsing response into XML: '+ response);
            //call errorCallback callback
            errorCallback != null ? errorCallback() : Paperlit.noop();
        }
    }, errorCallback);
};


// file i/o browser fallbacks

Paperlit.propertiesSimulator = '{}';

Paperlit.readFile = function(fileName, successCallback, errorCallback) {
    if(fileName == Paperlit.propertiesFileName){
        if(Paperlit.lockReadWriteFile){
            // postpone call
            setTimeout(function(){
                Paperlit.readFile(fileName, successCallback, errorCallback);
            }, 10);
            return;
        }
        // avoid concurrent read/write
        Paperlit.lockReadWriteFile = true;
       
        setTimeout(function(){
            // unlock
            Paperlit.lockReadWriteFile = false;
            setTimeout(function(){
                successCallback(Paperlit.propertiesSimulator);
            }, 1);
        }, 50);
    }
    else{
        // no other file supported in browser
        setTimeout(errorCallback, 1);
    }
};

Paperlit.writeFile = function(text, successCallback, errorCallback, fileName){
    if(fileName == Paperlit.propertiesFileName){
        if(Paperlit.lockReadWriteFile){
            // postpone call
            setTimeout(function(){
                Paperlit.writeFile(text, successCallback, errorCallback, fileName);
            }, 10);
            return;
        }
        // avoid concurrent read/write
        Paperlit.lockReadWriteFile = true;
  
        setTimeout(function(){
            // unlock properties write
            Paperlit.lockReadWriteFile = false;
            Paperlit.propertiesSimulator = text;
            setTimeout(successCallback, 1);
        }, 50);
    }
    else{
        // no other file supported in browser
        setTimeout(errorCallback, 1);
    }
}
    
/* read properties from properties.pp5 */
Paperlit.readProperties = function(successCallback, errorCallback, fileName) {
    if(typeof fileName == 'undefined'){
        fileName = Paperlit.propertiesFileName;
    }
    // lock properties on write
    Paperlit.lockWriteProperties = true;

    Paperlit.readFile(fileName, function(text){
        // unlock properties on write
        Paperlit.lockWriteProperties = false;
        //Paperlit.log.info('Paperlit.readProperties, read: ' + text);
        try{
            var propertiesJSON = JSON.parse(text);
            setTimeout(function(){
                successCallback(propertiesJSON);
            }, 1);
        }
        catch(e){
            //reset all
            //alert('Paperlit.readProperties, error: ' + e.message + '. Resetting all!!!');
            Paperlit.writeProperties({}, function(){
                // reload after reset
                Paperlit.readProperties(successCallback, errorCallback, fileName); 
            });
        }
    }, function(){
        // unlock properties on write
        Paperlit.lockWriteProperties = false;
        Paperlit.missingPropertyFileFallback(successCallback);
    });
};            

/* overwrite properties (write properties on file)*/
Paperlit.overwriteProperties = function(propertiesString, successCallback, errorCallback, fileName){
    if(Paperlit.lockWriteProperties){
        // postpone call
        setTimeout(function(){
            Paperlit.overwriteProperties(propertiesString, successCallback, errorCallback, fileName);
        }, 10);
        return;
    }
    // lock property write
    Paperlit.lockWriteProperties = true;
    if(typeof fileName == 'undefined'){
        fileName = Paperlit.propertiesFileName;
    }
    // write on property file
    Paperlit.writeFile(propertiesString, function(){
        // unlock property write
        Paperlit.lockWriteProperties = false;
        setTimeout(successCallback, 0);
    }, function(){
        // unlock property write
        Paperlit.lockWriteProperties = false;
        setTimeout(errorCallback, 0);
    }, fileName);
};

Paperlit.missingPropertyFileFallback = function(successCallback){
    setTimeout(function(){
        successCallback({});
    }, 1);
};

/* Call overwrite properties and return modification date. overwriteProperties is defined for browser (simulation) and device (real file write)*/
Paperlit.writeProperties = function(properties, successCallback, errorCallback){
    errorCallback = typeof errorCallback != 'undefined' ? errorCallback : Paperlit.noop;
    try{
        // add date
        properties.modified = (new Date).toString();
        var propertiesString = JSON.stringify(properties);
        Paperlit.overwriteProperties(propertiesString, function(){
            // pass modification date to the success callback
            successCallback(properties.modified);
        }, errorCallback);
    }
    catch(e){
        //alert('Paperlit.writeProperties: error ' + e.message);
    }
    
};

/* Unique read and write properties. this is used for reading and if necessary modified properties*/
Paperlit.readWriteProperties = function(readCallback, errorCallback, fileName){
    if (Paperlit.readWriteLock){
        Paperlit.log.info('Paperlit.readWriteProperties: readWriteLock lock, waiting');
        //retry 
        setTimeout(Paperlit.readWriteProperties, 100, readCallback, errorCallback, fileName);
        return;
    }
    
    if(typeof fileName == 'undefined'){
        fileName = Paperlit.propertiesFileName;
    }
    
    var proxiedErrorCallback = function(){
        Paperlit.log.info('Paperlit.readWriteProperties: error');
        Paperlit.readWriteLock = false;
        setTimeout(errorCallback, 1);
    }
    
    var readC = function(properties){
        // apply callback
        try{
            Paperlit.log.info('Paperlit.readWriteProperties: read properties');
            var result = readCallback(properties);
            Paperlit.log.info('Paperlit.readWriteProperties: executed read callback');
            if (typeof result != 'undefined' && typeof result.needToWrite != 'undefined' && result.needToWrite){
                Paperlit.log.info('Paperlit.readWriteProperties: need to write properties');
                Paperlit.writeProperties(result.properties,  function(){
                    Paperlit.log.info('Paperlit.readWriteProperties: written properties');
                    Paperlit.readWriteLock = false;
                    if (typeof result.onWriteSuccess != 'undefined' && result.onWriteSuccess){
                         setTimeout(result.onWriteSuccess, 1);
                    }
                }, proxiedErrorCallback);
                return;
            }
            else{
                Paperlit.log.info('Paperlit.readWriteProperties: do not need to write back properties. End');
                // nothing to do, unlock monitor
                Paperlit.readWriteLock = false;
            }
        }
        catch(e){
            Paperlit.log.info('Paperlit.readWriteProperties: exception parsing properties ' + e.message);
            // exception
            Paperlit.readWriteLock = false;
            proxiedErrorCallback();
        }
    };
    
    Paperlit.readWriteLock = true;
    Paperlit.log.info('Paperlit.readWriteProperties: reading properties');
    Paperlit.readProperties(readC, proxiedErrorCallback, fileName);
};

Paperlit.noop = function() {
};
/**
 *** Trigger paperlit ready event on document
 *** This event is triggered after phonegap deviceready if phonegap is available. Otherwise immediatly
 **/
Paperlit.triggerReady = function() {
    Paperlit.log.info('Paperlit.triggerReady: triggering ppReady event');
    // old ready status
    Paperlit.ready = true;
    // mark that this event has been triggerd
    Paperlit.events['ppReady'] = true;
    //trigger ppReady event
    var evObj = document.createEvent('Events');
    evObj.initEvent('ppReady',false,true);
    document.dispatchEvent(evObj);

    // setup Adobe bridge
    // set global to state that the bridge is ready
    window.overlayBridgeReady = true;
    
    Paperlit.log.info('Bridge ready');
    // call listener to bridge ready event
    if (typeof window.onoverlaybridgeready != 'undefined'){
        Paperlit.log.info('calling onoverlaybridgeready');
        setTimeout(window.onoverlaybridgeready, 1);
    }

    Paperlit.notifyClient('initialized');
    // verify if the panel has a content associated and refresh it
    Paperlit.getPanelContentInfo(function(url){
        if(url){
            Paperlit.log.info('Found content for panel at ' + url);
            Paperlit._templateLoader = document.createElement('div');
            Paperlit._templateLoader.setAttribute('id', 'pp-ajax-loader');

            Paperlit._newtworkTimeoutAlert = document.createElement('div');
            Paperlit._newtworkTimeoutAlert.setAttribute('id', 'pp-newtwork-timeout-alert');
            Paperlit._newtworkTimeoutAlert.style.display = 'none';
            Paperlit._newtworkTimeoutAlert.style.position = 'fixed';
            Paperlit._newtworkTimeoutAlert.style.top = '50%';
            Paperlit._newtworkTimeoutAlert.style.left = '10%';
            Paperlit._newtworkTimeoutAlert.style.width = '80%';
            Paperlit._newtworkTimeoutAlert.style.height = '1rem';
            Paperlit._newtworkTimeoutAlert.style.fontSize = '1rem';
            Paperlit._newtworkTimeoutAlert.style.lineHeight = '1rem';
            Paperlit._newtworkTimeoutAlert.style.textAlign = 'center';
            Paperlit._newtworkTimeoutAlert.style.marginTop = '10px';
            Paperlit._newtworkTimeoutAlert.style.color = '#000000';
            document.body.appendChild(Paperlit._newtworkTimeoutAlert);
         
            Paperlit._templateLoader.innerHTML = '<img  src="../../../assets/ajax-loader-ffffff.gif" style="position: fixed; top: 50%; left: 50%;	margin-top: -8px;margin-left: -8px;"/>';
            try{
            	if(window.getComputedStyle(document.body, null).getPropertyValue('background-color') == 'rgb(0, 0, 0)'){
            		Paperlit._templateLoader.innerHTML = '<img  src="../../../assets/ajax-loader-black.gif" style="position: fixed; top: 50%; left: 50%;	margin-top: -8px;margin-left: -8px;"/>';
            		Paperlit._newtworkTimeoutAlert.style.color = '#FFFFFF';
            	}
            	else if(window.getComputedStyle(document.body, null).getPropertyValue('background-color') ==  'rgb(37, 37, 37)'){
            		Paperlit._templateLoader.innerHTML = '<img  src="../../../assets/ajax-loader-2a2a2a.gif" style="position: fixed; top: 50%; left: 50%;	margin-top: -8px;margin-left: -8px;"/>';
            		Paperlit._newtworkTimeoutAlert.style.color = '#FFFFFF';
            	}
            }
            catch(e){

            }
            document.body.appendChild(Paperlit._templateLoader);
            
            // this panel has an associated content -> it is managed by external code
            // manage panel events
            
            // IMPORTANT: ppOnCreated will be managed after panel content is ready

            /* Triggered when the panel has been started (it is on screen) */
            Paperlit.addWebViewEventListener('ppOnStart', function(){
                Paperlit.notifyClient('started');
            });

            /* Triggered when the panel has been stopped (out of screen before reload) */
            Paperlit.addWebViewEventListener('ppOnStop', function(){
                /* Let the client reload the panel */
                Paperlit.notifyClient('stopped');
            });

            // get panel content
            Paperlit.getPanelContent();
        }
        else{
            Paperlit.log.info('No content for this panel');
            if(Paperlit.isAndroid){
	            // trigger onCreate and onStart
	            if(typeof Paperlit.events['ppOnCreate'] === 'undefined'){
			        Paperlit.onCreate();
			    } 
	            if(typeof Paperlit.events['ppOnStart'] === 'undefined'){
			        Paperlit.onStart();
			    } 
			}
        }
    });

	// check log settings 
	Paperlit.getAppConfiguration(function(configuration){
		if(Paperlit.isNative == false){
			return;
		}

		if(Paperlit.utils.getSetting(configuration, 'newsstand-debug-ui')){
			// show button
    		Paperlit._logButton.style.display = 'block';
    	}
    	else{
    		// reset log functions
    		Paperlit.log.info = function(msg) {};
    		Paperlit.log.error = function(msg) {};
    		Paperlit.log.debug = function(msg) {};
    		Paperlit._logDiv.innerHTML = "";
    	}
	});
};

Paperlit.queue = {};
Paperlit.queue.commands = [];

// will contain the command to test
Paperlit.queue.supportedCommands = [];

Paperlit.checkIfCommandIsSupported = function(commandName, supportedCallback, notSupportedCallback){
    Paperlit.queue.supportedCommands.push({
        "commandName" : commandName,
        "supportedCallback": supportedCallback,
        "notSupportedCallback": notSupportedCallback 
    });
    //alert('enqueued');
};

/* verify if commands are supported by native client */
Paperlit.processSupportedCommandsQueue = function() {
    Paperlit.log.info('processSupportedCommandsQueue: processing' + Paperlit.queue.supportedCommands.length + ' commands');
    if(Paperlit.queue.supportedCommands.length > 0){
        var command = Paperlit.queue.supportedCommands.shift();
        Paperlit.isMethodSupported(command.commandName, function(supported) {
            try{
                supported = supported === true || supported === 'true';
                if (supported) {
                    command.supportedCallback();
                }
                else{
                    command.notSupportedCallback();
                }             
            }
            catch(e){
                command.notSupportedCallback();
            }
            finally{
                // process next command if any
                Paperlit.processSupportedCommandsQueue();
            }
        },
        function(){
            try{
                command.notSupportedCallback();
            }
            finally{
                // process next command if any
                Paperlit.processSupportedCommandsQueue();
            }
        });
    }
    else{
        // start the app
        Paperlit.start();
    }
};
    
/**
 *** Process the queue command
 **/
Paperlit.processCommandsQueue = function() {
    Paperlit.log.info('processCommandsQueue: processing'+Paperlit.queue.commands.length + ' commands');
    var funk;
    while(Paperlit.queue.commands.length>0) {
        funk = Paperlit.queue.commands.shift();
        setTimeout(funk,0);
    }
}
/**
 *  Before native client is ready, some functions are simply enqueued
 */
Paperlit.getProductPrice = function(productId, successCallback, errorCallback) {
    errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
    var funk = function() {
        Paperlit.getProductPrice(productId, successCallback, errorCallback)
    };
    Paperlit.queue.commands.push(funk);
    //Paperlit.log.info('Paperlit.getProductPrice: enqueued');
};

/** COMMON API **/

/** 
 *  Returns a json with the issues of the required publication.
 * 	html: if true --> the list will include also html5 issues
 */
Paperlit.getIssuesWithProductIdAsJSON = function (publicationId, count, html, successCallback, errorCallback, maxAgeInSeconds, getFolioIssues, issueId) {
    try {
        //alert('asking for products getIssuesWithProductIdAsJSON');
        maxAgeInSeconds = typeof maxAgeInSeconds != 'undefined' ? maxAgeInSeconds : 0;
     
        errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
       
        Paperlit.getIssueFeedUrl(publicationId, issueId, function(url){
            Paperlit.loadFeedUrl(url, maxAgeInSeconds, Paperlit.loadIssuesWithProductIdAsJSON, successCallback, errorCallback);
        }, count, html, getFolioIssues);
        
    } catch(e) {
        Paperlit.log.error('Paperlit.getIssuesWithProductIdAsJSON: error' + e.message);
        errorCallback();
    }

};

Paperlit.getIssuesWithProductIdAsXML = function (publicationId, count, html, successCallback, errorCallback, maxAgeInSeconds, getFolioIssues, issueId) {
    try {
        //alert('asking for products getIssuesWithProductIdAsJSON');
        maxAgeInSeconds = typeof maxAgeInSeconds != 'undefined' ? maxAgeInSeconds : 0;
        errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
       
        Paperlit.getIssueFeedUrl(publicationId, issueId, function(url){
            Paperlit.loadFeedUrl(url, maxAgeInSeconds, Paperlit.loadIssuesWithProductIdAsXML, successCallback, errorCallback);
        }, count, html, getFolioIssues);
       
    } catch(e) {
        Paperlit.log.error('Paperlit.getIssuesWithProductIdAsXML: error' + e.message);
        errorCallback();
    }

};

/* returns the url to be used for loading issue rss feed. It takes into account parameters and app configuration */
Paperlit.getIssueFeedUrl = function(publicationId, issueId, callback, count, html, getFolioIssues){
    count = count > 0 ? count : 12;
    html = html ? 'true' : 'false';
    var baseUrl = 'http://www.paperlit.com/rss/' + publicationId + '/' + (typeof issueId != 'undefined' && issueId != null ? (issueId) : '') + '?extras=true';
    var parameters = '&count=' + count + '&deviceId={uniqueIdentifier}&bundleId={bundleId}&bundleVersion={bundleVersion}&html=' + html;
    
    if(typeof getFolioIssues != 'undefined' && getFolioIssues === true){
        parameters = parameters + '&folioVersion={folioVersion}';
    }
    
    var defaultUrl =  baseUrl + parameters;
    
    //try to get an app configuration to see if there is feed-url inside
    if (typeof Paperlit.getAppConfiguration != 'undefined'){
        Paperlit.getAppConfiguration(function(configuration){
            var url = defaultUrl;
            // use feed-url
            var feedUrl = Paperlit.utils.getSetting(configuration, 'feeds-url');
            // use parameters that has been specified in app configuration (if any)
            if (feedUrl && feedUrl.indexOf('?') >= 0){
                baseUrl = 'http://www.paperlit.com/rss/' + publicationId + '/' + (typeof issueId != 'undefined' && issueId != null ? (issueId) : '') + '?';
                url = baseUrl + feedUrl.substring(feedUrl.indexOf('?') + 1) + parameters;
            }
            callback(url);
        }, function(){
            // load url
            callback(defaultUrl);
        });
        return;
    }
    
    callback(defaultUrl);
};

Paperlit._resourcesToBeNotified = {};

Paperlit.getResource = function(resourceUrl, successCallback, errorCallback){
    Paperlit.getCachedUrl(resourceUrl, 0, successCallback, errorCallback);
};

Paperlit.invalidateResourceOn = function(resourceUrl, events, successCallback, errorCallback){
    Paperlit._resourcesToBeNotified[resourceUrl] = 1;
    setTimeout(successCallback, 1);
};

Paperlit.invalidateAllResources = function(successCallback, errorCallback){
    try{
        $.each(Paperlit._resourcesToBeNotified, function(key, value){
            Paperlit.onResourceUpdated(key, key);    
        });
        successCallback();
    }
    catch(e){
        errorCallback();
    }
};

Paperlit.notifyResourceUpdate = function(resourceUrl, successCallback, errorCallback){
    Paperlit._resourcesToBeNotified[resourceUrl] = 1;
    setTimeout(successCallback, 1);
}

Paperlit.loadFeedUrl = function(url, maxAgeInSeconds, callback, successCallback, errorCallback){
   // if browser replace url parameters to simulate device
    if (Paperlit.isNative == false){
        Paperlit.replaceUrlParameters(url, function(replacedUrl){
            callback(replacedUrl, maxAgeInSeconds, successCallback, errorCallback);
        });
    }
    else{
        callback(url, maxAgeInSeconds, successCallback, errorCallback);
    }
};

Paperlit.loadIssuesWithProductIdAsJSON = function(url, maxAgeInSeconds, successCallback, errorCallback){ 
//'http://s20.paperlit.com/rss/computerarts/?extras=true&bundleid=com.paperlit.ipad.computerarts&count=30&bundleversion=5&deviceid=fc15627d-af06-4771-8f8b-a226038d5614'//
    try {
        Paperlit.getCachedUrlAsXML(url, maxAgeInSeconds, function (response) {
            Paperlit.log.debug('Paperlit.loadIssuesWithProductIdAsJSON: ok');
            var items = response.getElementsByTagName("item");
            var item;
            var issuesJSON = {issues: []};
            var issue;
            for (var i = 0; i < items.length; i++) {
                item = items[i];
                issue = {
                    issueId: item.getElementsByTagName("issueid")[0].childNodes[0].nodeValue,
                    issueName: item.getElementsByTagName("issuename")[0].childNodes[0].nodeValue,
                    productId: item.getElementsByTagName("productid")[0] != null ? item.getElementsByTagName("productid")[0].childNodes[0].nodeValue : null,
                    publicationName: item.getElementsByTagName("pubname")[0].childNodes[0].nodeValue,
                    publicationId: item.getElementsByTagName("pubid")[0].childNodes[0].nodeValue,
					thumbnailUrl: item.getElementsByTagName("thumbnail")[0] != null ? item.getElementsByTagName("thumbnail")[0].getAttribute('url') : null,
                    metadataUrl: item.getElementsByTagName("metadata")[0] != null ? item.getElementsByTagName("metadata")[0].getAttribute('url') : null,
                    fullMetadataUrl: item.getElementsByTagName("fullmetadata")[0] != null ? item.getElementsByTagName("fullmetadata")[0].getAttribute('url') : null,
					compositionUrl: item.getElementsByTagName("composition")[0] != null ? item.getElementsByTagName("composition")[0].getAttribute('url') : null
		        }

                if(item.getElementsByTagName("pubDate").length > 0){
                    try{
                        issue.pubDate = new Date(item.getElementsByTagName("pubDate")[0].childNodes[0].nodeValue);
                    }
                    catch(e){}
                }

                // get sid
                // e.g., http://paperlitus.blob.core.windows.net/issues/nationalreview/2013-10-14/59c914de/metadata.xml
                // remove file name
                var fullMetadataUrlPath = issue.fullMetadataUrl.substring(0, issue.fullMetadataUrl.lastIndexOf('/'));
                // remove all exept seed
                issue.seed = fullMetadataUrlPath.substring(fullMetadataUrlPath.lastIndexOf('/') + 1);
				
                if(item.getElementsByTagName("price").length > 0){
                    var price = item.getElementsByTagName("price")[0];
                    issue.price = {
                        currency : price.getAttribute('currency'),
                        symbol: price.getAttribute('symbol'),
                        amount: price.getAttribute('amount'),
                        formatted : price.getAttribute('formatted')
                    }   
                }
                
                if(item.getElementsByTagName("billing").length > 0){
                    var billing = item.getElementsByTagName("billing")[0];
                    issue.billing = {
                        authorized : billing.getAttribute('authorized') == 'true',
                        reason: billing.getAttribute('reason'),
                        productId: billing.getAttribute('productid'),
                    };
                    // issue is owned if reading is authorized and issue is not free
                    issue.billing.owned = issue.billing.authorized && issue.billing.reason != "free";   
                }
				
                //add amazon cdn images
                issue.pages = {};
                var pages = item.getElementsByTagName("pages");
                var page, image, imageUrl, imageName;
                for (var j = 1; j < pages.length; j++) {
                    page = pages[j];
                    imageUrl =  page.getAttribute('url');
                    switch(j) {
                        case 1:
                            imageName = 'small';
                            break;
                        case 2:
                            imageName = 'medium';
                            break;
                        case 3:
                            imageName = 'large';
                    }
                    image = {
                        url : imageUrl,
                        width: page.getAttribute("width"),
                        height: page.getAttribute("height")
                    }
                    issue.pages[imageName] = image;
                }
                
                issue.type = 'undefined';
                // check if folio or pdf
                if(item.getElementsByTagName("pdf").length > 0){
                    issue.type = 'pdf';
                    issue.length = item.getElementsByTagName("pdf")[0].getAttribute('length');
                }
                else if(item.getElementsByTagName("folio").length > 0){
                    issue.type = 'folio';
                    issue.length = item.getElementsByTagName("folio")[0].getAttribute('length');
                }
                
                issuesJSON.issues.push(issue);
                //add publicationId to the feed
                if (i==0) {
                    issuesJSON.publicationId = issue.publicationId;
                }
            }
            successCallback(issuesJSON);
        }, errorCallback);
    } catch(e) {
        Paperlit.log.error('Paperlit.getIssuesWithProductIdAsJSON: error');
        errorCallback();
    }
};
/**
 *  Get the latest issue of the specified publication
 * 	html: if true --> also html5 issues are considered
 */
Paperlit.getLatestIssue = function (publicationId, html, successCallback, errorCallback, maxAgeInSeconds) {
    Paperlit.getIssuesWithProductIdAsJSON(publicationId, 1, html, function(issuesJSON) {
        if(issuesJSON && typeof issuesJSON.issues != 'undefined' && issuesJSON.issues.length > 0){
            successCallback(issuesJSON.issues[0]);
            return;
        }
        successCallback(false);
    }, errorCallback, maxAgeInSeconds);
};
/**
 ** Get the page image url 
**/
Paperlit.getPageImageUrl = function(pageUrl, page) {
    var numZeropad = page + '';
    // do not zero pad when serving from paperlit.com to avoid bug in pages service 04/10/2013
    if(pageUrl.indexOf('paperlit.com') == -1){
        while(numZeropad.length < 3) {
            numZeropad = "0" + numZeropad;
        }
    }
    return pageUrl.replace(/\{.*?\}/g, numZeropad)
};

/**
 * returns url for a issue page square image of the given size
 * Device pixel ratio is taken into consideration while computing image size
 */
Paperlit.getIssuePageSquareThumbImageUrl = function (issue, pageId, square) {
    // only width was specified, height will adjust as needed
    var devicePixelRatio = (typeof window.devicePixelRatio != "undefined") ?  window.devicePixelRatio : 1;
    var url = 'http://www.paperlit.com/api/pages.aspx?p=' + issue.publicationId + '&i=' + issue.issueId + '&pageId=' + pageId + '&square=' + Math.round(square * devicePixelRatio);
    return url;
};
/**
 * returns url for a page image of given width/height
 * Device pixel ratio is taken into consideration while computing image size
 */
Paperlit.getIssuePageImageUrl = function (issue, pageId, width, height) {
    return (Paperlit.getIssuePage(issue, pageId, width, height)).url;
};

Paperlit.getIssuePage = function(issue, pageId, width, height){
    // only width was specified, height will adjust as needed
    var devicePixelRatio = (typeof window.devicePixelRatio != "undefined") ?  window.devicePixelRatio : 1;
    // if we have CDN urls
    if (issue.pages) {
        var size = 'small';
        if (width && Math.round(width * devicePixelRatio) > 355)
            size ='medium';
        if (width && Math.round(width * devicePixelRatio) > 758)
            size ='large';
        if (height && Math.round(height * devicePixelRatio) > 480 && size != 'large')
            size ='medium';
        if (height && Math.round(height * devicePixelRatio) > 768)
            size ='large';
        
        var page = issue.pages[size];
        // format page url
        page.url = Paperlit.getPageImageUrl(page.url, pageId);
        return page;
    } else {
        var url = 'http://cdn.paperlit.com/pages/' + issue.publicationId + '?issueId=' + issue.issueId +  '&pageId=' + pageId;
        var page = { url : "", width: 0, height: 0};
        if (typeof width != 'undefined') {
            url += '&width=' + Math.round(width * devicePixelRatio);
            page.width = Math.round(width * devicePixelRatio);
        }
        if (typeof height  != 'undefined') {
            url += '&height=' + Math.round(height * devicePixelRatio);
            page.height = Math.round(height * devicePixelRatio);
        }
        page.url = url;
        return page;
    }    
};
Paperlit.getIssueCover = function (issue, width, height) {
    return Paperlit.getIssuePage(issue, 1, width, height);
};
/**
 * returns url for cover image of given width/height
 * Device pixel ratio is taken into consideration while computing image size
 */
Paperlit.getIssueCoverImageUrl = function (issue, width, height) {
    return Paperlit.getIssuePageImageUrl(issue, 1, width, height);
};
/** returns <img> tag for cover image of given width/height */
Paperlit.getIssueCoverImage = function (issue, width, height, maxAgeInSeconds) {
    return Paperlit.getIssuePageImage (issue, 1, width, height, maxAgeInSeconds);
};
/** returns <img> tag for page image of given width/height */
Paperlit.getIssuePageImage = function (issue, pageId, width, height, maxAgeInSeconds) {
    var url = Paperlit.getIssuePageImageUrl (issue, pageId, width, height);
    var img = document.createElement('img');
    if (width != null) {
        img.style.maxWidth = width + 'px';
    }
    if (height != null) {
        img.style.maxHeight = height + 'px';
    }
    var placeholder = document.createElement('div');
    var onLoad = function(){
        placeholder.appendChild(this);
    }
    img.addEventListener('load', onLoad, false);
    maxAgeInSeconds = typeof maxAgeInSeconds != 'undefined' ? maxAgeInSeconds : 0;
    Paperlit.getCachedUrl(url, maxAgeInSeconds, function(cachedUrl){
        img.src = cachedUrl; 
    });
    return placeholder;
};

Paperlit.getServerPageUrl = function(issue, pageId, width, height){
    var url = 'http://cdn.paperlit.com/pages/' + issue.publicationId + '/' + issue.issueId +  '/' + pageId + '?border=false';
    if (typeof width != 'undefined') {
        url += '&width=' + Math.round(width);
    }
    if (typeof height  != 'undefined') {
        url += '&height=' + Math.round(height);
    }
    // consider seed when caching images, otherwise if a issue is republished under the same issueid the cache is not aware of it
    if(typeof issue.seed !== 'undefined'){
        url += '&seed=' + issue.seed;
    }
    return url;
}

/** Returns url of a cached issue page image */
Paperlit.getCachedPageUrl = function(issue, pageId, successCallback, errorCallback, width, height){
    try {
        var url = Paperlit.getServerPageUrl(issue, pageId, width, height);
        // long age because covers never expire. seed is used when republishing content
        Paperlit.getCachedUrl(url, 60*60*24*30*12, function(cachedUrl) {successCallback({url: cachedUrl})}, errorCallback)
    } catch(e) {
        Paperlit.log.error('Paperlit.getCachedPageUrl: error calling PaperlitCommands.getCachedPage');
    }
};



/** 
 ** Returns a read/buy link for the specific issue 
 ** InApp specify if we want to enable/disable billing (default: true) 
 ** New: now it considers also billing field if any
 ** and if the issue is on device
*/
Paperlit.getIssueReadUrl = function (issue, inapp, isOnDevice) {
    var productId = null;
    // check if billing field is available
    if (typeof issue.billing != 'undefined'){
        // check authorization field
        if (issue.billing.authorized === 'false' || issue.billing.authorized === false){
            productId = issue.billing.productId;
        }
    }
    else if(typeof issue.productId != 'undefined'){ 
        productId = issue.productId;
    }
    inapp = typeof inapp == 'undefined' ? true : inapp;
    // if the issue is on the device consider it as free
    if (typeof isOnDevice != 'undefined' && isOnDevice){
        inapp = false;
    }
    
    //if not free and inapp is enabled
    if (productId != null && inapp) {
		// check if paypal and not on iOS
		if (inapp == 'paypal' && !Paperlit.isIOS){
			return 'paperlit://paypal?productId=' + productId + '&publicationId=' + issue.publicationId + '&issueId=' + issue.issueId + '&product=' + encodeURIComponent(issue.publicationName + ' | ' + issue.issueName) + '&currency=' + issue.price.currency + '&symbol=' +  encodeURIComponent(issue.price.symbol) + '&amount=' + issue.price.amount + '&formatted=' +  encodeURIComponent(issue.price.formatted);
		}
		else{
			return 'paperlit://buy/' + productId;
		}
    } else {
        return 'paperlit://read/' + issue.publicationId + "/" + issue.issueId + (typeof issue.pageId != 'undefined' && issue.pageId != null ? '/' + issue.pageId : '');
    }
};

/** returns cover image that can be clicked to read <a> + <img> */
Paperlit.getIssueCoverClickToRead = function (issue, width, height, inapp, maxAgeInSeconds, isOnDevice) {
    var actionUrl =  Paperlit.getIssueReadUrl(issue, inapp, isOnDevice);
    var imageTag =  Paperlit.getIssueCoverImage(issue, width, height, maxAgeInSeconds);
    var link = document.createElement('a');
    link.setAttribute("onclick", "Paperlit.openUrl('" + actionUrl + "');");
    link.appendChild(imageTag);
    return link;
};
/* returns a button with buy or read label for an issue */
Paperlit.getIssueButton = function(issue, read, buy, inapp, noprice, isOnDevice, priceString, options) {
    try {
		var actionUrl =  Paperlit.getIssueReadUrl(issue, inapp, isOnDevice);
        var issueButton = document.createElement('a');
            
		// if free 
        if (actionUrl.indexOf('paperlit://read/') >= 0) {
            issueButton.setAttribute('class','ppIssueButton ppIssueButtonRead');
            issueButton.innerHTML = read;
        } else {
            //Insert a call to get the issue price from the application (if supported).
            //The callback will show the price upon the buy button
            var buttonId = 'IssueButton-' + (new Date()).getTime() + parseInt(Math.random() * 10000, 10);
            issueButton.setAttribute('id', buttonId);
            issueButton.setAttribute('class','ppIssueButton ppIssueButtonBuy');
            
            if (typeof noprice == 'undefined' || noprice == false){
				var backofficePrice;
				if(typeof issue.price != 'undefined' && issue.price.amount > 0){
					backofficePrice = issue.price.formatted;
				}
    			//check if it is a paypal product or the platform is not iOS and there is a backoffice price
    			if (actionUrl.indexOf('paperlit://paypal') >= 0 && backofficePrice) {
    				//price comes from backoffice in the issue object
    				issueButton.innerHTML = issue.price.formatted;
    			}
    			else{
    				//write the default string while getting price from itunes
    				issueButton.innerHTML = buy;
                	//require price
    				setTimeout(function(productId, issueButton, backofficePrice, priceString){
						//alert('asking price for ' + productId);
						Paperlit.getProductPrice(productId, function(price) {
                            if(priceString){
                                issueButton.innerHTML = priceString.replace('{price}', price);
                            }
                            else{
							    issueButton.innerHTML = price;
                            }
							issueButton = null;
						}, function() {
							//alert('asking price failed ' + productId);
							// if it fails, set backoffice price on android
							if(Paperlit.isAndroid && backofficePrice){
                                if(priceString){
                                    issueButton.innerHTML = priceString.replace('{price}', backofficePrice);
                                }
                                else{
                                    issueButton.innerHTML = backofficePrice;
                                }
							}
						});
					}, 0, issue.productId, issueButton, backofficePrice, priceString);
                
    			}
            }
            else{
                //write the default buy string
   				issueButton.innerHTML = buy;
            }
			
        }

        var openUrlClosure = Paperlit.openUrlClosure(actionUrl);
        if(typeof options === 'undefined' || typeof options.handlerOnClickEvent === 'undefined' || options.handlerOnClickEvent === true){
            issueButton.addEventListener(Paperlit.clickEventName, function(event){
                if (event){
                    event.stopPropagation();
                }
                openUrlClosure();
            });
        }
        
        if(typeof $ !== 'undefined'){
            $(issueButton).on('ppTap', function(event){
                if (event){
                    event.stopPropagation();
                }
                openUrlClosure();
            });
        }
        else{
            issueButton.addEventListener('ppTap', function(event){
                if (event){
                    event.stopPropagation();
                }
                openUrlClosure();
            });
        }
        
        return issueButton;
    } catch(e) {
        Paperlit.log.error('Paperlit.getIssueButton: failed '+e.message)
    }

};

Paperlit.openUrlClosure = function(url){
    return function(){
        Paperlit.openUrl(url);
    };
};

/** sets the product price got from the app store to the button*/
Paperlit.setProductPrice = function(element, productId) {
    try {
        Paperlit.getProductPrice(productId, function(price) {
			element.innerHTML = price;
			element = null;
        }, function(error) {
            Paperlit.log.error('Paperlit.setProductPrice: failed getting price '+error);
        });
    } catch(e) {
    }
};

    
/* returns a button for deleting local issue copy */
Paperlit.getDeleteLocalIssueButton = function(issue, deleteString) {
    var issueButton = document.createElement('a');
    issueButton.setAttribute('class','ppIssueButton ppIssueButtonDelete');
    issueButton.setAttribute('data-publicationId', issue.publicationId );
    issueButton.setAttribute('data-issueId', issue.issueId );
    issueButton.innerHTML = deleteString;
    return  issueButton;
};

/* replace the parameters using app state*/
Paperlit.replaceUrlParameters = function(url, successCallback, errorCallback){
    Paperlit.getState(function(appState){
        url = Paperlit.replaceParameters(appState, url);
        successCallback(url);  
    }, errorCallback); 
};

Paperlit.replaceParameters = function(appState, url){
    if (appState.uniqueIdentifier){
        url = url.replace('{uniqueIdentifier}', appState.uniqueIdentifier);
        url = url.replace('{deviceId}', appState.uniqueIdentifier);
    }
    if (appState.bundleId)
        url = url.replace('{bundleId}', appState.bundleId);
    if (appState.bundleVersion) 
        url = url.replace('{bundleVersion}', appState.bundleVersion);
    if (appState.publicationId)
        url = url.replace('{publicationId}', appState.publicationId);    
    if (appState.issueId)
        url = url.replace('{issueId}', appState.issueId);
    if (appState.pageId){  
        url = url.replace('{pageId}', appState.pageId);
        url = url.replace('{page}', appState.pageId);
    }
    if (appState.annotationId)    
        url = url.replace('{annotationId}', appState.annotationId);
    if (appState.publicationName)    
        url = url.replace('{publicationName}', appState.publicationName);
    if (appState.issueName)      
        url = url.replace('{issueName}', appState.issueName);
    if (appState.hash_UUID){      
        url = url.replace('{hash_UUID}', appState.hash_UUID);
    }
    if (appState.uniqueIdentifier_UUID){      
        url = url.replace('{uniqueIdentifier_UUID}', appState.uniqueIdentifier_UUID);
    }
    if (appState.systemName){      
        url = url.replace('{systemName}', appState.systemName);
    }
    if (appState.systemVersion){      
        url = url.replace('{systemVersion}', appState.systemVersion);
    }
    if (appState.model){      
        url = url.replace('{model}', appState.model);
    }
    if (appState.timestamp){      
        url = url.replace('{timestamp}', appState.timestamp);
    }
    if (appState.folioVersion){      
        url = url.replace('{folioVersion}', appState.folioVersion);
    }
    
    return url;
};

Paperlit.replaceUrlParametersSync = function(url){
    var appState = Paperlit.getBrowserState();
    return Paperlit.replaceParameters(appState, url);
}

/* Save current navigator.geolocation.getCurrentPosition */
if(navigator.geolocation){
    Paperlit.geolocation = {getCurrentPosition: navigator.geolocation.getCurrentPosition};
}


Paperlit.deleteArchivedIssue = function(publicationId, issueId, successCallback, errorCallback) {
    setTimeout(successCallback, 100);   
}


Paperlit.parseJSON = function(text, successCallback, errorCallback){
    try{
        successCallback(JSON.parse(text));
    }
    catch(e){
        errorCallback();
    }
}

Paperlit._getAppConfigurationCallbacks = [];


Paperlit._callConfigurationErrorCallbacks = function(){
    	for(var i=0,l=Paperlit._getAppConfigurationCallbacks.length; i<l; i++){
    		try{
    			Paperlit._getAppConfigurationCallbacks[i].errorCallback();
    		}
    		catch(e){}
    	}
    	Paperlit._getAppConfigurationCallbacks = [];
    }

Paperlit._callConfigurationSuccessCallbacks = function(configuration){
    	for(var i=0,l=Paperlit._getAppConfigurationCallbacks.length; i<l; i++){
    		try{
    			Paperlit._getAppConfigurationCallbacks[i].successCallback(configuration);
    		}
    		catch(e){}
    	}
    	Paperlit._getAppConfigurationCallbacks = [];
    }

/* load local app configuration */
Paperlit.getAppConfiguration = function(successCallback, errorCallback, forceReload){

	if(forceReload){
		Paperlit.log.info('Paperlit.getAppConfiguration: required a reload of the configuration');
		Paperlit._appConfiguration = null;
		// remove current callbacks, a ppOnConfigurationChange event will be triggered
		Paperlit._getAppConfigurationCallbacks = [];
	}
    // check if already loaded
    else if(typeof Paperlit._appConfiguration !== 'undefined' && Paperlit._appConfiguration != null){
    	console.log('Paperlit.getAppConfiguration: returning current config');
        successCallback(Paperlit._appConfiguration);
        return;
    }

    // first load of app configuration
    // store callbacks
    Paperlit._getAppConfigurationCallbacks.push({successCallback: successCallback, errorCallback: errorCallback});

	if(Paperlit._getAppConfigurationCallbacks.length > 1){
		// already loading configuration
		return;
	}    
	/*	*/

    if(Paperlit.isNative === false){
    	// if using firebase.js
    	if(typeof Paperlit._myFirebaseRef == 'undefined'){ 
	    	// when not inapp use firebase js
			Paperlit._myFirebaseRef = new Firebase("https://torid-torch-8564.firebaseio.com/");
			Paperlit._myFirebaseRef.on('value', function (snapshot) {
				Paperlit.log.info('Paperlit.getAppConfiguration: firebase.js received config');
				// update configuration
				Paperlit._appConfiguration = snapshot.val();

				if(typeof Paperlit._appPreviousConfiguration == 'undefined'){
					Paperlit.log.info('Paperlit.getAppConfiguration: firebase.js received first config, call callbacks waiting for configuration');
					Paperlit._callConfigurationSuccessCallbacks(Paperlit._appConfiguration);
				}
				else{
					Paperlit.log.info('Paperlit.getAppConfiguration: firebase.js received new config, trigger update event');
					// check if only css keys have changed
					var hasChangedOnlyCSS = true;
					for(p in Paperlit._appConfiguration) {
					    if(Paperlit._appConfiguration[p] !==  Paperlit._appPreviousConfiguration[p]){
					    	Paperlit.log.info('Paperlit.getAppConfiguration: property ' + p + ' has changed');
					    	if(p.indexOf('css') == -1){
					    		hasChangedOnlyCSS = false;
					    		break;
					    	}
					    }
					}
					var evObj = document.createEvent('Events');
				    evObj.initEvent('ppOnConfigurationChange', false, true);
				    evObj.hasChangedOnlyCSS = hasChangedOnlyCSS;
				    document.dispatchEvent(evObj);
				}
				// store current configuration as the old one
				Paperlit._appPreviousConfiguration = Paperlit._appConfiguration;
			}, 	function (errorObject) {
			  	Paperlit.log.error('The read failed: ' + errorObject.code);
			  	Paperlit._callConfigurationErrorCallbacks();
			});
		}
    }
    else{
    	var success = function(configurationText){
	        // parse text into json
	        Paperlit.parseJSON(configurationText, function(configuration){
				// update configuration
				Paperlit._appConfiguration = configuration;

				if(typeof Paperlit._appPreviousConfiguration == 'undefined'){
					Paperlit.log.info('Paperlit.getAppConfiguration: loaded first config, call callbacks waiting for configuration');
					Paperlit._callConfigurationSuccessCallbacks(Paperlit._appConfiguration);
				}
				else{
					Paperlit.log.info('Paperlit.getAppConfiguration: reloaded config, trigger update event');
					// check if only css keys have changed
					var hasChangedOnlyCSS = true;
					
					for(p in Paperlit._appConfiguration) {
					    if(Paperlit._appConfiguration[p] !==  Paperlit._appPreviousConfiguration[p]){
					    	Paperlit.log.info('Paperlit.getAppConfiguration: property ' + p + ' has changed');
					    	if(p.indexOf('css') == -1){
					    		hasChangedOnlyCSS = false;
					    		break;
					    	}
					    }
					}
					
					var evObj = document.createEvent('Events');
				    evObj.initEvent('ppOnConfigurationChange', false, true);
				    evObj.hasChangedOnlyCSS = hasChangedOnlyCSS;
				    document.dispatchEvent(evObj);
				}

				Paperlit._appPreviousConfiguration = Paperlit._appConfiguration;

	        }, Paperlit._callConfigurationErrorCallbacks);
	    }
	    var configFileName = Paperlit.appRootPath + '/merged.json?nocaching=' + (new Date()).getTime();
	    // look for merged settings
	    Paperlit.getAsyncUrl(configFileName, success, function(){
	        // look one folder backward
	        Paperlit.getAsyncUrl('../' + configFileName, success, Paperlit._callConfigurationErrorCallbacks);
	    });
    }

};

/* track event can take n arguments. Last two should be callbacks*/
Paperlit.trackEvent = function() {
    try {
        if (arguments.length < 3){
            throw "missing arguments";
        }
        var args = Array.prototype.slice.call(arguments);
        
        var errorCallback = args.pop();
        var successCallback = args.pop();
        console.log('Paperlit.trackEvent: calling ' + args.toString());
        successCallback();
    } catch(e) {
        console.log('exception calling Paperlit.trackEvent ' + e.message);
    }
}


// new API


Paperlit.getIssues = function (publicationId, successCallback, errorCallback, count){
    // default is all issues
    count = typeof count != 'undefined' ? count : 10000;
    Paperlit.getIssuesWithProductIdAsJSON(publicationId, count, false, successCallback, errorCallback, 0, true);
};

Paperlit.getPdfIssues = function (publicationId, successCallback, errorCallback, count){
    var filterIssues = function(issuesJSON){
        var filteredIssues = issuesJSON.issues.filter(function(issue, index, array){
            return issue.type === 'pdf';
        });
        issuesJSON.issues = filteredIssues;
        successCallback(issuesJSON);
    }
    
    Paperlit.getIssues(publicationId, filterIssues, errorCallback, count);
};

Paperlit.getFolioIssues = function (publicationId, successCallback, errorCallback, count){
    var filterIssues = function(issuesJSON){
        var filteredIssues = issuesJSON.issues.filter(function(issue, index, array){
            return issue.type === 'folio';
        });
        issuesJSON.issues = filteredIssues;
        successCallback(issuesJSON);
    }
    
    Paperlit.getIssues(publicationId, filterIssues, errorCallback, count);
};

/* Returns all the free issues */
Paperlit.getFreeIssues = function (publicationId, successCallback, errorCallback, count){
    
    var filterIssues = function(issuesJSON){
        var filteredIssues = issuesJSON.issues.filter(function(issue, index, array){
            return typeof issue.billing != 'undefined' && typeof issue.billing.reason != 'undefined' && issue.billing.reason === "free";
        });
        issuesJSON.issues = filteredIssues;
        successCallback(issuesJSON);
    }
    
    Paperlit.getIssues(publicationId, filterIssues, errorCallback, count);
}; 


/* issues as XML */

Paperlit.getIssuesAsXML = function (publicationId, successCallback, errorCallback, count){
    // default is all issues
    count = typeof count != 'undefined' ? count : 10000;
    Paperlit.getIssuesWithProductIdAsXML(publicationId, count, false, successCallback, errorCallback, 0, true);
};


/** Returns xml document containing all the user subscriptions. For each subscription details are provided (start date, duration, transactions, etc...) */
Paperlit.getUserSubscriptions = function(successCallback, errorCallback) {
    var subscriptionUrl = 'http://www.paperlit.com/billing/Subscriptions.ashx?deviceId={uniqueIdentifier}&bundleId={bundleId}';
    if (Paperlit.isNative == false){
        Paperlit.replaceUrlParameters(subscriptionUrl, function(replacedUrl){
            Paperlit.getCachedUrlAsXML(replacedUrl, 0, successCallback, errorCallback);
        });
    }
    else{
        Paperlit.getCachedUrlAsXML(subscriptionUrl, 0, successCallback, errorCallback);
    }
};

/* Check if */
Paperlit.getFirstValidSubscription = function(subscriptions, productId){
    try{
        productId = typeof productId === 'undefined' ? null : productId;
        var subscriptionElements = subscriptions.getElementsByTagName('Subscription');
        for(var i=0, l=subscriptionElements.length; i<l; i++){
            var subscription = subscriptionElements[i];
            var isCurrentSubscriber = subscription.getAttribute('IsCurrentSubscriber').toLowerCase() === 'true';
            var subscriptionProductId = subscription.getAttribute('ProductId');
            // product id is null even if empty string
            if(typeof subscriptionProductId === 'undefined' || subscriptionProductId === ''){
                subscriptionProductId = null;
            }
            // valid subscription only if product id is null or product id
            if(isCurrentSubscriber && (productId == null || subscriptionProductId == productId)){
                // $this->hasFreeTrial = endsWith($s['ProductId'], 'freetrial');
                return subscription;
            }
        }
    }
    catch(e){
    }
    return false;
};

Paperlit.getSubscriptionButton = function(subscription, billingConfiguration, priceString, placeholder){
    try{
        var inapp = Paperlit.utils.getSetting(billingConfiguration, 'billing-paypal') != null ? 'paypal' : true;
        var backofficeSubscriptionPrice = typeof subscription['in-app-subscription-price'] === 'undefined' ? null : subscription['in-app-subscription-price'];
        var priceString = typeof priceString != 'undefined' && priceString != null ? priceString : subscription['in-app-subscription-description'];
        // ask price and if it fails show backoffice price
        // if it succeeds, show only if <> 0. If the price is 0 we assume that some price STRING has been configured in the backoffice (e.g., Free)
        var subscriptionButton = document.createElement('a');
        subscriptionButton.setAttribute('class','pp-subscription-button');
        if(backofficeSubscriptionPrice && /[1-9]/.test(backofficeSubscriptionPrice) == false){
            // if the backoffice price is not a number (e.g., "free" for free subscriptions)
            // use this string as placeholder instead of the default "buy" string
            // we do not want to show backoffice prices (e.g. $3.00) before asking the store because if there is a mismatch 
            // the user will see a different price as soon as the store responds.
            subscriptionButton.innerHTML = priceString.replace('{price}', backofficeSubscriptionPrice);
        }
       
        var productId = subscription['in-app-subscription-productid'];
        var actionUrl;
        if (inapp == 'paypal' && !Paperlit.isIOS){
            //take only price
            var amount = parseFloat(backofficeSubscriptionPrice.replace( /^\D+/g, ''));
            actionUrl = 'paperlit://paypal?productId=' + productId + '&product=' + encodeURIComponent(subscription['in-app-subscription-name']) + 
                '&currency=' + Paperlit.utils.getSetting(billingConfiguration, 'currency-code') + '&symbol=' + Paperlit.utils.getSetting(billingConfiguration, 'currency-symbol') + '&amount=' + amount;
            // there is no store, set price immediately
            subscriptionButton.innerHTML = priceString.replace('{price}', backofficeSubscriptionPrice);
        }
        else{
            setTimeout(function(productId, subscriptionButton, backofficeSubscriptionPrice, priceString){
                Paperlit.getProductPrice(productId, function(price) {
                    if(/[1-9]/.test(price)){
                        // replace only if store price is not 0 (E.g, a free subscription should remain with the string configured in backoffice "Free")
                        subscriptionButton.innerHTML = priceString.replace('{price}', price);
                    }
                    subscriptionButton = null;
                }, function() {
                    // if it fails, set backoffice price (if any)
                    if(backofficeSubscriptionPrice){
                       subscriptionButton.innerHTML = priceString.replace('{price}', backofficeSubscriptionPrice);
                    }
                    subscriptionButton = null;
                });
            }, 0, productId, subscriptionButton, backofficeSubscriptionPrice, priceString);

            // show hellipsis while checking the product price
            subscriptionButton.innerHTML = priceString.replace('{price}', typeof placeholder != 'undefined' ? placeholder : '&hellip;');
        
            actionUrl = 'paperlit://buy/' + productId;
        }

        var openUrlClosure = Paperlit.openUrlClosure(actionUrl);
        
        subscriptionButton.addEventListener('ppTap', function(event){
            if (event){
                event.stopPropagation();
            }
            openUrlClosure();
        });

        return subscriptionButton;
    }
    catch(e){}
};

/* Returns to the successCallback the status of the single sign on. True if a single sign on transaction is currently active. False otherwise. */
Paperlit.hasSingleSignOnActive = function(successCallback, errorCallback){
    Paperlit.getUserSubscriptions(function(response){
        if (response != null){
            var subscriptions = response.getElementsByTagName('Subscription');
            var subscription;
            // scan subscriptions
            for(var i=0; i<subscriptions.length; i++){
                subscription = subscriptions[i];
                if (subscription.getAttribute('ProductId') == 'singlesignon' && subscription.getAttribute('IsCurrentSubscriber') == 'true'){
                    // we have found an active sso subscription. Stop here, return the record
                    successCallback(subscription);
                    return;
                }
            }
            //default is false
            successCallback(false);
            return;
        }
        
        setTimeout(errorCallback, 1);
    }, errorCallback);
};

Paperlit.proxy = function(fn, context) {
    var args = Array.prototype.slice.call(arguments, 2);
    return function() {
    	// append arguments
    	var funkArgs = Array.prototype.slice.call(arguments);
    	for (var i = 0; i < funkArgs.length; i++) {
		    args.push(funkArgs[i]);
		}
        fn.apply(context, args);
    };
};


/* returns an array of subscriptions that are available in the application */
Paperlit.getBillingConfiguration = function(successCallback, errorCallback, maxAgeInSeconds){
    maxAgeInSeconds = typeof maxAgeInSeconds === 'undefined' ? 0 : maxAgeInSeconds;
    var test = Paperlit.utils.getUrlParameter('test');
    var jsonUrl, versionedJsonUrl;
    if(test == null || test === 'false'){
        test = false;
    }
    if(test === 'true'){
        test = true;
    }
    // configuration file url from amazon
    if (test) {
        versionedJsonUrl = "http://paperlit-apps.s3.amazonaws.com/{bundleId}.test/assets/billing_{bundleVersion}.json";
        jsonUrl = "http://paperlit-apps.s3.amazonaws.com/{bundleId}.test/assets/billing.json";
    } else {
        versionedJsonUrl = "http://paperlit-apps.s3.amazonaws.com/{bundleId}/assets/billing_{bundleVersion}.json";
        jsonUrl = "http://paperlit-apps.s3.amazonaws.com/{bundleId}/assets/billing.json";
    }

    // check if a versioned file exists for this specific bundleVersion
    // otherwise load the unversioned file
    Paperlit.getCachedUrlAsJSON(versionedJsonUrl, maxAgeInSeconds, successCallback, 
        Paperlit.proxy(Paperlit.getCachedUrlAsJSON, this, jsonUrl, maxAgeInSeconds, successCallback, errorCallback)
    );
    
}

/* returns an array of subscriptions that are available in the application */
Paperlit.getInappSubscriptionsList = function(successCallback, errorCallback, maxAgeInSeconds){
    var callback = function(response){
        try{
            // retrieve in-app subscription list
            if (response){
                if(typeof response['in-app-subscriptions'] !== 'undefined'){
                    successCallback(response['in-app-subscriptions']);
                }
                else{
                    successCallback([]);
                }
                return;
            }
        }
        catch(e){

        }
        setTimeout(errorCallback, 1);
    };
    // load billing configuration
    Paperlit.getBillingConfiguration(callback, errorCallback, maxAgeInSeconds);
};


Paperlit.loadIssuesWithProductIdAsXML = function(url, maxAgeInSeconds, successCallback, errorCallback){ 
//'http://s20.paperlit.com/rss/computerarts/?extras=true&bundleid=com.paperlit.ipad.computerarts&count=30&bundleversion=5&deviceid=fc15627d-af06-4771-8f8b-a226038d5614'//
    try {
        Paperlit.getCachedUrlAsXML(url, maxAgeInSeconds, successCallback, errorCallback);
    } catch(e) {
        Paperlit.log.error('Paperlit.loadIssuesWithProductIdAsXML: error' + e.message);
        errorCallback();
    }
};


Paperlit.getIssue = function (publicationId, issueId, successCallback, errorCallback){
    Paperlit.getIssuesWithProductIdAsJSON(publicationId, 1, false, function(issuesJSON) {
        if(issuesJSON && typeof issuesJSON.issues != 'undefined' && issuesJSON.issues.length > 0){
            successCallback(issuesJSON.issues[0]);
            return;
        }
        successCallback(false);
    }, errorCallback, 0, true, issueId);
};

/* Bookmarks */

// Old getBookmarks method that support also browser
Paperlit.getBookmarks = function(successCallback, errorCallback){
    if(typeof $ === 'undefined'){
        errorCallback();
        throw 'method not supported without jquery';
    }
    errorCallback = typeof errorCallback != 'undefined' ? errorCallback : Paperlit.noop;
    Paperlit.readWriteProperties(function(properties){
        //alert('Paperlit.getBookmarks: got properties');
        if(typeof properties.bookmarks == 'undefined'){
            // returns empty array
             properties.bookmarks = [];
        }
       
        var gotNativeBookmarks = function(response){
            //alert('response ' + response)
            var bookmarks = this.bookmarks;
            // add folio bookmarks
            var modified = this.modified || null;
            if(response !== false){
                try{
                    var xml = $.parseXML(response);
                    var nativeUpdate = $(xml).find('Favorites').attr('Update');
                    if(nativeUpdate){
                        if(modified){
                            modified = modified + nativeUpdate;
                        }
                        else{
                             modified = nativeUpdate;
                        }
                    }
                    else{
                        //alert('cannot retrieve update from native bookmarks')
                    }
                    //alert('success1 ' + response)
                    var favorites = $(response).find('Favorite');
                    var favorite, id, issue;
                    var baseUrl = window.location.href.substring(0, window.location.href.lastIndexOf('/')).replace('file://', '') + '../../../Documents/';
                    for (var i=favorites.length-1; i>=0; i--){
                        favorite = $(favorites[i]);
                        id = favorite.attr('UniqueId');
                        if(id.indexOf('paperlit://read/') > -1){
                            issue = id.replace('paperlit://read/','');
                            issue = issue.split('/');
                            var imageUrl = favorite.attr('ImageUrl');
                            if(imageUrl && Paperlit.isIOS && imageUrl[0] != '/'){
                                imageUrl = baseUrl + imageUrl;
                            }
                            
                            
                            bookmarks.push({
                                "bookmarkId": id, 
                                "publicationId": issue[0],
                                "issueId": issue[1],
                                "pageId": issue[2].replace(/^[0]+/g,""),
                                "annotationId":  "",
                                "publicationName":  "",
                                "issueName":  favorite.attr('Title') ? favorite.attr('Title') : '',
                                "isNative" : true,
                                "imageUrl" : imageUrl,
                                "subtitle":  favorite.attr('Subtitle') ? favorite.attr('Subtitle') : ''
                            });
                        }
                    }
                }
                catch(e){
                    
                }
            }
            else{
                //alert('error loading native bookmarks');
            }
            // returns bookmarks
            setTimeout(function(){ 
                successCallback(bookmarks, modified);
            }, 1); 
        };
        
        Paperlit.readFile(Paperlit.nativeBookmarksFileName, $.proxy(gotNativeBookmarks, properties), $.proxy(gotNativeBookmarks, properties, false));
        
        return {needToWrite: false};
        
    }, errorCallback);
};

Paperlit.saveBookmark = function(bookmark, successCallback, errorCallback){
	errorCallback();
};

Paperlit.removeBookmark = function(bookmark, successCallback, errorCallback){
	errorCallback();
};

/* Social */

Paperlit.social = {};
Paperlit.social.login = function(platform, successCallback, errorCallback){
    errorCallback();
};               

/* Social feature, logout on platform*/
Paperlit.social.logout = function(platform, successCallback, errorCallback){
    errorCallback();
};    

/* Social feature, get login status on platform*/
Paperlit.social.getLoginStatus = function(platform, successCallback, errorCallback){
    errorCallback();
};  

/* share url on platform */
Paperlit.social.share = function(platform, url, successCallback, errorCallback){
    errorCallback();
};  

/* inviteFriends on platform */
Paperlit.social.inviteFriends = function(platform, successCallback, errorCallback){
    errorCallback();
};  


Paperlit.getJsonp = function(url, successCallback, errorCallback){
    var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
    if(url.indexOf('?') > -1){
        url = url + '&callback=' + functionsName.successFuncName;
    }
    else{
        url = url + '?callback=' + functionsName.successFuncName;
    }

    var errorCallbackId = functionsName.errorFuncName.replace('Paperlit._anomFunkMap.', '');
    //Paperlit.alert('loading script ' + url);
    var script = document.createElement("script")
    script.type = "text/javascript";
    script.onload = function(){
         var scripts = document.getElementsByTagName("script");
         //Paperlit.alert('script loaded : ' + scripts[scripts.length-1].innerText);

    };
    script.onerror = Paperlit._anomFunkMap[errorCallbackId];
    script.src = url;
    document.getElementsByTagName("head")[0].appendChild(script);

    setTimeout(function(errorCallbackId){
        // call error callback after timeout
        try{
            if(typeof Paperlit._anomFunkMap[errorCallbackId] != 'undefined'){
                Paperlit._anomFunkMap[errorCallbackId]();
            }
        }
        catch(e){}
    }, 30000, errorCallbackId);
};

Paperlit.checkFeatureAvailability = function(feature, successCallback, errorCallback){
    if(typeof $ === 'undefined'){
        errorCallback();
        return;
    }
    var url = 'http://www.paperlit.com/php/services/featureavailability/featureavailability.php?deviceId={deviceId}&bundleId={bundleId}&feature=' + feature;
    Paperlit.replaceUrlParameters(url, function(replacedUrl){
        if($){
            $.ajax({
                url: url,
                dataType: 'jsonp',
                success: successCallback,
                error: errorCallback
            })
        }
        else{
            errorCallback();
            //Paperlit.getJsonp(replacedUrl, successCallback, errorCallback);
        }
    }, errorCallback);
};

/* Mini app specific command (only iOS) */
// closes the current miniapp
Paperlit.closeApp = function(errorCallback) { 
    try {
        //simulate async
        setTimeout(function(){errorCallback()}, 1);
    } catch(e) {
        //alert('PaperlitCommands.closeApp: exception');
    }
};

// tells the system the app is ready to be displayed
Paperlit.startApp = function(successCallback, errorCallback) {
    try {
        //simulate async
        setTimeout(function(){successCallback()}, 1);
    } catch(e) {
        //alert('PaperlitCommands.startApp: exception');
    }  
};

// saves the provided base64 encoded data as the application content
Paperlit.saveBase64AppContent = function(dataUrl, successCallback, errorCallback) {
    try {
        //simulate async
        setTimeout(function(){successCallback()}, 1);
    } catch(e) {
        //alert('PaperlitCommands.saveBase64AppContent: exception');
    }  
};

// saves the provided base64 encoded data as the application content, closes the app when finished
Paperlit.saveBase64AppContentAndExit = function(dataUrl, errorCallback) {
    try {
        //simulate async
        setTimeout(function(){successCallback()}, 1);
    } catch(e) {
        //alert('PaperlitCommands.saveBase64AppImageContentAndExit: exception');
    }  
};

// exports the provided base64 encoded data as an image to the photo album
Paperlit.exportBase64AppImageContentToPhotos = function(dataUrl, width, height, overPdf, successCallback, errorCallback) {
    try {
        //simulate async
        setTimeout(function(){successCallback()}, 1);
    } catch(e) {
        //alert('PaperlitCommands.exportBase64AppImageContentToPhotos: exception');
    }  
};

// exports the provided base64 encoded data as an image to the photo album
Paperlit.exportBase64AppImageContentRectToPhotos = function(dataUrl, width, height, rectString, successCallback, errorCallback) {
    try {
        //simulate async
        setTimeout(function(){successCallback()}, 1);
    } catch(e) {
        //alert('PaperlitCommands.exportBase64AppImageContentToPhotos: exception');
    }
};
/* end mini app command */

// retrieves the path to the saved miniapp content
Paperlit.getPathToAppContentFile = function(successCallback, errorCallback) {
    try {
        //simulate async
        setTimeout(function(){errorCallback()}, 1);
    } catch(e) {
        //alert('PaperlitCommands.getPathToAppContentFile: exception');
    }  
};

Paperlit.isLocalUrl = function(url){
    return url.indexOf('://') == -1 || // no protocol, assuming file
        url.indexOf('file://') != -1 ||  // file protocol
        url[0] == '/' || // starting with slash (root, absolute url)
        url[0] == '.'; // starting with dot (current folder, relative url)
}

Paperlit.isRunningLocally = Paperlit.isLocalUrl(window.location.href);

Paperlit.loadScript = function(url, successCallback, errorCallback){
    var script = document.createElement("script")
    script.type = "text/javascript";
    script.onload = successCallback;
    script.onerror = errorCallback;
    script.src = url;
    document.getElementsByTagName("head")[0].appendChild(script);
}

Paperlit.isJavascriptInterfaceLoaded = false;
Paperlit.triggerJavascriptInterfaceLoaded = function(){
    if(Paperlit.isJavascriptInterfaceLoaded){
        return;
    }
    Paperlit.isJavascriptInterfaceLoaded = true;
    var evObj = document.createEvent('Events');
    evObj.initEvent('ppJavascriptInterfaceLoaded', false, true);
    document.dispatchEvent(evObj);
}

/* Load Cordova */
Paperlit._loadCordova = function(errorCallback){
	Paperlit.log.info('Paperlit._loadCordova: loading cordova');
    var cordovaFileName = 'cordova.js';
    if(Paperlit.isRunningLocally === false){
        // running remotely, we need to distinguish platforms
        cordovaFileName = Paperlit.isIOS ? 'cordova-ios.js' : 'cordova-android.js';
    }
    cordovaFileName = Paperlit.appRootPath + '/' + cordovaFileName;
    Paperlit.log.info('Paperlit._loadCordova: loading cordova from ' + cordovaFileName);
    // try to load cordova
    Paperlit.loadScript(cordovaFileName, function(){
    	Paperlit.log.info('Paperlit._loadCordova: loaded cordova from ' + cordovaFileName);
        Paperlit.isCordova = typeof cordova !== 'undefined';
        if(Paperlit.isCordova){
            Paperlit.log.info('Found ' + cordovaFileName);
            Paperlit.triggerJavascriptInterfaceLoaded();
            //in Cordova any event handler registered after the deviceready event fires has its callback function called immediately.
            document.addEventListener("deviceready", Paperlit.onDeviceReady, false);
            // old android browsers used state "loaded" instead of "complete".
            // we have to dispach a fake DOMContentLoaded event otherwise cordova does not start
            if(Paperlit.isAndroid && document.readyState == 'loaded'){
                Paperlit.log.info('dispaching fake DOMContentLoaded event');
                var evObj = document.createEvent('Events');
                evObj.initEvent('DOMContentLoaded', true, true);
                document.dispatchEvent(evObj);
            }
        }
    }, errorCallback);
};


Paperlit._loadPhoneGap = function(errorCallback){
    var phoneGapFileName = 'phonegap.js';
    if(Paperlit.isRunningLocally && Paperlit.isIOS){
        // load fixed version of phonegap.js 0.9.4 IOS from local library folder
        phoneGapFileName = 'lib/phonegap-ios-0.9.4.js';
    }

    phoneGapFileName = Paperlit.appRootPath + '/' + phoneGapFileName;
    Paperlit.log.info('Paperlit._loadPhoneGap: loading phonegap from ' + phoneGapFileName);
    // try to load phonegap
    Paperlit.loadScript(phoneGapFileName, function(){
        Paperlit.isPhoneGap = typeof PhoneGap !== 'undefined';
        if(Paperlit.isPhoneGap){
            Paperlit.log.info('Found phonegap.js');
            Paperlit.triggerJavascriptInterfaceLoaded();
            if(Paperlit.isDeviceReady){
                Paperlit.log.info('Phonegap device ready event already fired.');
                // device is already ready but we have lost the event because the handler has not been registered on time (could happen in ios phonegap 0.9.4)
                Paperlit.onDeviceReady();
            }
            else{
                Paperlit.log.info('Registered handler for phonegap device ready event');
                // register handler directly to onDeviceReady
                document.addEventListener("deviceready", Paperlit.onDeviceReady, false);
            }
            // if document is ready we could have missed DOMContentLoaded event
            if (Paperlit.isAndroid && PhoneGap.onDOMContentLoaded.fired === false && (document.readyState == 'complete' || document.readyState == 'interactive')) {
                Paperlit.log.info('Forcing PhoneGap.onDOMContentLoaded.fire on Android');
                PhoneGap.onDOMContentLoaded.fire();
            }
        }
    }, errorCallback);
};

/* DEPRECATED: USED ONLY BY REMOTE PAGES set some compatibility methods for android running on a remote page before cordova integration
   same code is available in the Paperlit.onDeviceReady method for android running local before cordova integration */
Paperlit._setupAndroidCompatibilityMode = function(){
    // get Product price from product id
    Paperlit.getProductPrice = function(productId, successCallback, errorCallback) {
        errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
        try {
            Paperlit.log.info('Paperlit.getProductPrice for Android: '+productId);
            var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
            // send command 
            Paperlit.gap.exec('getProductPrice/' + productId + '/' + functionsName.successFuncName + '/' + functionsName.errorFuncName);
        } catch(e) {
            Paperlit.log.error('exception calling PaperlitCommands.getProductPrice: '+productId);
            errorCallback();
        }

    };
    
    //TODO: be sure that the client does not crash if command is not supported
    Paperlit.social.share = function(platform, url, successCallback, errorCallback) {
        errorCallback = errorCallback != null ? errorCallback : Paperlit.noop;
        try {
            var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
            // send command 
            Paperlit.gap.exec('share/' + platform + '/' + encodeURIComponent(url) + '/' + functionsName.successFuncName + '/' + functionsName.errorFuncName);
        } catch(e) {
            errorCallback();
        }
    };
};

Paperlit._startAsBrowser = function(){
	// no native support, assuming browser
    Paperlit.log.info('No js interface found, assuming browser');
    var system = Paperlit.utils.getUrlParameter('platform');
    if(system == 'android'){
        Paperlit.isAndroid = true;
    }
    else if(system == 'ios'){
        Paperlit.isIOS = true;
    }
    if(typeof $ !== 'undefined'){
        // add class to declare that it is running in browser
        $('body').addClass('pp-browser');
    }

    // overwrite getProductPrice 
    Paperlit.getProductPrice = function(productId, successCallback, errorCallback){
        // not supported
        setTimeout(errorCallback, 1);
    };

    Paperlit.setNavigatorNotification();
    // start
    Paperlit.start();

    Paperlit.onCreate();
};

Paperlit.setNavigatorNotification = function(){
	navigator.notification = {};
    navigator.notification.confirm = function(message, callback, title, options){
        confirm(message);
    }

    navigator.notification.alert = function(message, callback, title, options){
        alert(message);
        if(callback){
        	callback();
        }
    }

    navigator.notification.prompt = function(message, callback, title, options){
        var response = prompt(message);
        callback({buttonIndex: 1, input1: response});
    }
}

Paperlit.setNavigatorConnection = function(){
	navigator.connection = {type: null};
	// emulate connection Cordova
	window.Connection = {
        UNKNOWN: "unknown",
        ETHERNET: "ethernet",
        WIFI: "wifi",
        CELL_2G: "2g",
        CELL_3G: "3g",
        CELL_4G: "4g",
        CELL:"cellular",
        NONE: "none"
	}
};

// do nothing if on browser
Paperlit.notifyClient = function(eventName){
    Paperlit.log.info('Paperlit.notifyClient (browser) ' + eventName);
    if(eventName == 'created'){
        // trigger on start
        Paperlit.onStart();
    }
};

Paperlit.notifyPublicationUpdate = function(publicationId, successCallback, errorCallback){
    errorCallback('not supported');
};

Paperlit.reload = function(){
	location.reload(true);
};

/* Download a content zipped package and return the path to the unzipped stuff */
Paperlit.updatePanelContent = function(url, maxAgeInSeconds, parameters){
    //if(Paperlit.isNative){
        // default is cached content
        maxAgeInSeconds = typeof maxAgeInSeconds === 'undefined' || maxAgeInSeconds == null ? 1000000000 : maxAgeInSeconds;
        //alert('updatePanelContent ' + maxAgeInSeconds + ', ' + url);
        Paperlit._newtworkTimeoutAlert.style.display = 'none';

        if(typeof Paperlit._panelContentNetworkTimeout !== 'undefined' && Paperlit._panelContentNetworkTimeout != null){
        	Paperlit.log.info('Paperlit.updatePanelContent: still loading');
        	return;
        }
        // set timeout for network
        Paperlit._panelContentNetworkTimeout = setTimeout(function(){
        	if(typeof Paperlit._lastPanelContentPath === 'undefined'){
        		// if never rendered
        		Paperlit._newtworkTimeoutAlert.innerHTML = 'Stiamo riscontrando lentezza nel download dei contenuti. Verifica la tua connessione.';
        		Paperlit._newtworkTimeoutAlert.style.display = 'block';
        	}
        }, 12 * 1000);
        //alert('look for content  ' + url);
        Paperlit.getCachedUrl(url, maxAgeInSeconds, function(cachedContentPath){
            if(typeof Paperlit._lastPanelContentPath === 'undefined' || (Paperlit.isNative && (Paperlit._lastPanelContentPath != cachedContentPath))){
                var oldPanelContentPath = Paperlit._lastPanelContentPath;
                Paperlit._lastPanelContentPath = cachedContentPath;
                // remove timeout
                if(typeof Paperlit._panelContentNetworkTimeout != 'undefined' && Paperlit._panelContentNetworkTimeout != null){
		        	clearTimeout(Paperlit._panelContentNetworkTimeout);
		        	Paperlit._panelContentNetworkTimeout = null;
		        }
                
                //alert('last =' + oldPanelContentPath + ', new=' + cachedContentPath + ' maxAgeInSeconds ' + maxAgeInSeconds + ' different ?' + (oldPanelContentPath != cachedContentPath));
                if(typeof oldPanelContentPath !== 'undefined'){

		            // content updated -> reload the same template in order to refresh the view
                    if(oldPanelContentPath != cachedContentPath){
                    	//alert('old '+ oldPanelContentPath + ' new ' + cachedContentPath); return;
                     	Paperlit.reload();
                    }
                    else{
                        alert('error updating');
                    }
                }
                else{
                	// do not show alert
                	
			        // be sure to hide newtwork timeout
			        Paperlit._newtworkTimeoutAlert.style.display = 'none';

                    Paperlit.log.info('Paperlit.updatePanelContent: received panel content update for url' + Paperlit._lastPanelContentPath);
                    //load panel parameters to be passed when updating content
                    Paperlit.getAppConfiguration(function(configuration){
                        if(typeof parameters !== 'undefined' && parameters != null){
                            // convert into object
                            parameters = JSON.parse(parameters);
                        }
                        Paperlit.log.info('Paperlit.updatePanelContent: Calling panel content update event with url ' + Paperlit._lastPanelContentPath);

                        if(typeof $ !== 'undefined'){
	                        $('body').off('templateRendered').on('templateRendered', function(){
	                        	if(typeof Paperlit._templateRenderingTimeout !== 'undefined' && Paperlit._templateRenderingTimeout != null){
	                        		clearTimeout(Paperlit._templateRenderingTimeout);
	                        		Paperlit._templateRenderingTimeout = null;
	                        	}
	                        	Paperlit.log.info('Paperlit.updatePanelContent: got templateRendered for url ' + Paperlit._lastPanelContentPath);
	                        	//Paperlit.alert('received templated rendered');
	                        	setTimeout(function(){
	                        		if(Paperlit.isAndroid){
	                        			// hack to force view render
	                        			document.body.scrollTop = 1;
        								document.body.scrollTop = 0;
        							}
		                        	Paperlit.addWebViewEventListener('ppOnCreate', function(){
						                Paperlit.notifyClient('created');
						                //setTimeout(Paperlit.reload, 10000);
						            });
						        }, 1);
	                        });
	                    }
	                    if(typeof Paperlit._templateLoader !== 'undefined'){
	                    	Paperlit._templateLoader.style.display = 'none';
	                    }
	                    
	                    // panels with content will wait till here before confirming that they are created
                        // e.g. to trigger splash fade out if it is the first panel
                        if(typeof Paperlit._templateRenderingTimeout === 'undefined'){
                        	Paperlit._templateRenderingTimeout = setTimeout(function(){
                        		Paperlit._templateRenderingTimeout = null;
	                        	Paperlit.addWebViewEventListener('ppOnCreate', function(){
	                        		Paperlit.log.info('Paperlit.updatePanelContent: force onCreate after 4s for url ' + Paperlit._lastPanelContentPath);
					                Paperlit.notifyClient('created');
					            });
					        }, 4000);
                        }

                        // notify panel that the content has changed
                        Paperlit.onPanelContentUpdate(Paperlit._lastPanelContentPath, parameters);
                        
                        // check content update when going on resume (only current panel)
		             	Paperlit.addWebViewEventListener('ppOnResume', function(){
		             		//alert('RESUME check content')
                    		Paperlit.log.info('Paperlit.updatePanelContent: onResume check if update is available');
			                Paperlit.checkIfPanelContentHasBeenUpdated();
			            });

		             	// check content update when going on pause, because we cannot get onresume except on first panel
			            Paperlit.addWebViewEventListener('ppOnPause', function(){
                    		Paperlit.log.info('Paperlit.updatePanelContent: onPause check if update is available');
			                Paperlit.checkIfPanelContentHasBeenUpdated();
			            });


                    }, Paperlit.noop);
                }
            }
            else{
                Paperlit.log.info('Paperlit.updatePanelContent: content is not updated');
            }

            // check if content has been updated
			setTimeout(Paperlit.checkIfPanelContentHasBeenUpdated, 1000 * 30);

        }, function(){
        	if(typeof Paperlit._panelContentNetworkTimeout != 'undefined' && Paperlit._panelContentNetworkTimeout != null){
	        	clearTimeout(Paperlit._panelContentNetworkTimeout);
	        	Paperlit._panelContentNetworkTimeout  = null;
	        }
	        if(typeof Paperlit._templateLoader !== 'undefined'){
            	Paperlit._templateLoader.style.display = 'none';
            }
	        Paperlit._newtworkTimeoutAlert.innerHTML = '<p onclick="Paperlit._templateLoader.style.display = \'block\'; Paperlit.getPanelContent();">Impossibile aggiornare il contenuto di questa pagina. Verifica la tua connessione di rete e fai tap qui per riprovare</p>';
	        // be sure to hide newtwork timeout
	        Paperlit._newtworkTimeoutAlert.style.display = 'block';

            Paperlit.log.error('updatePanelContent: error loading data');
        });

	
};

Paperlit._convertCustomParameters = function(parameters){
    try{
        if(parameters){
            // split
            parameters = parameters.split(',');
            var parameter;
            var parameterDictionary = {};
            for(var i=0, l=parameters.length; i<l; i++){
                parameter = parameters[i].split(':');
                parameterDictionary[parameter[0]] = parameter[1];
            }
            parameters = JSON.stringify(parameterDictionary);
            return parameters;
        }
    }
    catch(e){
    }
    return null;
}

Paperlit.getPanelContentInfo = function(callback){
    var getPanelContentInfo = function(state){
        var panelIndex;
        if(Paperlit.isNative){
            if(typeof state.newsstandPanelIndex === 'undefined'){
                // this app does not support panels
                Paperlit.log.info('Paperlit.getPanelContentInfo this app does not support panels');
                callback(false);
                return;
            }
            panelIndex = state.newsstandPanelIndex;
            if(Paperlit.isAndroid){
            	// fix a native bug
            	panelIndex = panelIndex - 1;
            }
            if(panelIndex == 0){
                panelIndex = 1;
            }
        }
        else{ 
            // browser simulation is on panel 1
            panelIndex = 1;
        }
        Paperlit.log.info('Paperlit.getPanelContentInfo panel ' + panelIndex);
        Paperlit.getAppConfiguration(function(configuration){
            var url = Paperlit.utils.getSetting(configuration, 'newsstand-panel-' + panelIndex + '-content-url');
            var parameters = Paperlit.utils.getSetting(configuration, 'newsstand-panel-' + panelIndex + '-parameters');
            parameters = Paperlit._convertCustomParameters(parameters);
            callback(url, parameters);
            
        }, Paperlit.noop);
    }
    try{
        Paperlit.log.info('Paperlit.getPanelContentInfo current url ' + window.location.href);
        var url = Paperlit.utils.getUrlParameter('contentUrl');
        if(url){
            // if we have a url
            var parameters = Paperlit.utils.getUrlParameter('customParameters');
            parameters = Paperlit._convertCustomParameters(parameters);
            callback(url, parameters);
        }
        else{
        	if(window.location.pathname.indexOf('templates/') == -1){
        		// this view cannot have content
        		return;
        	}
            if (typeof Paperlit.state !== 'undefined'){
                getPanelContentInfo(Paperlit.state);
            }
            else{
                Paperlit.getState(getPanelContentInfo, Paperlit.noop);
            }
        }
    }
    catch(e){
        Paperlit.log.error('Paperlit.getPanelContentInfo exception ' + e.message)
    }
}

/* Download a content zipped package from the url related to the panel and return the path to the unzipped stuff */
Paperlit.getPanelContent = function(){
    Paperlit.getPanelContentInfo(function(url, parameters){
    	if(url){
	        // get cached content 
	        // 25/06/2014 do not use cached content. Only fresh content
	        //Paperlit.updatePanelContent(url, null, parameters);
	        // check if new content (age 0)
	        Paperlit.updatePanelContent(url, 0, parameters);
	    }
    });
};

Paperlit.checkIfPanelContentHasBeenUpdated = function(){
	Paperlit.log.info('Paperlit.checkIfPanelContentHasBeenUpdated: start');
    Paperlit.getPanelContentInfo(function(url, parameters){
    	if(url){
        	// check if new content (age 0)
        	Paperlit.updatePanelContent(url, 0, parameters);
        }
    });
};

/* This is called by templates with pagination to communicate to the client at which page they are positionated (m page of n pages) */
Paperlit.onPageChange = function(m,n){
    // stub for the moment
    Paperlit.log.info('Paperlit.onPageChange ' + m + ' , ' + n);
}; 

Paperlit._prefetchUrls = [];

Paperlit.prefetchContents = function(){
	// wait before prefetching
	setTimeout(function(){
		//only in wifi
		if(Paperlit.isNative === false || navigator.connection.type === Connection.WIFI){
		    // scan for elements with attribute pp-prefetch and prefetch url
		    var prefetchElements = document.querySelectorAll('[pp-prefetch]');
		    Paperlit.log.info('Paperlit.prefetchContents: prefetching ' + prefetchElements.length + ' elements');
		    for(var i=0, l=prefetchElements.length; i<l; i++){
		        var prefetchElement = prefetchElements[i];
		        var url = prefetchElement.getAttribute('pp-prefetch');
		        if(url.indexOf('paperlit://') != -1){
		        	// skip paperlit read
		        	continue;
		        }
		        Paperlit._prefetchUrls.push(url);
		    }
		    Paperlit._prefetchContents();
		}
		else{
			 Paperlit.log.info('Paperlit.prefetchContents: no prefetch because connection is not wifi');
		}
	}, 3000);
};

Paperlit._prefetchContents = function(){
	if(Paperlit.isNative === false || navigator.connection.type === Connection.WIFI){
		if(Paperlit._prefetchUrls.length > 0){
			var url = Paperlit._prefetchUrls.shift();
			Paperlit.log.info('Paperlit.prefetchContents: prefetching ' + url);
			// cache contents
	        Paperlit.getCachedUrl(url, 1000000000, Paperlit._prefetchContents, Paperlit._prefetchContents);
		}
	}
};

Paperlit.close = function(){
    Paperlit.openUrl('paperlit://close');
};


Paperlit.setUser = function(user, successCallback, errorCallback){
	Paperlit.readWriteProperties(function(properties){
		properties.user = user;
		setTimeout(successCallback, 1, {result: "success"});
		return {needToWrite: true, properties: properties};
	});
};

Paperlit.getUser = function(successCallback, errorCallback){
	Paperlit.readWriteProperties(function(properties){
		if(typeof properties.user !== 'undefined'){
			successCallback({result: "success", user: properties.user});
		}
		else{
			successCallback({result: "error"})
		}
		return {needToWrite: false, properties: properties};
	});
};

Paperlit.removeUser = function(successCallback, errorCallback){
	Paperlit.readWriteProperties(function(properties){
		if(typeof properties.user !== 'undefined'){
			delete properties.user;
			successCallback({result: "success"});
		}
		else{
			successCallback({result: "success"})
		}
		return {needToWrite: true, properties: properties};
	});
};

Paperlit.rcs = {};

Paperlit.rcs.isSubscribed = function(successCallback, errorCallback){
	Paperlit.getAppConfiguration(function(appConfiguration){
		var authorizationUrl = Paperlit.utils.getSetting(appConfiguration, 'billing-authorization-url');
	  	Paperlit.replaceUrlParameters(authorizationUrl, function(replacedUrl){
            try{
            	// replace userId, rcs appId
            	var rcsAppId = Paperlit.utils.getSetting(appConfiguration, 'billing-rcs-appid');
            	replacedUrl = replacedUrl.replace('{rcsAppId}', rcsAppId);
            	Paperlit.getUser(function(response){
            		if(response.result == 'success'){
            			replacedUrl = replacedUrl.replace('{userId}', response.user.userId);
            			$.ajax({
		            		url: replacedUrl,
		            		success: function(xml){
		            			var subscriptions = $(xml).find('subscription');
		            			if(subscriptions.length > 0){
		            				successCallback({result: "success", suscriptions: subscriptions});
		            			}
		            			else{
		            				successCallback({result: "error", message :"not subscribed"})
		            			}
		            		},
		            		error: function(){
		            			successCallback({result: "error"});
		            		},
		            		dataType: 'xml'
		            	})
            		}
            		else{
            			successCallback({result: "error"});
            		}
            		
            	}, function(){
            		successCallback({result: "error"});
            	});
            	
            }
            catch(e){
            	successCallback({result: "error"});
            }
        });
	});
};


Paperlit.rcs.isLogged = function (successCallback, errorCallback) {
	if(typeof $ === 'undefined'){
		Paperlit.alert('Missing jquery');
		return;
	}
	Paperlit.getAppConfiguration(function(configuration){
		var url = Paperlit.utils.getSetting(configuration, 'billing-inapp-services-base-url') + 'islogged';	
		Paperlit.getState(function(state){
			var rcsAppId = '';
			if(typeof state.rcsAppId != 'undefined'){
				// android has app id in state
				rcsAppId = state.rcsAppId;
			}
			else{
				rcsAppId = Paperlit.utils.getSetting(configuration, 'billing-rcs-appid');
			}
			$.ajax(
				{
					url: url, 
					data: "appid=" + rcsAppId, 
				    headers: { 
				        "Accept" : "application/json",
				        "Content-Type": "application/json"
				    },
					success: function(data, textStatus, jqXHR) {
						if (typeof data.logged != 'undefined' && data.logged == 'false'){
							errorCallback(data);
							return;
						}
						if (typeof data.root != 'undefined' && typeof data.root.ssourl != 'undefined' && data.root.ssourl != null) {
							$.ajax({
								url: data.root.ssourl,
								success: function(){
									successCallback(data);
								},
								error: errorCallback
							});
						}
						else{
							if (successCallback != undefined){
								successCallback(data);
							}
						}
						
					},
					error: function(jqXHR, textStatus, errorThrown) {
						var data = {error: "generic"}; 
						try {
							data = $.parseJSON(jqXHR.responseText);					
						} catch (e) {
							//nothing
						}
						if (errorCallback != undefined){
							errorCallback(data);
						}
					}
				}
			);
		});
	});
}

Paperlit.rcs.autoLogin = function(successCallback, errorCallback){
	Paperlit.rcs.isLogged(function(user){
		user.userId = "" + user.root.runaid;
		user.userEmail = user.root.email;

		// set user
		Paperlit.setUser(user, successCallback, errorCallback);
	}, errorCallback);
}

Paperlit.getCachedAdvUrl = Paperlit.getCachedUrl;

Paperlit.getCachedAdvUrlAsString = function(url, maxAgeInSeconds, successCallback, errorCallback){
	Paperlit.log.info('Paperlit.getCachedAdvUrlAsString: required url ' + url);
	Paperlit.getCachedAdvUrl(url, maxAgeInSeconds, function(cachedUrl){
		Paperlit.log.info('Paperlit.getCachedAdvUrlAsString: got cached url ' + cachedUrl + ' loading text ');
		// load from cached file
		Paperlit.getCachedUrlAsString(cachedUrl, 0, successCallback, errorCallback);	
	}, errorCallback); 
}

Paperlit.rcsAdv = {};

/* load adv url as a string via cache after setting random number */
Paperlit.rcsAdv.getAdv = function(key, callback, show, maxAgeInSeconds, doNotPreload){
	// wait ppReady to use adv cache
	Paperlit.addWebViewEventListener('ppReady', function(){
		doNotPreload = typeof doNotPreload === 'undefined' ? false : doNotPreload;
		Paperlit.log.info('Paperlit.rcsAdv.getAdv: asked for key ' + key);

		Paperlit.getAppConfiguration(function(appConfiguration){
			var isAdvEnabled = Paperlit.utils.getSetting(appConfiguration, 'advertising-rcs-enabled');
			if(isAdvEnabled === false){
				callback(false);
				return;
			}

		  	Paperlit.getState(function(appState){
		  		var panelUrl = Paperlit.utils.getDeviceSettingSync(appConfiguration, appState, 'newsstand-panel-' + appState.newsstandPanelIndex + '-url');
		  		// dashboard has adv only on 2nd page
		  		if(panelUrl != null && panelUrl.indexOf('dashboard') != -1){
		  			Paperlit.log.info('Paperlit.rcsAdv.getAdv: this is a dashboard panel');
		  			// this is a dashboard page
		  			var panelMinus1 = Paperlit.utils.getDeviceSettingSync(appConfiguration, appState, 'newsstand-panel-' + (appState.newsstandPanelIndex-1) + '-url');
		  			var panelMinus2 = Paperlit.utils.getDeviceSettingSync(appConfiguration, appState, 'newsstand-panel-' + (appState.newsstandPanelIndex-2) + '-url');
		  			// if panel -1 is a dashboard panel a and panel-2 does not exist or is not a dashboard -> this is the second dashboard panel
		  			if(panelMinus1 && panelMinus1.indexOf('dashboard') != -1 && (panelMinus2 == null || panelMinus2.indexOf('dashboard') == -1)){
		  				Paperlit.log.info('Paperlit.rcsAdv.getAdv: this is the second dashboard panel. Look for adv');
		  			}
		  			else{
		  				Paperlit.log.info('Paperlit.rcsAdv.getAdv: this is NOT second dashboard panel.');
		  				// this is not the dashbaord second page -> no adv
		  				callback(false);
		  				return;
		  			}
		  		}

		  		var currentPath = Paperlit.isIOS ? window.location.pathname : window.location.href;
		  		//M510, M310, M001 on phone, M300 phone, M500 directly on openX
				if(currentPath.indexOf('M510') != -1 || currentPath.indexOf('M310') != -1 || (currentPath.indexOf('M001') != -1 && appState.deviceClass == 'phone') || (currentPath.indexOf('M300') != -1 && appState.deviceClass == 'phone') || currentPath.indexOf('M500') != -1){
					Paperlit.log.info('Paperlit.rcsAdv.getAdv: ' + currentPath + ' go directly to openx');
					// real media does not support this positions so go directly to openX
					Paperlit.rcsAdv.getOpenXPosition(callback);
					return;
				}


				maxAgeInSeconds = typeof maxAgeInSeconds === 'undefined' ? 1000000000 : maxAgeInSeconds;
				// show
				show = typeof show === 'undefined' ? true : show;
				var platformKey = key + '-' + (Paperlit.isIOS ? 'ios' : 'android')
				Paperlit.log.info('Paperlit.rcsAdv.getAdv: asked for key platform ' + platformKey);
				// get settings for platform, deviceClass eg. ios-tablet
				Paperlit.utils.getDeviceSetting(appConfiguration, platformKey, function(url){
					if(url){
						Paperlit.log.info('Paperlit.rcsAdv.getAdv: asked key ' + platformKey + ' has been mapped to ' + url);
						// only preload
						// replace random number
						url = url.replace('RANDOMNUMBER', appState.randomNumber);
						// load adv with high expiration has the url is invalidated by the random number and there could be concurrent calls to the same
						// url by both the preloader and the real adv position
						Paperlit.log.info('preloading position ' + key + ' with url ' + url);

						Paperlit.getCachedAdvUrlAsString(url, maxAgeInSeconds, Paperlit.rcsAdv._advLoaded(key, url, show, callback, doNotPreload), function(){
							Paperlit.log.error('error caching adv url ' + url);
							callback(false);
						}, false);
					}
					else{
						Paperlit.log.info('Paperlit.rcsAdv.getAdv: asked key ' + platformKey + ' has no mapping in configuration');
					}
				});
				
			}, function(){
				Paperlit.log.error('cannot get state');
				callback(false);
			});
		}, function(){
			Paperlit.log.error('cannot get configuration');
			callback(false);
		});
	});
}

Paperlit.rcsAdv.getOpenXPosition = function(callback){
	Paperlit.getState(function(state){
		// check if fallback is provided
		var currentPath = Paperlit.isIOS ? window.location.pathname : window.location.href;
		var fallbackUrl = null;
		if(currentPath.indexOf('M001') != -1){
			if(state.deviceClass == 'tablet'){
				fallbackUrl = 'http://ads.rcs.it/www/delivery/afr.php?cb=<random_number>&zoneid=1453';
			}
			else{
				fallbackUrl = 'http://ads.rcs.it/www/delivery/afr.php?cb=<random_number>&zoneid=1517';
			}	
		}
		else if(currentPath.indexOf('M510') != -1){
			fallbackUrl = 'http://ads.rcs.it/www/delivery/afr.php?cb=<random_number>&zoneid=1454';	
		}
		else if(currentPath.indexOf('M310') != -1){
			if(state.deviceClass == 'tablet'){
				fallbackUrl = 'http://ads.rcs.it/www/delivery/afr.php?cb=<random_number>&zoneid=1455';
			}
			else{
				fallbackUrl = 'http://ads.rcs.it/www/delivery/afr.php?cb=<random_number>&zoneid=1521';
			}
		}
		else if(currentPath.indexOf('M500') != -1){
			fallbackUrl = 'http://ads.rcs.it/www/delivery/afr.php?cb=<random_number>&zoneid=1519';
		}
		else if(currentPath.indexOf('M300') != -1){
			if(state.deviceClass == 'tablet'){
				fallbackUrl = 'http://ads.rcs.it/www/delivery/afr.php?cb=<random_number>&zoneid=1515';
			}
			else{
				fallbackUrl = 'http://ads.rcs.it/www/delivery/afr.php?cb=<random_number>&zoneid=1518';
			}
		}
		else if(currentPath.indexOf('videowall') != -1){
			fallbackUrl = 'http://ads.rcs.it/www/delivery/afr.php?cb=<random_number>&zoneid=1516'
		}

		if(fallbackUrl == null){
			Paperlit.log.info('Paperlit.rcsAdv.getOpenXPosition: no fallback');
			callback(false);
		}
		else{
			Paperlit.log.info('Paperlit.rcsAdv.getOpenXPosition: getting openx ' + fallbackUrl);
			fallbackUrl = fallbackUrl.replace('<random_number>', (Math.floor(Math.random() * (100000000000 - 1 + 1)) + 1));
			Paperlit.getCachedUrlAsString(fallbackUrl, 0, function(advString){
					var html = document.createElement('div');
					html.innerHTML = advString;

					var currentPath = Paperlit.isIOS ? window.location.pathname : window.location.href;
					if(currentPath.indexOf('apertura') != -1){
						// pass image url instead of html
						var img = html.getElementsByTagName('img');
						if(img.length > 0){
							Paperlit.log.info('Paperlit.rcsAdv.getOpenXPosition: apertura url ' + img[0].src);
							callback(img[0].src);
						}
						else{
							callback(false);
						}
						return;
					}

					var styles = html.getElementsByTagName('style');
					for(var i=0, l=styles.length; i<l; i++){
						var style = styles[i];
						style.parentNode.removeChild(style);
					}
					

					var links = html.getElementsByTagName('a');
					if(links.length > 0){
						// unset all other links
						for(var i=0, l=links.length; i<l; i++){
							link = links[i];
							if(link.getAttribute('href')){
								link.addEventListener('touchstart', function(e){
									e.stopPropagation();
								});

								link.addEventListener('click', function(e){
									e.stopPropagation();
									e.preventDefault();
									var link = this;
									var href = link.getAttribute('pp-href');
									var isSpecialurl = false;
									//http://ads.rcs.it/www/delivery/ck.php?oaparams=2__bannerid=10499__zoneid=1516__cb=fb054867b9__oadest=paperlit%3A%2F%2Fnewsstand
									if(href.indexOf('delivery/ck.php?') != -1){
										var url = href.replace(/__/g, '&');
										url = Paperlit.utils.getParameterFromUrl(url, 'oadest');
										href = url;
										isSpecialurl = true;
										href = href.replace('paperlit://read/Formula+1+2014/', 'paperlit://read/gazzsportspeciali/');
										//alert('opening ' + href);
									}
									try{
										Paperlit.log.info('tapped on openX link ' + href +  ' is online = ' + (typeof navigator.connection === 'undefined' ? 'true' : navigator.connection.type !== Connection.NONE ? 'true' : 'false'));
										// if the link is clickable and the device is online, open the url in a fullscreen window
										if(href && (Paperlit.isNative === false || navigator.connection.type !== Connection.NONE)){
											if(!isSpecialurl){
												// open in browser
												if(href.indexOf('#') == -1){
											        href = href + '#';
											    }
											    else{
											        href = href + '&';
											    }
										    	href = href + 'ppbt=browser';
										    }
											Paperlit.openUrl(href);
										}
									}
									catch(e){
										Paperlit.log.error('exception ' + e.message);
									}	
								});
								link.setAttribute('pp-href', link.getAttribute('href'));
								link.removeAttribute('href');
							}
						}
					}
					Paperlit.log.info('Paperlit.rcsAdv.getOpenXPosition: html ' + html);
					callback(html);
			}, function(){
				callback(false);
			}, false);
		}
	});
};

/* look for script in html response, execute all script except the one with "Cliccabile" string 
		   For this one, it preloads the image via cache, then replace the image url with the cached url and then call a callback */
Paperlit.rcsAdv._advLoaded = function(key, url, show, myCallback, doNotPreload){
	return function(advString){
		var callback = myCallback;

		Paperlit.log.info('preloaded position ' + key + ' for url ' + url + ' content = ' + advString);
		try{
			// parse to grab images and video
			var html = document.createElement('div');
			html.innerHTML = advString;
			var that = this;
			var cachingAssets = false;
			var scripts = html.getElementsByTagName('SCRIPT');

			var skipThisScript = -1;

			// process all html elements
			for(var i=0, l=scripts.length; i<l; i++){
				var script = scripts[i];
				var scriptText = script.text;
				// check if it is the script that will set the adv
				if(scriptText.indexOf('_capimg') != -1){
					Paperlit.log.info('found _capimg');
					Paperlit.log.info('_capimg');
					eval.call(window, scriptText);
				
					var newCallback = function(value){
						// this will erogate the cookied for the campaign
						if(scriptText.indexOf('x40_capimg') != -1){
							Paperlit.log.info('waiting for x40_capimg');
							if(Paperlit.isAndroid){
								Paperlit.log.info('loading cookie url ' + x40_capimg.src);
								// to be sure that the cookie will be set load image using getCacheUrl
								Paperlit.getCachedUrl(x40_capimg.src, 0, function(cachedUrl){
									Paperlit.log.info('loaded cookie url ' + cachedUrl);
									myCallback(value);
								});
							}
							else if(x40_capimg.complete){
								Paperlit.log.info('x40_capimg completed');
								myCallback(value);
							}
							else{
								x40_capimg.addEventListener('load', function(){
									Paperlit.log.info('x40_capimg loaded');
									myCallback(value);
								})
							}
						}
						else{
							if(Paperlit.isAndroid){
								Paperlit.log.info('loading cookie url ' + x30_capimg.src);
								// to be sure that the cookie will be set load image using getCacheUrl
								Paperlit.getCachedUrl(x30_capimg.src, 0, function(cachedUrl){
									Paperlit.log.info('loaded cookie url ' + cachedUrl);
									myCallback(value);
								});
							}
							else if(x30_capimg.complete){
								Paperlit.log.info('x30_capimg completed');
								myCallback(value);
							}
							else{
								x30_capimg.addEventListener('load', function(){
									Paperlit.log.info('x30_capimg loaded');
									myCallback(value);
								})
							}
						}
					}
					// replace callback
					callback = newCallback;
					// remove this script otherwise it will be executed twice in the next loop
					skipThisScript = i;
					//scripts.splice(i, 1);
					// jump out
					break;
				}
			}

			// process all html elements
			for(var i=0, l=scripts.length; i<l; i++){
				var script = scripts[i];
				
				var scriptText = script.text;
				if (i == skipThisScript){
					Paperlit.log.info('Paperlit.rcsAdv._advLoaded: skipping script ' + i + ' with text ' + scriptText);
					continue;
				}
				
				// check if it is the script that will set the adv
				if(scriptText.indexOf('oas_img') != -1){
					// remove code that will show the image
					scriptText = scriptText.replace('document.write(oas_img);', '');
					// eval script in global scope
					eval.call(window, scriptText);
					// cache asset image
					Paperlit.log.info('found asset to cache for ' + key + ' with url ' + url + ' asset url = ' + oas_urlimg);
					Paperlit.getCachedUrl(oas_urlimg, 1000000000, Paperlit.rcsAdv._gotCachedAsset(key, url, show, callback, doNotPreload), function(){
						Paperlit.log.error('Paperlit.rcsAdv._advLoaded: error caching asset for key ' + key);
						Paperlit.log.error('Paperlit.rcsAdv._advLoaded: calling callback ' + callback);
						callback(false);
					});
					cachingAssets = true;
				}
				else if(show === false){
					// execute script only while preloading
					var script = document.createElement("script")
				    script.type = "text/javascript";
				    script.text = scriptText;
				    document.getElementsByTagName("head")[0].appendChild(script);
				}
				
			}
			// no asset found to show
			if(!cachingAssets){

				Paperlit.log.info('Nothing to cache for key ' + key + ' with url ' + url);
				if(show){
					Paperlit.rcsAdv.getOpenXPosition(callback);
				}
				else{
					callback(false);
				}
				
				return;
				
				// DEMO BANNER
				window.oas_img = '<a href="http://www.paperlit.com" target=_blank><div><img src="pippo.jpg" BORDER=0></a></div>';
				// make it cliccable
				window.Cliccabile = true;
				Paperlit.getCachedUrl('http://paperlit.com/site/images/paperlit_logo.png', 1000000000, Paperlit.rcsAdv._gotCachedAsset(key, url, show, callback, doNotPreload), function(){
					callback(false);
				});
			}
		}
		catch(e){
			Paperlit.log.error('Paperlit.rcsAdv._advLoaded: Exception ' + e.message);
			callback(false);
		}
	};
}

/* 	
	show adv if it is not a transparent pixel, if the client is online OR if it is offline and the campaign is not expired (EndDate) 
	After DurataAnimazione sec it sends Paperlit.close.
	Replace href of links with a check that opens the link in a browser view only if the client is online
	If the adv is an interstitial then it preloads the next adv while showing the current one
*/
Paperlit.rcsAdv._gotCachedAsset =  function(key, url, show, callback, doNotPreload){
	return function(cachedUrl){
		try{
			Paperlit.log.info('cached asset for ' + key);
			if(show){
				if(Paperlit.isNative){
					Paperlit.log.info('Connection ' + navigator.connection.type + 
						(navigator.connection.type !== Connection.NONE ? ' ONLINE' : ' OFFLINE'));
				}
				// check if we need to show this: 
				// 1) there is a creativity
				// 2) online or offline and not expired
				/*if(oas_file_gif == "spacer.gif"){
					Paperlit.log.info('show fake adv');
					if(key.indexOf('advertising-rcs-interstitial') == 0){
						callback('<img src="http://www.fuorisalone.it/2008/files/fs0700069_banner_nokia.jpg" style="width:100%; height: 100%">');
					}
					else{
						callback('<img src="http://www.fuorisalone.it/2008/files/fs0700069_banner_nokia.jpg">');
					}
					return;
				}*/
				var hasToShow = (Paperlit.isNative === false || (navigator.connection.type !== Connection.NONE) ||  
					((navigator.connection.type === Connection.NONE) && ((new Date()).getTime() <= (new Date(EndDate)).getTime())));

				// check if ghost or blank pixel 
				if(hasToShow){
					// for adv opening we simply return image url
					// NOT USED ANYMORE BY REPLY
					/*if(key.indexOf('opening') != -1){
						// simply return image url as string
						callback(cachedUrl);
						return;
					}*/

					Paperlit.log.info('Paperlit.rcsAdv._gotCachedAsset: preparing adv to show for '+ key + ' oas img ' + oas_img); 
					// get link
					var doc = document.createElement('div');
					doc.setAttribute('data-type', 'adv');
					doc.innerHTML = oas_img;
					var images = doc.getElementsByTagName('img');
					if(images.length > 0){
						var image = images[0];
						// replace source with cached url
						image.src = cachedUrl;
						var link = image.parentNode;
						Paperlit.log.info('Paperlit.rcsAdv._gotCachedAsset: replacing image'); 
						if(link.nodeName.toLowerCase() == 'a'){
							// found link
							if(link.getAttribute('href')){
								Paperlit.log.info('Paperlit.rcsAdv._gotCachedAsset: replacing link ' + link.getAttribute('href')); 
								link.ppHref =  link.getAttribute('href');
								link.removeAttribute('href');

								link.addEventListener('touchstart', function(e){
									e.stopPropagation();
								});

								link.addEventListener('click', Paperlit.proxy(function(e){
									e.stopPropagation();
									e.preventDefault();
									var link = this;
									var href = link.ppHref;
									Paperlit.log.info('Paperlit.rcsAdv._gotCachedAsset: tap ' + href); 
									try{
										Paperlit.log.info('tapped on link ' + href + ' is clickable = ' + (typeof Cliccabile == 'undefined' ? 'not specified assuming YES ' : Cliccabile) + ' is online = ' + (typeof navigator.connection === 'undefined' ? 'true' : navigator.connection.type !== Connection.NONE ? 'true' : 'false'));
										// if the link is clickable and the device is online, open the url in a fullscreen window
										if((typeof Cliccabile === 'undefined' || Cliccabile == true) && href && (Paperlit.isNative === false || navigator.connection.type !== Connection.NONE)){
											// open in browser
											if(href.indexOf('#') == -1){
										        href = href + '#';
										    }
										    else{
										        href = href + '&';
										    }
										    href = href + 'ppbt=browser';
											Paperlit.openUrl(href);
										}
									}
									catch(e){
										Paperlit.log.error('exception ' + e.message);
									}	
								}, link));
							}
						}

						var links = doc.getElementsByTagName('a');
						if(links.length > 0){
							Paperlit.log.info('Paperlit.rcsAdv._gotCachedAsset: removing links'); 
							// unset all other links
							for(var i=0, l=links.length; i<l; i++){
								link = links[i];
								if(link.getAttribute('href')){
									link.removeAttribute('href');
								}
							}
						}

						Paperlit.log.info('Paperlit.rcsAdv._gotCachedAsset: returning adv for ' + key + ' - ' + doc.innerHTML); 

							
						// add handler to ppResume to refresh campaign
						Paperlit.addWebViewEventListener('ppOnAdvReady', Paperlit.proxy(function(advDiv, key){
							try{
								Paperlit.log.info('Paperlit.rcsAdv._gotCachedAsset: ppOnAdvReady handler require key ' + key);
								advDiv.style.display = "none";
								Paperlit.rcsAdv.getAdv(key, Paperlit.proxy(function(oldAdvDiv, advDivPosition){
									try{
										Paperlit.log.info('Paperlit.rcsAdv._gotCachedAsset: re add old ' + Paperlit.getHTML(oldAdvDiv, true) + ' new ' +Paperlit.getHTML(advDivPosition, true) );
										// swap
									    var oldAdvDivParent = oldAdvDiv.parentNode;
									    Paperlit.log.info('Paperlit.rcsAdv._gotCachedAsset: parent is ' + oldAdvDivParent + ' is equal ' + ( oldAdvDiv == oldAdvDivParent ) + ' with content ' + Paperlit.getHTML(oldAdvDivParent, true));
									    Paperlit.log.info('Paperlit.rcsAdv._gotCachedAsset: removed old adv, granny is ' + oldAdvDivParent.parentNode + ' type of node is ' + Paperlit.getHTML(oldAdvDivParent.parentNode));
									    // add before
									    oldAdvDivParent.insertBefore(advDivPosition, oldAdvDiv);
									    // remove old
									    oldAdvDivParent.removeChild(oldAdvDiv);
									}
									catch(e){
										Paperlit.log.error('Paperlit.rcsAdv._gotCachedAsset: exception in ppOnAdvReady getAdv method ' + e.message);
									}
								}, this, advDiv));
							}
							catch(e){
								Paperlit.log.error('Paperlit.rcsAdv._gotCachedAsset: exception in ppOnAdvReady handler ' + e.message);
							}
							
						}, this, doc, key), true);
				
						// return adv first time the panel is resumed
						callback(doc);
						
						// count impression calling again the same position
						if(doNotPreload === false){
							Paperlit.log.info('Paperlit.rcsAdv._gotCachedAsset: counting impression for ' + key);
							// do not show, force 0 age in order to let the call go out to network, do not preload again
							Paperlit.rcsAdv.getAdv(key, function(){
								Paperlit.log.info('Paperlit.rcsAdv._gotCachedAsset: preloaded next for key ' + key);
							}, false, 0, true);
						}
						return;
					}
				}
				else{
					Paperlit.log.info('showAdv: nothing to show. Close ');
				}
			}
		}
		catch(e){
			Paperlit.log.error('Paperlit.rcsAdv._gotCachedAsset: Exception ' + e.message);
		}
		Paperlit.log.info('Paperlit.rcsAdv._gotCachedAsset: end, call function ' + callback);
		callback(false);
	}
};


Paperlit.getHTML = function(who, deep){
    if(!who || !who.tagName) return '';
    var txt, ax, el= document.createElement("div");
    el.appendChild(who.cloneNode(false));
    txt= el.innerHTML;
    if(deep){
        ax= txt.indexOf('>')+1;
        txt= txt.substring(0, ax)+who.innerHTML+ txt.substring(ax);
    }
    el= null;
    return txt;
}

Paperlit.getOpenLastIssueUrl = function(publicationId, callback){
	Paperlit.getIssuesWithProductIdAsJSON(publicationId, 1, false, function(issuesJSON){
		/*var currentDate = new Date();
		function pad(number) {
			var r = String(number);
			if (r.length === 1) {
				r = '0' + r;
			}
			return r;
	    }
	    if(currentDate.getUTCHours() < 5){
	    	// new issue is published at 5
	    	// go to yesterday	
	    	currentDate.setDate(currentDate.getDate()-1);
	    }
		var issueId = currentDate.getUTCFullYear()
	        + '-' + pad(currentDate.getUTCMonth() + 1)
	        + '-' + pad(currentDate.getUTCDate());*/
	    var publicationId = issuesJSON.issues[0].publicationId;
	    var issueId = issuesJSON.issues[0].issueId;

	    callback('paperlit://read/' + publicationId + '/' + issueId);
	});

  	
};

Paperlit.registerForPushNotifications = function(json, successCallback, errorCallback){
	Paperlit._testPush = json;
	setTimeout(successCallback, 10);
};

Paperlit.getRegisteredPushNotifications = function(successCallback, errorCallback){
	if(typeof Paperlit._testPush !== 'undefined'){
		setTimeout(successCallback, 100, {result: "success", data: Paperlit._testPush});
	}
	else{
		setTimeout(successCallback, 100, {result: "error", error: 'no push'});
	}
};


Paperlit.init = function(){
	
    Paperlit.isDeviceReady = false;
    Paperlit.isPhoneGap = typeof PhoneGap !== 'undefined';
    Paperlit.isCordova = typeof cordova !== 'undefined';


    // check if some native interface has been already loaded by script tag in the page
    if(Paperlit.isPhoneGap || Paperlit.isCordova){
        Paperlit.log.info('Js interface loaded');
        // attach handler
        document.addEventListener("deviceready", Paperlit.onDeviceReady, false);
    }
    else{
        Paperlit.log.info('Loading js interface');
        
        if(Paperlit.isAndroid){
        	Paperlit.log.info('Android, look for js bridge');
        	Paperlit.loadScript(Paperlit.appRootPath + '/gap.js', Paperlit.setupAndroidGapProtocol, function(){
        		Paperlit._loadCordova(Paperlit._startAsBrowser);
		    });
        }
        else{
        	Paperlit.log.info('ios, look for js bridge');
	        Paperlit._loadCordova(Paperlit._startAsBrowser);
       	}
    }
}; // end init

Paperlit.setupAndroidGapProtocol = function(){
	console.log('android do not use cordova');
	Paperlit.isGap = true;

	cordova = {}; // fake to recognize cordova
	//
	cordova.exec = function(successCallback, errorCallback, service, action, parameters){
		// covert into paperlit://gap
		//cordova.exec(successCallback, errorCallback, service, action, parameters); 
		try {
			var functionsName = Paperlit.getFunctionsName(successCallback, errorCallback);
			// send command 
			console.log('android gap/' + action + ' calling');
			// set method name
			var gapCommand = action + '/';
			// append paramters
			for(var i=0, l=parameters.length; i<l; i++){
				var parameter = parameters[i];
				if(parameter != null){
					if(typeof parameter != 'string'){
						parameter = JSON.stringify(parameter);
					}
					if(parameter != ''){
						gapCommand = gapCommand + encodeURIComponent(parameter) + '/';
					}
				}
			}
			// append callbacks
			gapCommand = gapCommand + functionsName.successFuncName + '/' + functionsName.errorFuncName;
			// call command
			Paperlit.gap.exec(gapCommand);
		} catch(e) {
			console.log('error android gap/' + action + ': ' + e.message);
			errorCallback();
		}
	}
	// set navigator namespace as fallback for missing implementation
	Paperlit.setNavigatorConnection();
	Paperlit.onConnectionChange = function(connectionType){
		navigator.connection.type = connectionType;
	};

	navigator.notification = {};

	navigator.notification.alert = function(message, callback, title, options){
        try {
            Paperlit.log.info('Paperlit.alert');
            cordova.exec(callback, Paperlit.noop, 'paperlit', 'alert', [message, title, "Ok"]);
        } catch(e) {
            Paperlit.log.error('exception calling PaperlitCommands.alert: '+ e.message);
        }
    };

    navigator.notification.prompt = function(message, callback, title, options, defaultValue){
        try {
            Paperlit.log.info('Paperlit.prompt');
            cordova.exec(callback, Paperlit.noop, 'paperlit', 'prompt', [message, title, options, defaultValue]);
        } catch(e) {
            Paperlit.log.error('exception calling PaperlitCommands.prompt: ' + e.message);
        }
    };

    navigator.notification.confirm = function(message, callback, title, options){
        try {
            Paperlit.log.info('Paperlit.confirm');
            cordova.exec(callback, Paperlit.noop, 'paperlit', 'confirm', [message, title, options]);
        } catch(e) {
            Paperlit.log.error('exception calling PaperlitCommands.confirm: ' + e.message);
        }
    };

	// trigger on device ready
	Paperlit.onDeviceReady();
};

/* Adobe DPS Api */
var adobeDPS = (function () {

}());

/* Track event triggering */
Paperlit.events = {};

Paperlit.onCreate = function(){
    if(typeof Paperlit.events['ppOnCreate'] === 'undefined'){
        Paperlit.events.ppOnCreate = true;
    }
    Paperlit.log.info('Paperlit.onCreate');
    var evObj = document.createEvent('Events');
    evObj.initEvent('ppOnCreate', true, true);
    document.dispatchEvent(evObj);
};

Paperlit.onStart = function(){
    if(typeof Paperlit.events['ppOnStart'] === 'undefined'){
        Paperlit.events.ppOnStart = true;
    }
    Paperlit.log.info('Paperlit.onStart');
    var evObj = document.createEvent('Events');
    evObj.initEvent('ppOnStart', true, true);
    document.dispatchEvent(evObj);
};

Paperlit.onStop = function(){
    if(typeof Paperlit.events['ppOnStop'] === 'undefined'){
        Paperlit.events.ppOnStop = true;
    }
    Paperlit.log.info('Paperlit.onStop');
    var evObj = document.createEvent('Events');
    evObj.initEvent('ppOnStop', true, true);
    document.dispatchEvent(evObj);
};

Paperlit.onResume = function(){
	if(typeof Paperlit.events['ppOnResume'] === 'undefined'){
        Paperlit.events.ppOnResume = true;
        Paperlit.events.ppOnPause = false;
    }
    Paperlit.log.info('Paperlit.onResume');
    Paperlit.resumeActivity();
};

Paperlit.onPause = function(){
	if(typeof Paperlit.events['ppOnPause'] === 'undefined'){
        Paperlit.events.ppOnPause = true;
        Paperlit.events.ppOnResume = false;
    }
    Paperlit.log.info('Paperlit.onPause');
    Paperlit.pauseActivity();
};

Paperlit.onResourceUpdated = function(resourceUrl, resourceCachedUrl){
    Paperlit.log.info('Paperlit.onResourceUpdated');
    var evObj = document.createEvent('Events');
    evObj.initEvent('ppOnResourceUpdated', true, true);
    evObj.ppParameters = {resourceUrl: resourceUrl, resourceCachedUrl: resourceCachedUrl};
    document.dispatchEvent(evObj);
};

Paperlit.onIssueDeleted = function(publicationId, issueId){
    Paperlit.log.info('Paperlit.onIssueDeleted');
    var evObj = document.createEvent('Events');
    evObj.initEvent('ppOnIssueDeleted', true, true);
    evObj.ppParameters = {publicationId: publicationId, issueId: issueId};
    document.dispatchEvent(evObj);
};

/* Notify the panel the its content has been updated and is available at the location "path" */
Paperlit.onPanelContentUpdate = function(path, parameters){
    
    var evObj = document.createEvent('Events');
    evObj.initEvent('ppOnPanelContentUpdate', true, true);
    evObj.ppParameters = {};
   
    if(typeof parameters !== 'undefined' && parameters != null){
        evObj.ppParameters = parameters;
    }
    // add path
    evObj.ppParameters['path'] = path;
    // store this event for later dispatching
    Paperlit._lastOnPanelContentUpdateEvent = evObj;
    Paperlit.log.info('Paperlit.onPanelContentUpdate ' + path + ' parameters ' + JSON.stringify(parameters));
    document.dispatchEvent(evObj);
};

Paperlit.onConfigurationUpdated = function(){
	if(Paperlit.isNative){
	    Paperlit.log.info('Paperlit.onConfigurationUpdated: received configuration update notification from client');
	    // force configuration reload
		Paperlit.getAppConfiguration(Paperlit.noop, Paperlit.noop, true);
	}
	else{
		Paperlit.log.error('Paperlit.onConfigurationUpdated: not supported in browser');
	}
};

Paperlit.onAdvReady = function(){
	Paperlit.log.info('Paperlit.onAdvReady');
    var evObj = document.createEvent('Events');
    evObj.initEvent('ppOnAdvReady', true, true);
    document.dispatchEvent(evObj);
};

/* Attach listener to client event */
Paperlit.addWebViewEventListener = function(eventName, handler, onlyOnce){
	onlyOnce = typeof onlyOnce === 'undefined' ? false : onlyOnce;
    Paperlit.log.info('Paperlit.addWebViewEventListener ' + eventName);
    var hasExecuted = false;
    if(eventName === 'ppOnPanelContentUpdate' && typeof Paperlit._lastOnPanelContentUpdateEvent !== 'undefined'){
    	Paperlit.log.info('Paperlit.addWebViewEventListener ' + eventName + ' event already triggered, dispatch event');
    	// dispatch last event immediately
    	handler(Paperlit._lastOnPanelContentUpdateEvent);
    }
    else if(typeof Paperlit.events[eventName] !== 'undefined' && Paperlit.events[eventName] === true){
        //Paperlit.alert(eventName + ' already triggered')
        Paperlit.log.info('Paperlit.addWebViewEventListener ' + eventName + ' event already triggered, call handler');
        // event already triggered, call handler
        try{
        	hasExecuted = true;
            handler();
        }
        catch(e){}
    }
    else{
        Paperlit.log.info('Paperlit.addWebViewEventListener ' + eventName + ' register handler');
    }

    if(onlyOnce){
    	if(hasExecuted){
    		return;
    	}
	    document.addEventListener(eventName, function(e) {
	        // remove event
	        e.target.removeEventListener(e.type, arguments.callee);
	        // call handler
	        handler(e);
	    });
	}
	else{
		document.addEventListener(eventName, handler);
	}
};


// KEEP AS LAST init 
Paperlit.init();

// init adobe bridge
Paperlit.initAdobeBridge();     
